@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.__main__
     ~~~~~~~~~~~~~~~
 
     The Sphinx documentation toolchain.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,4 +12,4 @@ import sys
 
 from sphinx.cmd.build import main
 
-sys.exit(main(sys.argv[1:]))  # type: ignore
+sys.exit(main(sys.argv[1:]))
