@@ -1,36 +1,37 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders
     ~~~~~~~~~~~~~~~
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+import pickle
 import time
-import warnings
 from os import path
+from typing import Any, Dict, Iterable, List, Sequence, Set, Tuple, Union
 
 from docutils import nodes
-from six.moves import cPickle as pickle
+from docutils.nodes import Node
 
-from sphinx.deprecation import RemovedInSphinx20Warning
-from sphinx.environment import CONFIG_OK, CONFIG_CHANGED_REASON
+from sphinx.config import Config
+from sphinx.environment import BuildEnvironment, CONFIG_OK, CONFIG_CHANGED_REASON
 from sphinx.environment.adapters.asset import ImageAdapter
 from sphinx.errors import SphinxError
+from sphinx.events import EventManager
 from sphinx.io import read_doc
 from sphinx.locale import __
-from sphinx.util import i18n, import_object, logging, rst, status_iterator
+from sphinx.util import import_object, logging, rst, progress_message, status_iterator
 from sphinx.util.build_phase import BuildPhase
 from sphinx.util.console import bold  # type: ignore
 from sphinx.util.docutils import sphinx_domains
-from sphinx.util.i18n import find_catalog
-from sphinx.util.matching import Matcher
+from sphinx.util.i18n import CatalogInfo, CatalogRepository, docname_to_domain
 from sphinx.util.osutil import SEP, ensuredir, relative_uri, relpath
 from sphinx.util.parallel import ParallelTasks, SerialTasks, make_chunks, \
     parallel_available
+from sphinx.util.tags import Tags
 
 # side effect: registers roles and directives
 from sphinx import roles       # noqa
@@ -43,36 +44,32 @@ except ImportError:
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, Iterable, List, Sequence, Set, Tuple, Union  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.config import Config  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
-    from sphinx.util.i18n import CatalogInfo  # NOQA
-    from sphinx.util.tags import Tags  # NOQA
+    from typing import Type  # for python3.5.1
+    from sphinx.application import Sphinx
 
 
 logger = logging.getLogger(__name__)
 
 
-class Builder(object):
+class Builder:
     """
     Builds target formats from the reST sources.
     """
 
     #: The builder's name, for the -b command line option.
-    name = ''  # type: unicode
+    name = ''
     #: The builder's output format, or '' if no document output is produced.
-    format = ''  # type: unicode
+    format = ''
     #: The message emitted upon successful build completion. This can be a
     #: printf-style template string with the following keys: ``outdir``,
     #: ``project``
-    epilog = ''  # type: unicode
+    epilog = ''
 
     #: default translator class for the builder.  This can be overridden by
     #: :py:meth:`app.set_translator()`.
-    default_translator_class = None  # type: nodes.NodeVisitor
+    default_translator_class = None  # type: Type[nodes.NodeVisitor]
     # doctree versioning method
-    versioning_method = 'none'  # type: unicode
+    versioning_method = 'none'
     versioning_compare = False
     # allow parallel write_doc() calls
     allow_parallel = False
@@ -81,14 +78,13 @@ class Builder(object):
 
     #: The list of MIME types of image formats supported by the builder.
     #: Image files are searched in the order in which they appear here.
-    supported_image_types = []  # type: List[unicode]
+    supported_image_types = []  # type: List[str]
     #: The builder supports remote images or not.
     supported_remote_images = False
     #: The builder supports data URIs or not.
     supported_data_uri_images = False
 
-    def __init__(self, app):
-        # type: (Sphinx) -> None
+    def __init__(self, app: "Sphinx") -> None:
         self.srcdir = app.srcdir
         self.confdir = app.confdir
         self.outdir = app.outdir
@@ -97,8 +93,7 @@ class Builder(object):
 
         self.app = app              # type: Sphinx
         self.env = None             # type: BuildEnvironment
-        self.warn = app.warn        # type: Callable
-        self.info = app.info        # type: Callable
+        self.events = app.events    # type: EventManager
         self.config = app.config    # type: Config
         self.tags = app.tags        # type: Tags
         self.tags.add(self.format)
@@ -107,30 +102,27 @@ class Builder(object):
         self.tags.add("builder_%s" % self.name)
 
         # images that need to be copied over (source -> dest)
-        self.images = {}  # type: Dict[unicode, unicode]
+        self.images = {}  # type: Dict[str, str]
         # basename of images directory
         self.imagedir = ""
         # relative path to image directory from current docname (used at writing docs)
-        self.imgpath = ""  # type: unicode
+        self.imgpath = ""
 
         # these get set later
         self.parallel_ok = False
         self.finish_tasks = None  # type: Any
 
-    def set_environment(self, env):
-        # type: (BuildEnvironment) -> None
+    def set_environment(self, env: BuildEnvironment) -> None:
         """Store BuildEnvironment object."""
         self.env = env
         self.env.set_versioning_method(self.versioning_method,
                                        self.versioning_compare)
 
-    def get_translator_class(self, *args):
-        # type: (Any) -> nodes.NodeVisitor
+    def get_translator_class(self, *args: Any) -> "Type[nodes.NodeVisitor]":
         """Return a class of translator."""
         return self.app.registry.get_translator_class(self)
 
-    def create_translator(self, *args):
-        # type: (Any) -> nodes.NodeVisitor
+    def create_translator(self, *args: Any) -> nodes.NodeVisitor:
         """Return an instance of translator.
 
         This method returns an instance of ``default_translator_class`` by default.
@@ -138,32 +130,14 @@ class Builder(object):
         """
         return self.app.registry.create_translator(self, *args)
 
-    @property
-    def translator_class(self):
-        # type: () -> Callable[[Any], nodes.NodeVisitor]
-        """Return a class of translator.
-
-        .. deprecated:: 1.6
-        """
-        translator_class = self.app.registry.get_translator_class(self)
-        if translator_class is None and self.default_translator_class is None:
-            warnings.warn('builder.translator_class() is now deprecated. '
-                          'Please use builder.create_translator() and '
-                          'builder.default_translator_class instead.',
-                          RemovedInSphinx20Warning, stacklevel=2)
-            return None
-        return self.create_translator
-
     # helper methods
-    def init(self):
-        # type: () -> None
+    def init(self) -> None:
         """Load necessary templates and perform initialization.  The default
         implementation does nothing.
         """
         pass
 
-    def create_template_bridge(self):
-        # type: () -> None
+    def create_template_bridge(self) -> None:
         """Return the template bridge configured."""
         if self.config.template_bridge:
             self.templates = import_object(self.config.template_bridge,
@@ -172,8 +146,7 @@ class Builder(object):
             from sphinx.jinja2glue import BuiltinTemplateLoader
             self.templates = BuiltinTemplateLoader()
 
-    def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+    def get_target_uri(self, docname: str, typ: str = None) -> str:
         """Return the target URI for a document name.
 
         *typ* can be used to qualify the link characteristic for individual
@@ -181,8 +154,7 @@ class Builder(object):
         """
         raise NotImplementedError
 
-    def get_relative_uri(self, from_, to, typ=None):
-        # type: (unicode, unicode, unicode) -> unicode
+    def get_relative_uri(self, from_: str, to: str, typ: str = None) -> str:
         """Return a relative URI between two source filenames.
 
         May raise environment.NoUri if there's no way to return a sensible URI.
@@ -190,8 +162,7 @@ class Builder(object):
         return relative_uri(self.get_target_uri(from_),
                             self.get_target_uri(to, typ))
 
-    def get_outdated_docs(self):
-        # type: () -> Union[unicode, Iterable[unicode]]
+    def get_outdated_docs(self) -> Union[str, Iterable[str]]:
         """Return an iterable of output files that are outdated, or a string
         describing what an update build will build.
 
@@ -201,13 +172,11 @@ class Builder(object):
         """
         raise NotImplementedError
 
-    def get_asset_paths(self):
-        # type: () -> List[unicode]
+    def get_asset_paths(self) -> List[str]:
         """Return list of paths for assets (ex. templates, CSS, etc.)."""
         return []
 
-    def post_process_images(self, doctree):
-        # type: (nodes.Node) -> None
+    def post_process_images(self, doctree: Node) -> None:
         """Pick the best candidate for all image URIs."""
         images = ImageAdapter(self.env)
         for node in doctree.traverse(nodes.image):
@@ -240,13 +209,11 @@ class Builder(object):
 
     # compile po methods
 
-    def compile_catalogs(self, catalogs, message):
-        # type: (Set[CatalogInfo], unicode) -> None
+    def compile_catalogs(self, catalogs: Set[CatalogInfo], message: str) -> None:
         if not self.config.gettext_auto_build:
             return
 
-        def cat2relpath(cat):
-            # type: (CatalogInfo) -> unicode
+        def cat2relpath(cat: CatalogInfo) -> str:
             return relpath(cat.mo_path, self.env.srcdir).replace(path.sep, SEP)
 
         logger.info(bold(__('building [mo]: ')) + message)
@@ -255,63 +222,50 @@ class Builder(object):
                                        stringify_func=cat2relpath):
             catalog.write_mo(self.config.language)
 
-    def compile_all_catalogs(self):
-        # type: () -> None
-        catalogs = i18n.find_catalog_source_files(
-            [path.join(self.srcdir, x) for x in self.config.locale_dirs],
-            self.config.language,
-            charset=self.config.source_encoding,
-            force_all=True,
-            excluded=Matcher(['**/.?**']))
-        message = __('all of %d po files') % len(catalogs)
-        self.compile_catalogs(catalogs, message)
+    def compile_all_catalogs(self) -> None:
+        repo = CatalogRepository(self.srcdir, self.config.locale_dirs,
+                                 self.config.language, self.config.source_encoding)
+        message = __('all of %d po files') % len(list(repo.catalogs))
+        self.compile_catalogs(set(repo.catalogs), message)
 
-    def compile_specific_catalogs(self, specified_files):
-        # type: (List[unicode]) -> None
-        def to_domain(fpath):
-            # type: (unicode) -> unicode
+    def compile_specific_catalogs(self, specified_files: List[str]) -> None:
+        def to_domain(fpath: str) -> str:
             docname = self.env.path2doc(path.abspath(fpath))
             if docname:
-                return find_catalog(docname, self.config.gettext_compact)
+                return docname_to_domain(docname, self.config.gettext_compact)
             else:
                 return None
 
-        specified_domains = set(map(to_domain, specified_files))
-        specified_domains.discard(None)
-        catalogs = i18n.find_catalog_source_files(
-            [path.join(self.srcdir, x) for x in self.config.locale_dirs],
-            self.config.language,
-            domains=list(specified_domains),
-            charset=self.config.source_encoding,
-            excluded=Matcher(['**/.?**']))
+        catalogs = set()
+        domains = set(map(to_domain, specified_files))
+        repo = CatalogRepository(self.srcdir, self.config.locale_dirs,
+                                 self.config.language, self.config.source_encoding)
+        for catalog in repo.catalogs:
+            if catalog.domain in domains and catalog.is_outdated():
+                catalogs.add(catalog)
         message = __('targets for %d po files that are specified') % len(catalogs)
         self.compile_catalogs(catalogs, message)
 
-    def compile_update_catalogs(self):
-        # type: () -> None
-        catalogs = i18n.find_catalog_source_files(
-            [path.join(self.srcdir, x) for x in self.config.locale_dirs],
-            self.config.language,
-            charset=self.config.source_encoding,
-            excluded=Matcher(['**/.?**']))
+    def compile_update_catalogs(self) -> None:
+        repo = CatalogRepository(self.srcdir, self.config.locale_dirs,
+                                 self.config.language, self.config.source_encoding)
+        catalogs = {c for c in repo.catalogs if c.is_outdated()}
         message = __('targets for %d po files that are out of date') % len(catalogs)
         self.compile_catalogs(catalogs, message)
 
     # build methods
 
-    def build_all(self):
-        # type: () -> None
+    def build_all(self) -> None:
         """Build all source files."""
         self.build(None, summary=__('all source files'), method='all')
 
-    def build_specific(self, filenames):
-        # type: (List[unicode]) -> None
+    def build_specific(self, filenames: List[str]) -> None:
         """Only rebuild as much as needed for changes in the *filenames*."""
         # bring the filenames to the canonical format, that is,
         # relative to the source directory and without source_suffix.
         dirlen = len(self.srcdir) + 1
         to_write = []
-        suffixes = None  # type: Tuple[unicode]
+        suffixes = None  # type: Tuple[str]
         suffixes = tuple(self.config.source_suffix)  # type: ignore
         for filename in filenames:
             filename = path.normpath(path.abspath(filename))
@@ -319,8 +273,7 @@ class Builder(object):
                 logger.warning(__('file %r given on command line is not under the '
                                   'source directory, ignoring'), filename)
                 continue
-            if not (path.isfile(filename) or
-                    any(path.isfile(filename + suffix) for suffix in suffixes)):
+            if not path.isfile(filename):
                 logger.warning(__('file %r given on command line does not exist, '
                                   'ignoring'), filename)
                 continue
@@ -334,8 +287,7 @@ class Builder(object):
         self.build(to_write, method='specific',
                    summary=__('%d source files given on command line') % len(to_write))
 
-    def build_update(self):
-        # type: () -> None
+    def build_update(self) -> None:
         """Only rebuild what was changed or added since last build."""
         to_build = self.get_outdated_docs()
         if isinstance(to_build, str):
@@ -346,21 +298,20 @@ class Builder(object):
                        summary=__('targets for %d source files that are out of date') %
                        len(to_build))
 
-    def build(self, docnames, summary=None, method='update'):
-        # type: (Iterable[unicode], unicode, unicode) -> None
+    def build(self, docnames: Iterable[str], summary: str = None, method: str = 'update') -> None:  # NOQA
         """Main build method.
 
         First updates the environment, and then calls :meth:`write`.
         """
         if summary:
-            logger.info(bold(__('building [%s]') % self.name) + ': ' + summary)
+            logger.info(bold(__('building [%s]: ') % self.name) + summary)
 
         # while reading, collect all warnings from docutils
         with logging.pending_warnings():
             updated_docnames = set(self.read())
 
         doccount = len(updated_docnames)
-        logger.info(bold(__('looking for now-outdated files... ')), nonl=1)
+        logger.info(bold(__('looking for now-outdated files... ')), nonl=True)
         for docname in self.env.check_dependents(self.app, updated_docnames):
             updated_docnames.add(docname)
         outdated = len(updated_docnames) - doccount
@@ -372,16 +323,14 @@ class Builder(object):
         if updated_docnames:
             # save the environment
             from sphinx.application import ENV_PICKLE_FILENAME
-            logger.info(bold(__('pickling environment... ')), nonl=True)
-            with open(path.join(self.doctreedir, ENV_PICKLE_FILENAME), 'wb') as f:
-                pickle.dump(self.env, f, pickle.HIGHEST_PROTOCOL)
-            logger.info(__('done'))
+            with progress_message(__('pickling environment')):
+                with open(path.join(self.doctreedir, ENV_PICKLE_FILENAME), 'wb') as f:
+                    pickle.dump(self.env, f, pickle.HIGHEST_PROTOCOL)
 
             # global actions
             self.app.phase = BuildPhase.CONSISTENCY_CHECK
-            logger.info(bold(__('checking consistency... ')), nonl=True)
-            self.env.check_consistency()
-            logger.info(__('done'))
+            with progress_message(__('checking consistency')):
+                self.env.check_consistency()
         else:
             if method == 'update' and not docnames:
                 logger.info(bold(__('no targets are out of date.')))
@@ -417,21 +366,20 @@ class Builder(object):
         # wait for all tasks
         self.finish_tasks.join()
 
-    def read(self):
-        # type: () -> List[unicode]
+    def read(self) -> List[str]:
         """(Re-)read all files new or changed since last update.
 
         Store all environment docnames in the canonical format (ie using SEP as
         a separator in place of os.path.sep).
         """
-        logger.info(bold('updating environment: '), nonl=True)
+        logger.info(bold(__('updating environment: ')), nonl=True)
 
         self.env.find_files(self.config, self)
         updated = (self.env.config_status != CONFIG_OK)
         added, changed, removed = self.env.get_outdated_files(updated)
 
         # allow user intervention as well
-        for docs in self.app.emit('env-get-outdated', self, added, changed, removed):
+        for docs in self.events.emit('env-get-outdated', self, added, changed, removed):
             changed.update(set(docs) & self.env.found_docs)
 
         # if files were added or removed, all documents with globbed toctrees
@@ -440,21 +388,23 @@ class Builder(object):
             # ... but not those that already were removed
             changed.update(self.env.glob_toctrees & self.env.found_docs)
 
-        if changed:
-            reason = CONFIG_CHANGED_REASON.get(self.env.config_status, '')
+        if updated:  # explain the change iff build config status was not ok
+            reason = (CONFIG_CHANGED_REASON.get(self.env.config_status, '') +
+                      (self.env.config_status_extra or ''))
             logger.info('[%s] ', reason, nonl=True)
-        logger.info('%s added, %s changed, %s removed',
+
+        logger.info(__('%s added, %s changed, %s removed'),
                     len(added), len(changed), len(removed))
 
         # clear all files no longer present
         for docname in removed:
-            self.app.emit('env-purge-doc', self.env, docname)
+            self.events.emit('env-purge-doc', self.env, docname)
             self.env.clear_doc(docname)
 
         # read all new and changed files
         docnames = sorted(added | changed)
         # allow changing and reordering the list of docs to read
-        self.app.emit('env-before-read-docs', self.env, docnames)
+        self.events.emit('env-before-read-docs', self.env, docnames)
 
         # check if we should do parallel or serial read
         if parallel_available and len(docnames) > 5 and self.app.parallel > 1:
@@ -471,7 +421,7 @@ class Builder(object):
             raise SphinxError('master file %s not found' %
                               self.env.doc2path(self.config.master_doc))
 
-        for retval in self.app.emit('env-updated', self.env):
+        for retval in self.events.emit('env-updated', self.env):
             if retval is not None:
                 docnames.extend(retval)
 
@@ -480,48 +430,43 @@ class Builder(object):
 
         return sorted(docnames)
 
-    def _read_serial(self, docnames):
-        # type: (List[unicode]) -> None
-        for docname in status_iterator(docnames, 'reading sources... ', "purple",
+    def _read_serial(self, docnames: List[str]) -> None:
+        for docname in status_iterator(docnames, __('reading sources... '), "purple",
                                        len(docnames), self.app.verbosity):
             # remove all inventory entries for that file
-            self.app.emit('env-purge-doc', self.env, docname)
+            self.events.emit('env-purge-doc', self.env, docname)
             self.env.clear_doc(docname)
             self.read_doc(docname)
 
-    def _read_parallel(self, docnames, nproc):
-        # type: (List[unicode], int) -> None
+    def _read_parallel(self, docnames: List[str], nproc: int) -> None:
         # clear all outdated docs at once
         for docname in docnames:
-            self.app.emit('env-purge-doc', self.env, docname)
+            self.events.emit('env-purge-doc', self.env, docname)
             self.env.clear_doc(docname)
 
-        def read_process(docs):
-            # type: (List[unicode]) -> bytes
+        def read_process(docs: List[str]) -> bytes:
             self.env.app = self.app
             for docname in docs:
                 self.read_doc(docname)
             # allow pickling self to send it back
             return pickle.dumps(self.env, pickle.HIGHEST_PROTOCOL)
 
-        def merge(docs, otherenv):
-            # type: (List[unicode], bytes) -> None
+        def merge(docs: List[str], otherenv: bytes) -> None:
             env = pickle.loads(otherenv)
             self.env.merge_info_from(docs, env, self.app)
 
         tasks = ParallelTasks(nproc)
         chunks = make_chunks(docnames, nproc)
 
-        for chunk in status_iterator(chunks, 'reading sources... ', "purple",
+        for chunk in status_iterator(chunks, __('reading sources... '), "purple",
                                      len(chunks), self.app.verbosity):
             tasks.add_task(read_process, chunk, merge)
 
         # make sure all threads have finished
-        logger.info(bold('waiting for workers...'))
+        logger.info(bold(__('waiting for workers...')))
         tasks.join()
 
-    def read_doc(self, docname):
-        # type: (unicode) -> None
+    def read_doc(self, docname: str) -> None:
         """Parse a file and add/update inventory entries for the doctree."""
         self.env.prepare_settings(docname)
 
@@ -546,8 +491,7 @@ class Builder(object):
 
         self.write_doctree(docname, doctree)
 
-    def write_doctree(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
+    def write_doctree(self, docname: str, doctree: nodes.document) -> None:
         """Write the doctree to a file."""
         # make it picklable
         doctree.reporter = None
@@ -556,13 +500,12 @@ class Builder(object):
         doctree.settings.env = None
         doctree.settings.record_dependencies = None
 
-        doctree_filename = self.env.doc2path(docname, self.env.doctreedir, '.doctree')
+        doctree_filename = path.join(self.doctreedir, docname + '.doctree')
         ensuredir(path.dirname(doctree_filename))
         with open(doctree_filename, 'wb') as f:
             pickle.dump(doctree, f, pickle.HIGHEST_PROTOCOL)
 
-    def write(self, build_docnames, updated_docnames, method='update'):
-        # type: (Iterable[unicode], Sequence[unicode], unicode) -> None
+    def write(self, build_docnames: Iterable[str], updated_docnames: Sequence[str], method: str = 'update') -> None:  # NOQA
         if build_docnames is None or build_docnames == ['__all__']:
             # build_all
             build_docnames = self.env.found_docs
@@ -580,9 +523,8 @@ class Builder(object):
                     docnames.add(tocdocname)
         docnames.add(self.config.master_doc)
 
-        logger.info(bold(__('preparing documents... ')), nonl=True)
-        self.prepare_writing(docnames)
-        logger.info(__('done'))
+        with progress_message(__('preparing documents')):
+            self.prepare_writing(docnames)
 
         if self.parallel_ok:
             # number of subprocesses is parallel-1 because the main process
@@ -592,8 +534,7 @@ class Builder(object):
         else:
             self._write_serial(sorted(docnames))
 
-    def _write_serial(self, docnames):
-        # type: (Sequence[unicode]) -> None
+    def _write_serial(self, docnames: Sequence[str]) -> None:
         with logging.pending_warnings():
             for docname in status_iterator(docnames, __('writing output... '), "darkgreen",
                                            len(docnames), self.app.verbosity):
@@ -603,10 +544,8 @@ class Builder(object):
                 self.write_doc_serialized(docname, doctree)
                 self.write_doc(docname, doctree)
 
-    def _write_parallel(self, docnames, nproc):
-        # type: (Sequence[unicode], int) -> None
-        def write_process(docs):
-            # type: (List[Tuple[unicode, nodes.Node]]) -> None
+    def _write_parallel(self, docnames: Sequence[str], nproc: int) -> None:
+        def write_process(docs: List[Tuple[str, nodes.document]]) -> None:
             self.app.phase = BuildPhase.WRITING
             for docname, doctree in docs:
                 self.write_doc(docname, doctree)
@@ -636,41 +575,35 @@ class Builder(object):
         logger.info(bold(__('waiting for workers...')))
         tasks.join()
 
-    def prepare_writing(self, docnames):
-        # type: (Set[unicode]) -> None
+    def prepare_writing(self, docnames: Set[str]) -> None:
         """A place where you can add logic before :meth:`write_doc` is run"""
         raise NotImplementedError
 
-    def write_doc(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
+    def write_doc(self, docname: str, doctree: nodes.document) -> None:
         """Where you actually write something to the filesystem."""
         raise NotImplementedError
 
-    def write_doc_serialized(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
+    def write_doc_serialized(self, docname: str, doctree: nodes.document) -> None:
         """Handle parts of write_doc that must be called in the main process
         if parallel build is active.
         """
         pass
 
-    def finish(self):
-        # type: () -> None
+    def finish(self) -> None:
         """Finish the building process.
 
         The default implementation does nothing.
         """
         pass
 
-    def cleanup(self):
-        # type: () -> None
+    def cleanup(self) -> None:
         """Cleanup any resources.
 
         The default implementation does nothing.
         """
         pass
 
-    def get_builder_config(self, option, default):
-        # type: (unicode, unicode) -> Any
+    def get_builder_config(self, option: str, default: str) -> Any:
         """Return a builder specific option.
 
         This method allows customization of common builder settings by
