@@ -1,43 +1,42 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders.gettext
     ~~~~~~~~~~~~~~~~~~~~~~~
 
     The MessageCatalogBuilder class.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from __future__ import unicode_literals
-
 from codecs import open
 from collections import defaultdict, OrderedDict
 from datetime import datetime, tzinfo, timedelta
 from os import path, walk, getenv
 from time import time
+from typing import Any, Dict, Iterable, Generator, List, Set, Tuple, Union
 from uuid import uuid4
 
-from six import iteritems, StringIO
+from docutils import nodes
+from docutils.nodes import Element
 
+from sphinx import addnodes
+from sphinx import package_dir
+from sphinx.application import Sphinx
 from sphinx.builders import Builder
 from sphinx.domains.python import pairindextypes
 from sphinx.errors import ThemeError
 from sphinx.locale import __
 from sphinx.util import split_index_msg, logging, status_iterator
 from sphinx.util.console import bold  # type: ignore
-from sphinx.util.i18n import find_catalog
+from sphinx.util.i18n import CatalogInfo, docname_to_domain
 from sphinx.util.nodes import extract_messages, traverse_translatable_index
-from sphinx.util.osutil import relpath, ensuredir, canon_path
+from sphinx.util.osutil import ensuredir, canon_path, relpath
 from sphinx.util.tags import Tags
+from sphinx.util.template import SphinxRenderer
 
 if False:
     # For type annotation
-    from typing import Any, DefaultDict, Dict, Iterable, List, Set, Tuple  # NOQA
-    from docutils import nodes  # NOQA
-    from sphinx.util.i18n import CatalogInfo  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
+    from typing import DefaultDict  # for python3.5.1
 
 logger = logging.getLogger(__name__)
 
@@ -60,21 +59,27 @@ msgstr ""
 "Content-Type: text/plain; charset=UTF-8\n"
 "Content-Transfer-Encoding: 8bit\n"
 
-"""[1:]
+"""[1:]  # RemovedInSphinx40Warning
 
 
-class Catalog(object):
+class Message:
+    """An entry of translatable message."""
+    def __init__(self, text: str, locations: List[Tuple[str, int]], uuids: List[str]):
+        self.text = text
+        self.locations = locations
+        self.uuids = uuids
+
+
+class Catalog:
     """Catalog of translatable messages."""
 
-    def __init__(self):
-        # type: () -> None
-        self.messages = []  # type: List[unicode]
+    def __init__(self) -> None:
+        self.messages = []  # type: List[str]
                             # retain insertion order, a la OrderedDict
-        self.metadata = OrderedDict()  # type: Dict[unicode, List[Tuple[unicode, int, unicode]]]  # NOQA
+        self.metadata = OrderedDict()   # type: Dict[str, List[Tuple[str, int, str]]]
                                         # msgid -> file, line, uid
 
-    def add(self, msg, origin):
-        # type: (unicode, MsgOrigin) -> None
+    def add(self, msg: str, origin: Union[Element, "MsgOrigin"]) -> None:
         if not hasattr(origin, 'uid'):
             # Nodes that are replicated like todo don't have a uid,
             # however i18n is also unnecessary.
@@ -82,29 +87,57 @@ class Catalog(object):
         if msg not in self.metadata:  # faster lookup in hash
             self.messages.append(msg)
             self.metadata[msg] = []
-        self.metadata[msg].append((origin.source, origin.line, origin.uid))
+        self.metadata[msg].append((origin.source, origin.line, origin.uid))  # type: ignore
+
+    def __iter__(self) -> Generator[Message, None, None]:
+        for message in self.messages:
+            positions = [(source, line) for source, line, uuid in self.metadata[message]]
+            uuids = [uuid for source, line, uuid in self.metadata[message]]
+            yield Message(message, positions, uuids)
 
 
-class MsgOrigin(object):
+class MsgOrigin:
     """
     Origin holder for Catalog message origin.
     """
 
-    def __init__(self, source, line):
-        # type: (unicode, int) -> None
+    def __init__(self, source: str, line: int) -> None:
         self.source = source
         self.line = line
         self.uid = uuid4().hex
 
 
+class GettextRenderer(SphinxRenderer):
+    def __init__(self, template_path: str = None, outdir: str = None) -> None:
+        self.outdir = outdir
+        if template_path is None:
+            template_path = path.join(package_dir, 'templates', 'gettext')
+        super().__init__(template_path)
+
+        def escape(s: str) -> str:
+            s = s.replace('\\', r'\\')
+            s = s.replace('"', r'\"')
+            return s.replace('\n', '\\n"\n"')
+
+        # use texescape as escape filter
+        self.env.filters['e'] = escape
+        self.env.filters['escape'] = escape
+
+    def render(self, filename: str, context: Dict) -> str:
+        def _relpath(s: str) -> str:
+            return canon_path(relpath(s, self.outdir))
+
+        context['relpath'] = _relpath
+        return super().render(filename, context)
+
+
 class I18nTags(Tags):
     """Dummy tags module for I18nBuilder.
 
     To translate all text inside of only nodes, this class
     always returns True value even if no tags are defined.
     """
-    def eval_condition(self, condition):
-        # type: (Any) -> bool
+    def eval_condition(self, condition: Any) -> bool:
         return True
 
 
@@ -118,34 +151,32 @@ class I18nBuilder(Builder):
                                 # be set by `gettext_uuid`
     use_message_catalog = False
 
-    def init(self):
-        # type: () -> None
-        Builder.init(self)
+    def init(self) -> None:
+        super().init()
         self.env.set_versioning_method(self.versioning_method,
                                        self.env.config.gettext_uuid)
         self.tags = I18nTags()
-        self.catalogs = defaultdict(Catalog)  # type: DefaultDict[unicode, Catalog]
+        self.catalogs = defaultdict(Catalog)  # type: DefaultDict[str, Catalog]
 
-    def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+    def get_target_uri(self, docname: str, typ: str = None) -> str:
         return ''
 
-    def get_outdated_docs(self):
-        # type: () -> Set[unicode]
+    def get_outdated_docs(self) -> Set[str]:
         return self.env.found_docs
 
-    def prepare_writing(self, docnames):
-        # type: (Set[unicode]) -> None
+    def prepare_writing(self, docnames: Set[str]) -> None:
         return
 
-    def compile_catalogs(self, catalogs, message):
-        # type: (Set[CatalogInfo], unicode) -> None
+    def compile_catalogs(self, catalogs: Set[CatalogInfo], message: str) -> None:
         return
 
-    def write_doc(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
-        catalog = self.catalogs[find_catalog(docname,
-                                             self.config.gettext_compact)]
+    def write_doc(self, docname: str, doctree: nodes.document) -> None:
+        catalog = self.catalogs[docname_to_domain(docname, self.config.gettext_compact)]
+
+        for toctree in self.env.tocs[docname].traverse(addnodes.toctree):
+            for node, msg in extract_messages(toctree):
+                node.uid = ''  # type: ignore  # Hack UUID model
+                catalog.add(msg, node)
 
         for node, msg in extract_messages(doctree):
             catalog.add(msg, node)
@@ -175,30 +206,25 @@ if source_date_epoch is not None:
 
 
 class LocalTimeZone(tzinfo):
-
-    def __init__(self, *args, **kw):
-        # type: (Any, Any) -> None
-        super(LocalTimeZone, self).__init__(*args, **kw)  # type: ignore
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)  # type: ignore
         self.tzdelta = tzdelta
 
-    def utcoffset(self, dt):
-        # type: (datetime) -> timedelta
+    def utcoffset(self, dt: datetime) -> timedelta:
         return self.tzdelta
 
-    def dst(self, dt):
-        # type: (datetime) -> timedelta
+    def dst(self, dt: datetime) -> timedelta:
         return timedelta(0)
 
 
 ltz = LocalTimeZone()
 
 
-def should_write(filepath, new_content):
-    # type: (unicode, unicode) -> bool
+def should_write(filepath: str, new_content: str) -> bool:
     if not path.exists(filepath):
         return True
     try:
-        with open(filepath, 'r', encoding='utf-8') as oldpot:  # type: ignore
+        with open(filepath, encoding='utf-8') as oldpot:
             old_content = oldpot.read()
             old_header_index = old_content.index('"POT-Creation-Date:')
             new_header_index = new_content.index('"POT-Creation-Date:')
@@ -219,14 +245,12 @@ class MessageCatalogBuilder(I18nBuilder):
     name = 'gettext'
     epilog = __('The message catalogs are in %(outdir)s.')
 
-    def init(self):
-        # type: () -> None
-        I18nBuilder.init(self)
+    def init(self) -> None:
+        super().init()
         self.create_template_bridge()
         self.templates.init(self)
 
-    def _collect_templates(self):
-        # type: () -> Set[unicode]
+    def _collect_templates(self) -> Set[str]:
         template_files = set()
         for template_path in self.config.templates_path:
             tmpl_abs_path = path.join(self.app.srcdir, template_path)
@@ -237,19 +261,18 @@ class MessageCatalogBuilder(I18nBuilder):
                         template_files.add(filename)
         return template_files
 
-    def _extract_from_template(self):
-        # type: () -> None
+    def _extract_from_template(self) -> None:
         files = list(self._collect_templates())
         files.sort()
-        logger.info(bold(__('building [%s]: ') % self.name), nonl=1)
+        logger.info(bold(__('building [%s]: ') % self.name), nonl=True)
         logger.info(__('targets for %d template files'), len(files))
 
         extract_translations = self.templates.environment.extract_translations
 
-        for template in status_iterator(files, __('reading templates... '), "purple",  # type: ignore  # NOQA
+        for template in status_iterator(files, __('reading templates... '), "purple",
                                         len(files), self.app.verbosity):
             try:
-                with open(template, 'r', encoding='utf-8') as f:  # type: ignore
+                with open(template, encoding='utf-8') as f:
                     context = f.read()
                 for line, meth, msg in extract_translations(context):
                     origin = MsgOrigin(template, line)
@@ -257,22 +280,23 @@ class MessageCatalogBuilder(I18nBuilder):
             except Exception as exc:
                 raise ThemeError('%s: %r' % (template, exc))
 
-    def build(self, docnames, summary=None, method='update'):
-        # type: (Iterable[unicode], unicode, unicode) -> None
+    def build(self, docnames: Iterable[str], summary: str = None, method: str = 'update') -> None:  # NOQA
         self._extract_from_template()
-        I18nBuilder.build(self, docnames, summary, method)
+        super().build(docnames, summary, method)
 
-    def finish(self):
-        # type: () -> None
-        I18nBuilder.finish(self)
-        data = dict(
-            version = self.config.version,
-            copyright = self.config.copyright,
-            project = self.config.project,
-            ctime = datetime.fromtimestamp(
-                timestamp, ltz).strftime('%Y-%m-%d %H:%M%z'),
-        )
-        for textdomain, catalog in status_iterator(iteritems(self.catalogs),  # type: ignore
+    def finish(self) -> None:
+        super().finish()
+        context = {
+            'version': self.config.version,
+            'copyright': self.config.copyright,
+            'project': self.config.project,
+            'last_translator': self.config.gettext_last_translator,
+            'language_team': self.config.gettext_language_team,
+            'ctime': datetime.fromtimestamp(timestamp, ltz).strftime('%Y-%m-%d %H:%M%z'),
+            'display_location': self.config.gettext_location,
+            'display_uuid': self.config.gettext_uuid,
+        }
+        for textdomain, catalog in status_iterator(self.catalogs.items(),
                                                    __("writing message catalogs... "),
                                                    "darkgreen", len(self.catalogs),
                                                    self.app.verbosity,
@@ -280,38 +304,16 @@ class MessageCatalogBuilder(I18nBuilder):
             # noop if config.gettext_compact is set
             ensuredir(path.join(self.outdir, path.dirname(textdomain)))
 
+            context['messages'] = list(catalog)
+            content = GettextRenderer(outdir=self.outdir).render('message.pot_t', context)
+
             pofn = path.join(self.outdir, textdomain + '.pot')
-            output = StringIO()
-            output.write(POHEADER % data)  # type: ignore
-
-            for message in catalog.messages:
-                positions = catalog.metadata[message]
-
-                if self.config.gettext_location:
-                    # generate "#: file1:line1\n#: file2:line2 ..."
-                    output.write("#: %s\n" % "\n#: ".join(  # type: ignore
-                        "%s:%s" % (canon_path(relpath(source, self.outdir)), line)
-                        for source, line, _ in positions))
-                if self.config.gettext_uuid:
-                    # generate "# uuid1\n# uuid2\n ..."
-                    output.write("# %s\n" % "\n# ".join(  # type: ignore
-                        uid for _, _, uid in positions))
-
-                # message contains *one* line of text ready for translation
-                message = message.replace('\\', r'\\'). \
-                    replace('"', r'\"'). \
-                    replace('\n', '\\n"\n"')
-                output.write('msgid "%s"\nmsgstr ""\n\n' % message)  # type: ignore
-
-            content = output.getvalue()
-
             if should_write(pofn, content):
-                with open(pofn, 'w', encoding='utf-8') as pofile:  # type: ignore
+                with open(pofn, 'w', encoding='utf-8') as pofile:
                     pofile.write(content)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_builder(MessageCatalogBuilder)
 
     app.add_config_value('gettext_compact', True, 'gettext')
@@ -319,6 +321,8 @@ def setup(app):
     app.add_config_value('gettext_uuid', False, 'gettext')
     app.add_config_value('gettext_auto_build', True, 'env')
     app.add_config_value('gettext_additional_targets', [], 'env')
+    app.add_config_value('gettext_last_translator', 'FULL NAME <EMAIL@ADDRESS>', 'gettext')
+    app.add_config_value('gettext_language_team', 'LANGUAGE <LL@li.org>', 'gettext')
 
     return {
         'version': 'builtin',
