@@ -1,25 +1,28 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.builders.latex.transforms
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     Transforms for LaTeX builder.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Any, Dict, List, Set, Tuple
+from typing import cast
+
 from docutils import nodes
+from docutils.nodes import Element, Node
 
 from sphinx import addnodes
+from sphinx.application import Sphinx
 from sphinx.builders.latex.nodes import (
     captioned_literal_block, footnotemark, footnotetext, math_reference, thebibliography
 )
+from sphinx.domains.citation import CitationDomain
 from sphinx.transforms import SphinxTransform
-
-if False:
-    # For type annotation
-    from typing import Dict, List, Set, Tuple, Union  # NOQA
+from sphinx.transforms.post_transforms import SphinxPostTransform
+from sphinx.util.nodes import NodeMatcher
 
 URI_SCHEMES = ('mailto:', 'http:', 'https:', 'ftp:')
 
@@ -29,12 +32,13 @@ class FootnoteDocnameUpdater(SphinxTransform):
     default_priority = 700
     TARGET_NODES = (nodes.footnote, nodes.footnote_reference)
 
-    def apply(self):
-        for node in self.document.traverse(lambda n: isinstance(n, self.TARGET_NODES)):
+    def apply(self, **kwargs: Any) -> None:
+        matcher = NodeMatcher(*self.TARGET_NODES)
+        for node in self.document.traverse(matcher):  # type: nodes.Element
             node['docname'] = self.env.docname
 
 
-class ShowUrlsTransform(SphinxTransform):
+class ShowUrlsTransform(SphinxPostTransform):
     """Expand references to inline text or footnotes.
 
     For more information, see :confval:`latex_show_urls`.
@@ -42,27 +46,27 @@ class ShowUrlsTransform(SphinxTransform):
     .. note:: This transform is used for integrated doctree
     """
     default_priority = 400
+    builders = ('latex',)
 
     # references are expanded to footnotes (or not)
     expanded = False
 
-    def apply(self):
-        # type: () -> None
+    def run(self, **kwargs: Any) -> None:
         try:
             # replace id_prefix temporarily
-            id_prefix = self.document.settings.id_prefix
-            self.document.settings.id_prefix = 'show_urls'
+            settings = self.document.settings  # type: Any
+            id_prefix = settings.id_prefix
+            settings.id_prefix = 'show_urls'
 
             self.expand_show_urls()
             if self.expanded:
                 self.renumber_footnotes()
         finally:
             # restore id_prefix
-            self.document.settings.id_prefix = id_prefix
+            settings.id_prefix = id_prefix
 
-    def expand_show_urls(self):
-        # type: () -> None
-        show_urls = self.document.settings.env.config.latex_show_urls
+    def expand_show_urls(self) -> None:
+        show_urls = self.config.latex_show_urls
         if show_urls is False or show_urls == 'no':
             return
 
@@ -84,8 +88,7 @@ class ShowUrlsTransform(SphinxTransform):
                         textnode = nodes.Text(" (%s)" % uri)
                         node.parent.insert(index + 1, textnode)
 
-    def get_docname_for_node(self, node):
-        # type: (nodes.Node) -> unicode
+    def get_docname_for_node(self, node: Node) -> str:
         while node:
             if isinstance(node, nodes.document):
                 return self.env.path2doc(node['source'])
@@ -96,25 +99,23 @@ class ShowUrlsTransform(SphinxTransform):
 
         return None  # never reached here. only for type hinting
 
-    def create_footnote(self, uri, docname):
-        # type: (unicode, unicode) -> Tuple[nodes.footnote, nodes.footnote_ref]
-        label = nodes.label('', '#')
-        para = nodes.paragraph()
-        para.append(nodes.reference('', nodes.Text(uri), refuri=uri, nolinkurl=True))
-        footnote = nodes.footnote(uri, label, para, auto=1, docname=docname)
+    def create_footnote(self, uri: str, docname: str) -> Tuple[nodes.footnote, nodes.footnote_reference]:  # NOQA
+        reference = nodes.reference('', nodes.Text(uri), refuri=uri, nolinkurl=True)
+        footnote = nodes.footnote(uri, auto=1, docname=docname)
         footnote['names'].append('#')
+        footnote += nodes.label('', '#')
+        footnote += nodes.paragraph('', '', reference)
         self.document.note_autofootnote(footnote)
 
-        label = nodes.Text('#')
-        footnote_ref = nodes.footnote_reference('[#]_', label, auto=1,
+        footnote_ref = nodes.footnote_reference('[#]_', auto=1,
                                                 refid=footnote['ids'][0], docname=docname)
+        footnote_ref += nodes.Text('#')
         self.document.note_autofootnote_ref(footnote_ref)
         footnote.add_backref(footnote_ref['ids'][0])
 
         return footnote, footnote_ref
 
-    def renumber_footnotes(self):
-        # type: () -> None
+    def renumber_footnotes(self) -> None:
         collector = FootnoteCollector(self.document)
         self.document.walkabout(collector)
 
@@ -127,10 +128,10 @@ class ShowUrlsTransform(SphinxTransform):
                     break
 
             # assign new footnote number
-            old_label = footnote[0].astext()
-            footnote[0].replace_self(nodes.label('', str(num)))
+            old_label = cast(nodes.label, footnote[0])
+            old_label.replace_self(nodes.label('', str(num)))
             if old_label in footnote['names']:
-                footnote['names'].remove(old_label)
+                footnote['names'].remove(old_label.astext())
             footnote['names'].append(str(num))
 
             # update footnote_references by new footnote number
@@ -144,35 +145,30 @@ class ShowUrlsTransform(SphinxTransform):
 class FootnoteCollector(nodes.NodeVisitor):
     """Collect footnotes and footnote references on the document"""
 
-    def __init__(self, document):
-        # type: (nodes.document) -> None
+    def __init__(self, document: nodes.document) -> None:
         self.auto_footnotes = []            # type: List[nodes.footnote]
-        self.used_footnote_numbers = set()  # type: Set[unicode]
+        self.used_footnote_numbers = set()  # type: Set[str]
         self.footnote_refs = []             # type: List[nodes.footnote_reference]
-        nodes.NodeVisitor.__init__(self, document)
+        super().__init__(document)
 
-    def unknown_visit(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_visit(self, node: Node) -> None:
         pass
 
-    def unknown_departure(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_departure(self, node: Node) -> None:
         pass
 
-    def visit_footnote(self, node):
-        # type: (nodes.footnote) -> None
+    def visit_footnote(self, node: nodes.footnote) -> None:
         if node.get('auto'):
             self.auto_footnotes.append(node)
         else:
             for name in node['names']:
                 self.used_footnote_numbers.add(name)
 
-    def visit_footnote_reference(self, node):
-        # type: (nodes.footnote_reference) -> None
+    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None:
         self.footnote_refs.append(node)
 
 
-class LaTeXFootnoteTransform(SphinxTransform):
+class LaTeXFootnoteTransform(SphinxPostTransform):
     """Convert footnote definitions and references to appropriate form to LaTeX.
 
     * Replace footnotes on restricted zone (e.g. headings) by footnotemark node.
@@ -340,8 +336,9 @@ class LaTeXFootnoteTransform(SphinxTransform):
     """
 
     default_priority = 600
+    builders = ('latex',)
 
-    def apply(self):
+    def run(self, **kwargs: Any) -> None:
         footnotes = list(self.document.traverse(nodes.footnote))
         for node in footnotes:
             node.parent.remove(node)
@@ -351,30 +348,25 @@ class LaTeXFootnoteTransform(SphinxTransform):
 
 
 class LaTeXFootnoteVisitor(nodes.NodeVisitor):
-    def __init__(self, document, footnotes):
-        # type: (nodes.document, List[nodes.footnote]) -> None
-        self.appeared = set()       # type: Set[Tuple[unicode, nodes.footnote]]
+    def __init__(self, document: nodes.document, footnotes: List[nodes.footnote]) -> None:
+        self.appeared = set()       # type: Set[Tuple[str, str]]
         self.footnotes = footnotes  # type: List[nodes.footnote]
-        self.pendings = []          # type: List[nodes.Node]
-        self.table_footnotes = []   # type: List[nodes.Node]
-        self.restricted = None      # type: nodes.Node
-        nodes.NodeVisitor.__init__(self, document)
+        self.pendings = []          # type: List[nodes.footnote]
+        self.table_footnotes = []   # type: List[nodes.footnote]
+        self.restricted = None      # type: nodes.Element
+        super().__init__(document)
 
-    def unknown_visit(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_visit(self, node: Node) -> None:
         pass
 
-    def unknown_departure(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_departure(self, node: Node) -> None:
         pass
 
-    def restrict(self, node):
-        # type: (nodes.Node) -> None
+    def restrict(self, node: Element) -> None:
         if self.restricted is None:
             self.restricted = node
 
-    def unrestrict(self, node):
-        # type: (nodes.Node) -> None
+    def unrestrict(self, node: Element) -> None:
         if self.restricted == node:
             self.restricted = None
             pos = node.parent.index(node)
@@ -383,37 +375,29 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
                 node.parent.insert(pos + i + 1, fntext)
             self.pendings = []
 
-    def visit_figure(self, node):
-        # type: (nodes.Node) -> None
+    def visit_figure(self, node: nodes.figure) -> None:
         self.restrict(node)
 
-    def depart_figure(self, node):
-        # type: (nodes.Node) -> None
+    def depart_figure(self, node: nodes.figure) -> None:
         self.unrestrict(node)
 
-    def visit_term(self, node):
-        # type: (nodes.Node) -> None
+    def visit_term(self, node: nodes.term) -> None:
         self.restrict(node)
 
-    def depart_term(self, node):
-        # type: (nodes.Node) -> None
+    def depart_term(self, node: nodes.term) -> None:
         self.unrestrict(node)
 
-    def visit_caption(self, node):
-        # type: (nodes.Node) -> None
+    def visit_caption(self, node: nodes.caption) -> None:
         self.restrict(node)
 
-    def depart_caption(self, node):
-        # type: (nodes.Node) -> None
+    def depart_caption(self, node: nodes.caption) -> None:
         self.unrestrict(node)
 
-    def visit_title(self, node):
-        # type: (nodes.Node) -> None
+    def visit_title(self, node: nodes.title) -> None:
         if isinstance(node.parent, (nodes.section, nodes.table)):
             self.restrict(node)
 
-    def depart_title(self, node):
-        # type: (nodes.Node) -> None
+    def depart_title(self, node: nodes.title) -> None:
         if isinstance(node.parent, nodes.section):
             self.unrestrict(node)
         elif isinstance(node.parent, nodes.table):
@@ -421,18 +405,15 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
             self.pendings = []
             self.unrestrict(node)
 
-    def visit_thead(self, node):
-        # type: (nodes.Node) -> None
+    def visit_thead(self, node: nodes.thead) -> None:
         self.restrict(node)
 
-    def depart_thead(self, node):
-        # type: (nodes.Node) -> None
+    def depart_thead(self, node: nodes.thead) -> None:
         self.table_footnotes += self.pendings
         self.pendings = []
         self.unrestrict(node)
 
-    def depart_table(self, node):
-        # type: (nodes.Node) -> None
+    def depart_table(self, node: nodes.table) -> None:
         tbody = list(node.traverse(nodes.tbody))[0]
         for footnote in reversed(self.table_footnotes):
             fntext = footnotetext('', *footnote.children)
@@ -440,16 +421,13 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
 
         self.table_footnotes = []
 
-    def visit_footnote(self, node):
-        # type: (nodes.Node) -> None
+    def visit_footnote(self, node: nodes.footnote) -> None:
         self.restrict(node)
 
-    def depart_footnote(self, node):
-        # type: (nodes.Node) -> None
+    def depart_footnote(self, node: nodes.footnote) -> None:
         self.unrestrict(node)
 
-    def visit_footnote_reference(self, node):
-        # type: (nodes.Node) -> None
+    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None:
         number = node.astext().strip()
         docname = node['docname']
         if self.restricted:
@@ -470,8 +448,7 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
         self.appeared.add((docname, number))
         raise nodes.SkipNode
 
-    def get_footnote_by_reference(self, node):
-        # type: (nodes.Node) -> nodes.Node
+    def get_footnote_by_reference(self, node: nodes.footnote_reference) -> nodes.footnote:
         docname = node['docname']
         for footnote in self.footnotes:
             if docname == footnote['docname'] and footnote['ids'][0] == node['refid']:
@@ -480,7 +457,7 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
         return None
 
 
-class BibliographyTransform(SphinxTransform):
+class BibliographyTransform(SphinxPostTransform):
     """Gather bibliography entries to tail of document.
 
     Before::
@@ -511,9 +488,9 @@ class BibliographyTransform(SphinxTransform):
                     ...
     """
     default_priority = 750
+    builders = ('latex',)
 
-    def apply(self):
-        # type: () -> None
+    def run(self, **kwargs: Any) -> None:
         citations = thebibliography()
         for node in self.document.traverse(nodes.citation):
             node.parent.remove(node)
@@ -523,42 +500,36 @@ class BibliographyTransform(SphinxTransform):
             self.document += citations
 
 
-class CitationReferenceTransform(SphinxTransform):
+class CitationReferenceTransform(SphinxPostTransform):
     """Replace pending_xref nodes for citation by citation_reference.
 
     To handle citation reference easily on LaTeX writer, this converts
     pending_xref nodes to citation_reference.
     """
     default_priority = 5  # before ReferencesResolver
+    builders = ('latex',)
 
-    def apply(self):
-        # type: () -> None
-        if self.app.builder.name != 'latex':
-            return
-
-        citations = self.env.get_domain('std').data['citations']
-        for node in self.document.traverse(addnodes.pending_xref):
-            if node['refdomain'] == 'std' and node['reftype'] == 'citation':
-                docname, labelid, _ = citations.get(node['reftarget'], ('', '', 0))
-                if docname:
-                    citation_ref = nodes.citation_reference('', *node.children,
-                                                            docname=docname, refname=labelid)
-                    node.replace_self(citation_ref)
+    def run(self, **kwargs: Any) -> None:
+        domain = cast(CitationDomain, self.env.get_domain('citation'))
+        matcher = NodeMatcher(addnodes.pending_xref, refdomain='citation', reftype='ref')
+        for node in self.document.traverse(matcher):  # type: addnodes.pending_xref
+            docname, labelid, _ = domain.citations.get(node['reftarget'], ('', '', 0))
+            if docname:
+                citation_ref = nodes.citation_reference('', '', *node.children,
+                                                        docname=docname, refname=labelid)
+                node.replace_self(citation_ref)
 
 
-class MathReferenceTransform(SphinxTransform):
+class MathReferenceTransform(SphinxPostTransform):
     """Replace pending_xref nodes for math by math_reference.
 
     To handle math reference easily on LaTeX writer, this converts pending_xref
     nodes to math_reference.
     """
     default_priority = 5  # before ReferencesResolver
+    builders = ('latex',)
 
-    def apply(self):
-        # type: () -> None
-        if self.app.builder.name != 'latex':
-            return
-
+    def run(self, **kwargs: Any) -> None:
         equations = self.env.get_domain('math').data['objects']
         for node in self.document.traverse(addnodes.pending_xref):
             if node['refdomain'] == 'math' and node['reftype'] in ('eq', 'numref'):
@@ -568,31 +539,80 @@ class MathReferenceTransform(SphinxTransform):
                     node.replace_self(refnode)
 
 
-class LiteralBlockTransform(SphinxTransform):
+class LiteralBlockTransform(SphinxPostTransform):
     """Replace container nodes for literal_block by captioned_literal_block."""
     default_priority = 400
+    builders = ('latex',)
 
-    def apply(self):
-        # type: () -> None
-        if self.app.builder.name != 'latex':
-            return
-
-        for node in self.document.traverse(nodes.container):
-            if node.get('literal_block') is True:
-                newnode = captioned_literal_block('', *node.children, **node.attributes)
-                node.replace_self(newnode)
+    def run(self, **kwargs: Any) -> None:
+        matcher = NodeMatcher(nodes.container, literal_block=True)
+        for node in self.document.traverse(matcher):  # type: nodes.container
+            newnode = captioned_literal_block('', *node.children, **node.attributes)
+            node.replace_self(newnode)
 
 
-class DocumentTargetTransform(SphinxTransform):
+class DocumentTargetTransform(SphinxPostTransform):
     """Add :doc label to the first section of each document."""
     default_priority = 400
+    builders = ('latex',)
 
-    def apply(self):
-        # type: () -> None
-        if self.app.builder.name != 'latex':
-            return
-
+    def run(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.start_of_file):
             section = node.next_node(nodes.section)
             if section:
                 section['ids'].append(':doc')  # special label for :doc:
+
+
+class IndexInSectionTitleTransform(SphinxTransform):
+    """Move index nodes in section title to outside of the title.
+
+    LaTeX index macro is not compatible with some handling of section titles
+    such as uppercasing done on LaTeX side (cf. fncychap handling of ``\\chapter``).
+    Moving the index node to after the title node fixes that.
+
+    Before::
+
+        <section>
+            <title>
+                blah blah <index entries=[...]/>blah
+            <paragraph>
+                blah blah blah
+            ...
+
+    After::
+
+        <section>
+            <title>
+                blah blah blah
+            <index entries=[...]/>
+            <paragraph>
+                blah blah blah
+            ...
+    """
+    default_priority = 400
+
+    def apply(self):
+        for node in self.document.traverse(nodes.title):
+            if isinstance(node.parent, nodes.section):
+                for i, index in enumerate(node.traverse(addnodes.index)):
+                    # move the index node next to the section title
+                    node.remove(index)
+                    node.parent.insert(i + 1, index)
+
+
+def setup(app: Sphinx) -> Dict[str, Any]:
+    app.add_transform(FootnoteDocnameUpdater)
+    app.add_post_transform(BibliographyTransform)
+    app.add_post_transform(CitationReferenceTransform)
+    app.add_post_transform(DocumentTargetTransform)
+    app.add_post_transform(IndexInSectionTitleTransform)
+    app.add_post_transform(LaTeXFootnoteTransform)
+    app.add_post_transform(LiteralBlockTransform)
+    app.add_post_transform(MathReferenceTransform)
+    app.add_post_transform(ShowUrlsTransform)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
