@@ -1,45 +1,46 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.directives.other
     ~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+from typing import Any, Dict, List
+from typing import cast
 
 from docutils import nodes
-from docutils.parsers.rst import Directive, directives
+from docutils.nodes import Element, Node
+from docutils.parsers.rst import directives
 from docutils.parsers.rst.directives.admonitions import BaseAdmonition
 from docutils.parsers.rst.directives.misc import Class
 from docutils.parsers.rst.directives.misc import Include as BaseInclude
-from six.moves import range
 
 from sphinx import addnodes
-from sphinx.locale import versionlabels, _
+from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
+from sphinx.domains.changeset import VersionChange  # NOQA  # for compatibility
+from sphinx.locale import _
 from sphinx.util import url_re, docname_join
-from sphinx.util.matching import patfilter
-from sphinx.util.nodes import explicit_title_re, set_source_info, \
-    process_index_entry
+from sphinx.util.docutils import SphinxDirective
+from sphinx.util.matching import Matcher, patfilter
+from sphinx.util.nodes import explicit_title_re
 
 if False:
     # For type annotation
-    from typing import Any, Dict, Generator, List, Tuple  # NOQA
-    from sphinx.application import Sphinx  # NOQA
+    from sphinx.application import Sphinx
 
 
-glob_re = re.compile('.*[*?\[].*')
+glob_re = re.compile(r'.*[*?\[].*')
 
 
-def int_or_nothing(argument):
-    # type: (unicode) -> int
+def int_or_nothing(argument: str) -> int:
     if not argument:
         return 999
     return int(argument)
 
 
-class TocTree(Directive):
+class TocTree(SphinxDirective):
     """
     Directive to notify Sphinx about the hierarchical structure of the docs,
     and to include a table-of-contents like tree in the current document.
@@ -60,10 +61,9 @@ class TocTree(Directive):
         'reversed': directives.flag,
     }
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         subnode = addnodes.toctree()
-        subnode['parent'] = self.state.document.settings.env.docname
+        subnode['parent'] = self.env.docname
 
         # (title, ref) pairs, where ref may be a document, or an external link,
         # and title may be None if the document's title is to be used
@@ -76,7 +76,7 @@ class TocTree(Directive):
         subnode['includehidden'] = 'includehidden' in self.options
         subnode['numbered'] = self.options.get('numbered', 0)
         subnode['titlesonly'] = 'titlesonly' in self.options
-        set_source_info(self, subnode)
+        self.set_source_info(subnode)
         wrappernode = nodes.compound(classes=['toctree-wrapper'])
         wrappernode.append(subnode)
         self.add_name(wrappernode)
@@ -86,14 +86,14 @@ class TocTree(Directive):
         return ret
 
     def parse_content(self, toctree):
-        env = self.state.document.settings.env
-        suffixes = env.config.source_suffix
+        suffixes = self.config.source_suffix
 
         # glob target documents
-        all_docnames = env.found_docs.copy()
-        all_docnames.remove(env.docname)  # remove current document
+        all_docnames = self.env.found_docs.copy()
+        all_docnames.remove(self.env.docname)  # remove current document
 
         ret = []
+        excluded = Matcher(self.config.exclude_patterns)
         for entry in self.content:
             if not entry:
                 continue
@@ -101,7 +101,7 @@ class TocTree(Directive):
             explicit = explicit_title_re.match(entry)
             if (toctree['glob'] and glob_re.match(entry) and
                     not explicit and not url_re.match(entry)):
-                patname = docname_join(env.docname, entry)
+                patname = docname_join(self.env.docname, entry)
                 docnames = sorted(patfilter(all_docnames, patname))
                 for docname in docnames:
                     all_docnames.remove(docname)  # don't include it again
@@ -125,14 +125,18 @@ class TocTree(Directive):
                         docname = docname[:-len(suffix)]
                         break
                 # absolutize filenames
-                docname = docname_join(env.docname, docname)
+                docname = docname_join(self.env.docname, docname)
                 if url_re.match(ref) or ref == 'self':
                     toctree['entries'].append((title, ref))
-                elif docname not in env.found_docs:
-                    ret.append(self.state.document.reporter.warning(
-                        'toctree contains reference to nonexisting '
-                        'document %r' % docname, line=self.lineno))
-                    env.note_reread()
+                elif docname not in self.env.found_docs:
+                    if excluded(self.env.doc2path(docname, None)):
+                        message = 'toctree contains reference to excluded document %r'
+                    else:
+                        message = 'toctree contains reference to nonexisting document %r'
+
+                    ret.append(self.state.document.reporter.warning(message % docname,
+                                                                    line=self.lineno))
+                    self.env.note_reread()
                 else:
                     all_docnames.discard(docname)
                     toctree['entries'].append((title, docname))
@@ -141,11 +145,12 @@ class TocTree(Directive):
         # entries contains all entries (self references, external links etc.)
         if 'reversed' in self.options:
             toctree['entries'] = list(reversed(toctree['entries']))
+            toctree['includefiles'] = list(reversed(toctree['includefiles']))
 
         return ret
 
 
-class Author(Directive):
+class Author(SphinxDirective):
     """
     Directive to give the name of the author of the current document
     or section. Shown in the output only if the show_authors option is on.
@@ -156,12 +161,10 @@ class Author(Directive):
     final_argument_whitespace = True
     option_spec = {}  # type: Dict
 
-    def run(self):
-        # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
-        if not env.config.show_authors:
+    def run(self) -> List[Node]:
+        if not self.config.show_authors:
             return []
-        para = nodes.paragraph(translatable=False)
+        para = nodes.paragraph(translatable=False)  # type: Element
         emph = nodes.emphasis()
         para += emph
         if self.name == 'sectionauthor':
@@ -173,85 +176,12 @@ class Author(Directive):
         else:
             text = _('Author: ')
         emph += nodes.Text(text, text)
-        inodes, messages = self.state.inline_text(self.arguments[0],
-                                                  self.lineno)
+        inodes, messages = self.state.inline_text(self.arguments[0], self.lineno)
         emph.extend(inodes)
-        return [para] + messages
 
-
-class Index(Directive):
-    """
-    Directive to add entries to the index.
-    """
-    has_content = False
-    required_arguments = 1
-    optional_arguments = 0
-    final_argument_whitespace = True
-    option_spec = {}  # type: Dict
-
-    def run(self):
-        # type: () -> List[nodes.Node]
-        arguments = self.arguments[0].split('\n')
-        env = self.state.document.settings.env
-        targetid = 'index-%s' % env.new_serialno('index')
-        targetnode = nodes.target('', '', ids=[targetid])
-        self.state.document.note_explicit_target(targetnode)
-        indexnode = addnodes.index()
-        indexnode['entries'] = []
-        indexnode['inline'] = False
-        set_source_info(self, indexnode)
-        for entry in arguments:
-            indexnode['entries'].extend(process_index_entry(entry, targetid))
-        return [indexnode, targetnode]
-
-
-class VersionChange(Directive):
-    """
-    Directive to describe a change/addition/deprecation in a specific version.
-    """
-    has_content = True
-    required_arguments = 1
-    optional_arguments = 1
-    final_argument_whitespace = True
-    option_spec = {}  # type: Dict
-
-    def run(self):
-        # type: () -> List[nodes.Node]
-        node = addnodes.versionmodified()
-        node.document = self.state.document
-        set_source_info(self, node)
-        node['type'] = self.name
-        node['version'] = self.arguments[0]
-        text = versionlabels[self.name] % self.arguments[0]
-        if len(self.arguments) == 2:
-            inodes, messages = self.state.inline_text(self.arguments[1],
-                                                      self.lineno + 1)
-            para = nodes.paragraph(self.arguments[1], '', *inodes, translatable=False)
-            set_source_info(self, para)
-            node.append(para)
-        else:
-            messages = []
-        if self.content:
-            self.state.nested_parse(self.content, self.content_offset, node)
-        if len(node):
-            if isinstance(node[0], nodes.paragraph) and node[0].rawsource:
-                content = nodes.inline(node[0].rawsource, translatable=True)
-                content.source = node[0].source
-                content.line = node[0].line
-                content += node[0].children
-                node[0].replace_self(nodes.paragraph('', '', content, translatable=False))
-            node[0].insert(0, nodes.inline('', '%s: ' % text,
-                                           classes=['versionmodified']))
-        else:
-            para = nodes.paragraph('', '',
-                                   nodes.inline('', '%s.' % text,
-                                                classes=['versionmodified']),
-                                   translatable=False)
-            node.append(para)
-        env = self.state.document.settings.env
-        # XXX should record node.source as well
-        env.note_versionchange(node['type'], node['version'], node, node.line)
-        return [node] + messages
+        ret = [para]  # type: List[Node]
+        ret += messages
+        return ret
 
 
 class SeeAlso(BaseAdmonition):
@@ -261,7 +191,7 @@ class SeeAlso(BaseAdmonition):
     node_class = addnodes.seealso
 
 
-class TabularColumns(Directive):
+class TabularColumns(SphinxDirective):
     """
     Directive to give an explicit tabulary column definition to LaTeX.
     """
@@ -271,15 +201,14 @@ class TabularColumns(Directive):
     final_argument_whitespace = True
     option_spec = {}  # type: Dict
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         node = addnodes.tabular_col_spec()
         node['spec'] = self.arguments[0]
-        set_source_info(self, node)
+        self.set_source_info(node)
         return [node]
 
 
-class Centered(Directive):
+class Centered(SphinxDirective):
     """
     Directive to create a centered line of bold text.
     """
@@ -289,18 +218,19 @@ class Centered(Directive):
     final_argument_whitespace = True
     option_spec = {}  # type: Dict
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         if not self.arguments:
             return []
-        subnode = addnodes.centered()
-        inodes, messages = self.state.inline_text(self.arguments[0],
-                                                  self.lineno)
+        subnode = addnodes.centered()  # type: Element
+        inodes, messages = self.state.inline_text(self.arguments[0], self.lineno)
         subnode.extend(inodes)
-        return [subnode] + messages
+
+        ret = [subnode]  # type: List[Node]
+        ret += messages
+        return ret
 
 
-class Acks(Directive):
+class Acks(SphinxDirective):
     """
     Directive for a list of names.
     """
@@ -310,19 +240,18 @@ class Acks(Directive):
     final_argument_whitespace = False
     option_spec = {}  # type: Dict
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         node = addnodes.acks()
         node.document = self.state.document
         self.state.nested_parse(self.content, self.content_offset, node)
         if len(node.children) != 1 or not isinstance(node.children[0],
                                                      nodes.bullet_list):
-            return [self.state.document.reporter.warning(
-                '.. acks content is not a list', line=self.lineno)]
+            reporter = self.state.document.reporter
+            return [reporter.warning('.. acks content is not a list', line=self.lineno)]
         return [node]
 
 
-class HList(Directive):
+class HList(SphinxDirective):
     """
     Directive for a list that gets compacted horizontally.
     """
@@ -334,32 +263,30 @@ class HList(Directive):
         'columns': int,
     }
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         ncolumns = self.options.get('columns', 2)
         node = nodes.paragraph()
         node.document = self.state.document
         self.state.nested_parse(self.content, self.content_offset, node)
         if len(node.children) != 1 or not isinstance(node.children[0],
                                                      nodes.bullet_list):
-            return [self.state.document.reporter.warning(
-                '.. hlist content is not a list', line=self.lineno)]
+            reporter = self.state.document.reporter
+            return [reporter.warning('.. hlist content is not a list', line=self.lineno)]
         fulllist = node.children[0]
         # create a hlist node where the items are distributed
         npercol, nmore = divmod(len(fulllist), ncolumns)
         index = 0
         newnode = addnodes.hlist()
         for column in range(ncolumns):
-            endindex = index + (column < nmore and (npercol + 1) or npercol)
-            col = addnodes.hlistcol()
-            col += nodes.bullet_list()
-            col[0] += fulllist.children[index:endindex]
+            endindex = index + ((npercol + 1) if column < nmore else npercol)
+            bullet_list = nodes.bullet_list()
+            bullet_list += fulllist.children[index:endindex]
+            newnode += addnodes.hlistcol('', bullet_list)
             index = endindex
-            newnode += col
         return [newnode]
 
 
-class Only(Directive):
+class Only(SphinxDirective):
     """
     Directive to only include text if the given tag(s) are enabled.
     """
@@ -369,23 +296,23 @@ class Only(Directive):
     final_argument_whitespace = True
     option_spec = {}  # type: Dict
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         node = addnodes.only()
         node.document = self.state.document
-        set_source_info(self, node)
+        self.set_source_info(node)
         node['expr'] = self.arguments[0]
 
         # Same as util.nested_parse_with_titles but try to handle nested
         # sections which should be raised higher up the doctree.
-        surrounding_title_styles = self.state.memo.title_styles
-        surrounding_section_level = self.state.memo.section_level
-        self.state.memo.title_styles = []
-        self.state.memo.section_level = 0
+        memo = self.state.memo  # type: Any
+        surrounding_title_styles = memo.title_styles
+        surrounding_section_level = memo.section_level
+        memo.title_styles = []
+        memo.section_level = 0
         try:
             self.state.nested_parse(self.content, self.content_offset,
-                                    node, match_titles=1)
-            title_styles = self.state.memo.title_styles
+                                    node, match_titles=True)
+            title_styles = memo.title_styles
             if (not surrounding_title_styles or
                     not title_styles or
                     title_styles[0] not in surrounding_title_styles or
@@ -406,46 +333,49 @@ class Only(Directive):
             # Use these depths to determine where the nested sections should
             # be placed in the doctree.
             n_sects_to_raise = current_depth - nested_depth + 1
-            parent = self.state.parent
+            parent = cast(nodes.Element, self.state.parent)
             for i in range(n_sects_to_raise):
                 if parent.parent:
                     parent = parent.parent
             parent.append(node)
             return []
         finally:
-            self.state.memo.title_styles = surrounding_title_styles
-            self.state.memo.section_level = surrounding_section_level
+            memo.title_styles = surrounding_title_styles
+            memo.section_level = surrounding_section_level
 
 
-class Include(BaseInclude):
+class Include(BaseInclude, SphinxDirective):
     """
     Like the standard "Include" directive, but interprets absolute paths
     "correctly", i.e. relative to source directory.
     """
 
-    def run(self):
-        # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
+    def run(self) -> List[Node]:
         if self.arguments[0].startswith('<') and \
            self.arguments[0].endswith('>'):
             # docutils "standard" includes, do not do path processing
-            return BaseInclude.run(self)
-        rel_filename, filename = env.relfn2path(self.arguments[0])
+            return super().run()
+        rel_filename, filename = self.env.relfn2path(self.arguments[0])
         self.arguments[0] = filename
-        env.note_included(filename)
-        return BaseInclude.run(self)
+        self.env.note_included(filename)
+        return super().run()
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+# Import old modules here for compatibility
+from sphinx.domains.index import IndexDirective  # NOQA
+
+deprecated_alias('sphinx.directives.other',
+                 {
+                     'Index': IndexDirective,
+                 },
+                 RemovedInSphinx40Warning)
+
+
+def setup(app: "Sphinx") -> Dict[str, Any]:
     directives.register_directive('toctree', TocTree)
     directives.register_directive('sectionauthor', Author)
     directives.register_directive('moduleauthor', Author)
     directives.register_directive('codeauthor', Author)
-    directives.register_directive('index', Index)
-    directives.register_directive('deprecated', VersionChange)
-    directives.register_directive('versionadded', VersionChange)
-    directives.register_directive('versionchanged', VersionChange)
     directives.register_directive('seealso', SeeAlso)
     directives.register_directive('tabularcolumns', TabularColumns)
     directives.register_directive('centered', Centered)
