@@ -1,33 +1,37 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.domains.javascript
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
     The JavaScript domain.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Any, Dict, Iterator, List, Tuple
+from typing import cast
+
 from docutils import nodes
-from docutils.parsers.rst import Directive, directives
+from docutils.nodes import Element, Node
+from docutils.parsers.rst import directives
 
 from sphinx import addnodes
+from sphinx.addnodes import desc_signature, pending_xref
+from sphinx.application import Sphinx
+from sphinx.builders import Builder
 from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType
 from sphinx.domains.python import _pseudo_parse_arglist
-from sphinx.locale import l_, _
+from sphinx.environment import BuildEnvironment
+from sphinx.locale import _, __
 from sphinx.roles import XRefRole
+from sphinx.util import logging
 from sphinx.util.docfields import Field, GroupedField, TypedField
+from sphinx.util.docutils import SphinxDirective
 from sphinx.util.nodes import make_refnode
 
-if False:
-    # For type annotation
-    from typing import Any, Dict, Iterator, List, Tuple  # NOQA
-    from docutils import nodes  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+
+logger = logging.getLogger(__name__)
 
 
 class JSObject(ObjectDescription):
@@ -39,14 +43,13 @@ class JSObject(ObjectDescription):
     has_arguments = False
 
     #: what is displayed right before the documentation entry
-    display_prefix = None  # type: unicode
+    display_prefix = None  # type: str
 
     #: If ``allow_nesting`` is ``True``, the object prefixes will be accumulated
     #: based on directive nesting
     allow_nesting = False
 
-    def handle_signature(self, sig, signode):
-        # type: (unicode, addnodes.desc_signature) -> Tuple[unicode, unicode]
+    def handle_signature(self, sig: str, signode: desc_signature) -> Tuple[str, str]:
         """Breaks down construct signatures
 
         Parses out prefix and argument list from construct definition. The
@@ -99,23 +102,19 @@ class JSObject(ObjectDescription):
                 _pseudo_parse_arglist(signode, arglist)
         return fullname, prefix
 
-    def add_target_and_index(self, name_obj, sig, signode):
-        # type: (Tuple[unicode, unicode], unicode, addnodes.desc_signature) -> None
+    def add_target_and_index(self, name_obj: Tuple[str, str], sig: str,
+                             signode: desc_signature) -> None:
         mod_name = self.env.ref_context.get('js:module')
-        fullname = (mod_name and mod_name + '.' or '') + name_obj[0]
+        fullname = (mod_name + '.' if mod_name else '') + name_obj[0]
         if fullname not in self.state.document.ids:
             signode['names'].append(fullname)
             signode['ids'].append(fullname.replace('$', '_S_'))
             signode['first'] = not self.names
             self.state.document.note_explicit_target(signode)
-            objects = self.env.domaindata['js']['objects']
-            if fullname in objects:
-                self.state_machine.reporter.warning(
-                    'duplicate object description of %s, ' % fullname +
-                    'other instance in ' +
-                    self.env.doc2path(objects[fullname][0]),
-                    line=self.lineno)
-            objects[fullname] = self.env.docname, self.objtype
+
+            domain = cast(JavaScriptDomain, self.env.get_domain('js'))
+            domain.note_object(fullname, self.objtype,
+                               location=(self.env.docname, self.lineno))
 
         indextext = self.get_index_text(mod_name, name_obj)
         if indextext:
@@ -123,8 +122,7 @@ class JSObject(ObjectDescription):
                                               fullname.replace('$', '_S_'),
                                               '', None))
 
-    def get_index_text(self, objectname, name_obj):
-        # type: (unicode, Tuple[unicode, unicode]) -> unicode
+    def get_index_text(self, objectname: str, name_obj: Tuple[str, str]) -> str:
         name, obj = name_obj
         if self.objtype == 'function':
             if not obj:
@@ -138,8 +136,7 @@ class JSObject(ObjectDescription):
             return _('%s (%s attribute)') % (name, obj)
         return ''
 
-    def before_content(self):
-        # type: () -> None
+    def before_content(self) -> None:
         """Handle object nesting before content
 
         :py:class:`JSObject` represents JavaScript language constructs. For
@@ -175,8 +172,7 @@ class JSObject(ObjectDescription):
                 objects = self.env.ref_context.setdefault('js:objects', [])
                 objects.append(prefix)
 
-    def after_content(self):
-        # type: () -> None
+    def after_content(self) -> None:
         """Handle object de-nesting after content
 
         If this class is a nestable object, removing the last nested class prefix
@@ -201,15 +197,15 @@ class JSCallable(JSObject):
     has_arguments = True
 
     doc_field_types = [
-        TypedField('arguments', label=l_('Arguments'),
+        TypedField('arguments', label=_('Arguments'),
                    names=('argument', 'arg', 'parameter', 'param'),
                    typerolename='func', typenames=('paramtype', 'type')),
-        GroupedField('errors', label=l_('Throws'), rolename='err',
+        GroupedField('errors', label=_('Throws'), rolename='err',
                      names=('throws', ),
                      can_collapse=True),
-        Field('returnvalue', label=l_('Returns'), has_arg=False,
+        Field('returnvalue', label=_('Returns'), has_arg=False,
               names=('returns', 'return')),
-        Field('returntype', label=l_('Return type'), has_arg=False,
+        Field('returntype', label=_('Return type'), has_arg=False,
               names=('rtype',)),
     ]
 
@@ -220,7 +216,7 @@ class JSConstructor(JSCallable):
     allow_nesting = True
 
 
-class JSModule(Directive):
+class JSModule(SphinxDirective):
     """
     Directive to mark description of a new JavaScript module.
 
@@ -247,18 +243,19 @@ class JSModule(Directive):
         'noindex': directives.flag
     }
 
-    def run(self):
-        # type: () -> List[nodes.Node]
-        env = self.state.document.settings.env
+    def run(self) -> List[Node]:
         mod_name = self.arguments[0].strip()
-        env.ref_context['js:module'] = mod_name
+        self.env.ref_context['js:module'] = mod_name
         noindex = 'noindex' in self.options
-        ret = []
+        ret = []  # type: List[Node]
         if not noindex:
-            env.domaindata['js']['modules'][mod_name] = env.docname
+            domain = cast(JavaScriptDomain, self.env.get_domain('js'))
+
+            domain.note_module(mod_name)
             # Make a duplicate entry in 'objects' to facilitate searching for
             # the module in JavaScriptDomain.find_obj()
-            env.domaindata['js']['objects'][mod_name] = (env.docname, 'module')
+            domain.note_object(mod_name, 'module', location=(self.env.docname, self.lineno))
+
             targetnode = nodes.target('', '', ids=['module-' + mod_name],
                                       ismod=True)
             self.state.document.note_explicit_target(targetnode)
@@ -271,8 +268,8 @@ class JSModule(Directive):
 
 
 class JSXRefRole(XRefRole):
-    def process_link(self, env, refnode, has_explicit_title, title, target):
-        # type: (BuildEnvironment, nodes.Node, bool, unicode, unicode) -> Tuple[unicode, unicode]  # NOQA
+    def process_link(self, env: BuildEnvironment, refnode: Element,
+                     has_explicit_title: bool, title: str, target: str) -> Tuple[str, str]:
         # basically what sphinx.domains.python.PyXRefRole does
         refnode['js:object'] = env.ref_context.get('js:object')
         refnode['js:module'] = env.ref_context.get('js:module')
@@ -296,12 +293,12 @@ class JavaScriptDomain(Domain):
     label = 'JavaScript'
     # if you add a new object type make sure to edit JSObject.get_index_string
     object_types = {
-        'function':  ObjType(l_('function'),  'func'),
-        'method':    ObjType(l_('method'),    'meth'),
-        'class':     ObjType(l_('class'),     'class'),
-        'data':      ObjType(l_('data'),      'data'),
-        'attribute': ObjType(l_('attribute'), 'attr'),
-        'module':    ObjType(l_('module'),    'mod'),
+        'function':  ObjType(_('function'),  'func'),
+        'method':    ObjType(_('method'),    'meth'),
+        'class':     ObjType(_('class'),     'class'),
+        'data':      ObjType(_('data'),      'data'),
+        'attribute': ObjType(_('attribute'), 'attr'),
+        'module':    ObjType(_('module'),    'mod'),
     }
     directives = {
         'function':  JSCallable,
@@ -321,33 +318,48 @@ class JavaScriptDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-        'modules': {},  # mod_name -> docname
-    }  # type: Dict[unicode, Dict[unicode, Tuple[unicode, unicode]]]
+        'modules': {},  # modname  -> docname
+    }  # type: Dict[str, Dict[str, Tuple[str, str]]]
 
-    def clear_doc(self, docname):
-        # type: (unicode) -> None
-        for fullname, (pkg_docname, _l) in list(self.data['objects'].items()):
-            if pkg_docname == docname:
-                del self.data['objects'][fullname]
-        for mod_name, pkg_docname in list(self.data['modules'].items()):
-            if pkg_docname == docname:
-                del self.data['modules'][mod_name]
+    @property
+    def objects(self) -> Dict[str, Tuple[str, str]]:
+        return self.data.setdefault('objects', {})  # fullname -> docname, objtype
 
-    def merge_domaindata(self, docnames, otherdata):
-        # type: (List[unicode], Dict) -> None
+    def note_object(self, fullname: str, objtype: str, location: Any = None) -> None:
+        if fullname in self.objects:
+            docname = self.objects[fullname][0]
+            logger.warning(__('duplicate object description of %s, other instance in %s'),
+                           fullname, docname, location=location)
+        self.objects[fullname] = (self.env.docname, objtype)
+
+    @property
+    def modules(self) -> Dict[str, str]:
+        return self.data.setdefault('modules', {})  # modname -> docname
+
+    def note_module(self, modname: str) -> None:
+        self.modules[modname] = self.env.docname
+
+    def clear_doc(self, docname: str) -> None:
+        for fullname, (pkg_docname, _l) in list(self.objects.items()):
+            if pkg_docname == docname:
+                del self.objects[fullname]
+        for modname, pkg_docname in list(self.modules.items()):
+            if pkg_docname == docname:
+                del self.modules[modname]
+
+    def merge_domaindata(self, docnames: List[str], otherdata: Dict) -> None:
         # XXX check duplicates
         for fullname, (fn, objtype) in otherdata['objects'].items():
             if fn in docnames:
-                self.data['objects'][fullname] = (fn, objtype)
+                self.objects[fullname] = (fn, objtype)
         for mod_name, pkg_docname in otherdata['modules'].items():
             if pkg_docname in docnames:
-                self.data['modules'][mod_name] = pkg_docname
+                self.modules[mod_name] = pkg_docname
 
-    def find_obj(self, env, mod_name, prefix, name, typ, searchorder=0):
-        # type: (BuildEnvironment, unicode, unicode, unicode, unicode, int) -> Tuple[unicode, Tuple[unicode, unicode]]  # NOQA
+    def find_obj(self, env: BuildEnvironment, mod_name: str, prefix: str, name: str,
+                 typ: str, searchorder: int = 0) -> Tuple[str, Tuple[str, str]]:
         if name[-2:] == '()':
             name = name[:-2]
-        objects = self.data['objects']
 
         searches = []
         if mod_name and prefix:
@@ -363,26 +375,26 @@ class JavaScriptDomain(Domain):
 
         newname = None
         for search_name in searches:
-            if search_name in objects:
+            if search_name in self.objects:
                 newname = search_name
 
-        return newname, objects.get(newname)
+        return newname, self.objects.get(newname)
 
-    def resolve_xref(self, env, fromdocname, builder, typ, target, node,
-                     contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
+    def resolve_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
+                     typ: str, target: str, node: pending_xref, contnode: Element
+                     ) -> Element:
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
-        searchorder = node.hasattr('refspecific') and 1 or 0
+        searchorder = 1 if node.hasattr('refspecific') else 0
         name, obj = self.find_obj(env, mod_name, prefix, target, typ, searchorder)
         if not obj:
             return None
         return make_refnode(builder, fromdocname, obj[0],
                             name.replace('$', '_S_'), contnode, name)
 
-    def resolve_any_xref(self, env, fromdocname, builder, target, node,
-                         contnode):
-        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[Tuple[unicode, nodes.Node]]  # NOQA
+    def resolve_any_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
+                         target: str, node: pending_xref, contnode: Element
+                         ) -> List[Tuple[str, Element]]:
         mod_name = node.get('js:module')
         prefix = node.get('js:object')
         name, obj = self.find_obj(env, mod_name, prefix, target, None, 1)
@@ -392,14 +404,11 @@ class JavaScriptDomain(Domain):
                  make_refnode(builder, fromdocname, obj[0],
                               name.replace('$', '_S_'), contnode, name))]
 
-    def get_objects(self):
-        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
-        for refname, (docname, type) in list(self.data['objects'].items()):
-            yield refname, refname, type, docname, \
-                refname.replace('$', '_S_'), 1
+    def get_objects(self) -> Iterator[Tuple[str, str, str, str, str, int]]:
+        for refname, (docname, type) in list(self.objects.items()):
+            yield refname, refname, type, docname, refname.replace('$', '_S_'), 1
 
-    def get_full_qualified_name(self, node):
-        # type: (nodes.Node) -> unicode
+    def get_full_qualified_name(self, node: Element) -> str:
         modname = node.get('js:module')
         prefix = node.get('js:object')
         target = node.get('reftarget')
@@ -409,12 +418,12 @@ class JavaScriptDomain(Domain):
             return '.'.join(filter(None, [modname, prefix, target]))
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_domain(JavaScriptDomain)
 
     return {
         'version': 'builtin',
+        'env_version': 1,
         'parallel_read_safe': True,
         'parallel_write_safe': True,
     }
