@@ -1,51 +1,46 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.environment.adapters.indexentries
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     Index entries adapters for sphinx.environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import bisect
 import re
 import unicodedata
 from itertools import groupby
+from typing import Any, Dict, Pattern, List, Tuple
+from typing import cast
 
-from six import text_type, iteritems
-
-from sphinx.locale import _
+from sphinx.builders import Builder
+from sphinx.domains.index import IndexDomain
+from sphinx.environment import BuildEnvironment
+from sphinx.errors import NoUri
+from sphinx.locale import _, __
 from sphinx.util import split_into, logging
 
-if False:
-    # For type annotation
-    from typing import Any, Dict, Pattern, List, Tuple  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
 
 logger = logging.getLogger(__name__)
 
 
-class IndexEntries(object):
-    def __init__(self, env):
-        # type: (BuildEnvironment) -> None
+class IndexEntries:
+    def __init__(self, env: BuildEnvironment) -> None:
         self.env = env
 
-    def create_index(self, builder, group_entries=True,
-                     _fixre=re.compile(r'(.*) ([(][^()]*[)])')):
-        # type: (Builder, bool, Pattern) -> List[Tuple[unicode, List[Tuple[unicode, Any]]]]  # NOQA
+    def create_index(self, builder: Builder, group_entries: bool = True,
+                     _fixre: Pattern = re.compile(r'(.*) ([(][^()]*[)])')
+                     ) -> List[Tuple[str, List[Tuple[str, Any]]]]:
         """Create the real index from the collected index entries."""
-        from sphinx.environment import NoUri
+        new = {}  # type: Dict[str, List]
 
-        new = {}  # type: Dict[unicode, List]
-
-        def add_entry(word, subword, main, link=True, dic=new, key=None):
-            # type: (unicode, unicode, unicode, bool, Dict, unicode) -> None
+        def add_entry(word: str, subword: str, main: str, link: bool = True,
+                      dic: Dict = new, key: str = None) -> None:
             # Force the word to be unicode if it's a ASCII bytestring.
             # This will solve problems with unicode normalization later.
             # For instance the RFC role will add bytestrings at the moment
-            word = text_type(word)
+            word = str(word)
             entry = dic.get(word)
             if not entry:
                 dic[word] = entry = [[], {}, key]
@@ -60,7 +55,8 @@ class IndexEntries(object):
                     # maintain links in sorted/deterministic order
                     bisect.insort(entry[0], (main, uri))
 
-        for fn, entries in iteritems(self.env.indexentries):
+        domain = cast(IndexDomain, self.env.get_domain('index'))
+        for fn, entries in domain.entries.items():
             # new entry types must be listed in directives/other.py!
             for type, value, tid, main, index_key in entries:
                 try:
@@ -89,20 +85,19 @@ class IndexEntries(object):
                         add_entry(first, _('see also %s') % second, None,
                                   link=False, key=index_key)
                     else:
-                        logger.warning('unknown index entry type %r', type, location=fn)
+                        logger.warning(__('unknown index entry type %r'), type, location=fn)
                 except ValueError as err:
                     logger.warning(str(err), location=fn)
 
         # sort the index entries; put all symbols at the front, even those
         # following the letters in ASCII, this is where the chr(127) comes from
-        def keyfunc(entry):
-            # type: (Tuple[unicode, List]) -> Tuple[unicode, unicode]
+        def keyfunc(entry: Tuple[str, List]) -> Tuple[str, str]:
             key, (void, void, category_key) = entry
             if category_key:
                 # using specified category key to sort
                 key = category_key
             lckey = unicodedata.normalize('NFD', key.lower())
-            if lckey.startswith(u'\N{RIGHT-TO-LEFT MARK}'):
+            if lckey.startswith('\N{RIGHT-TO-LEFT MARK}'):
                 lckey = lckey[1:]
             if lckey[0:1].isalpha() or lckey.startswith('_'):
                 lckey = chr(127) + lckey
@@ -119,8 +114,8 @@ class IndexEntries(object):
             #   func()
             #     (in module foo)
             #     (in module bar)
-            oldkey = ''  # type: unicode
-            oldsubitems = None  # type: Dict[unicode, List]
+            oldkey = ''
+            oldsubitems = None  # type: Dict[str, List]
             i = 0
             while i < len(newlist):
                 key, (targets, subitems, _key) = newlist[i]
@@ -141,15 +136,24 @@ class IndexEntries(object):
                 oldsubitems = subitems
                 i += 1
 
+        # sort the sub-index entries
+        def keyfunc2(entry: Tuple[str, List]) -> str:
+            key = unicodedata.normalize('NFD', entry[0].lower())
+            if key.startswith('\N{RIGHT-TO-LEFT MARK}'):
+                key = key[1:]
+            if key[0:1].isalpha() or key.startswith('_'):
+                key = chr(127) + key
+            return key
+
         # group the entries by letter
-        def keyfunc2(item):
-            # type: (Tuple[unicode, List]) -> unicode
+        def keyfunc3(item: Tuple[str, List]) -> str:
             # hack: mutating the subitems dicts to a list in the keyfunc
             k, v = item
-            v[1] = sorted((si, se) for (si, (se, void, void)) in iteritems(v[1]))
+            v[1] = sorted(((si, se) for (si, (se, void, void)) in v[1].items()),
+                          key=keyfunc2)
             if v[2] is None:
                 # now calculate the key
-                if k.startswith(u'\N{RIGHT-TO-LEFT MARK}'):
+                if k.startswith('\N{RIGHT-TO-LEFT MARK}'):
                     k = k[1:]
                 letter = unicodedata.normalize('NFD', k[0])[0].upper()
                 if letter.isalpha() or letter == '_':
@@ -160,4 +164,4 @@ class IndexEntries(object):
             else:
                 return v[2]
         return [(key_, list(group))
-                for (key_, group) in groupby(newlist, keyfunc2)]
+                for (key_, group) in groupby(newlist, keyfunc3)]
