@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.events
     ~~~~~~~~~~~~~
@@ -7,21 +6,25 @@
 
     Gracefully adapted from the TextPress system by Armin.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
+import warnings
 from collections import OrderedDict, defaultdict
+from typing import Any, Callable, Dict, List
 
-from six import itervalues
-
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.errors import ExtensionError
 from sphinx.locale import __
+from sphinx.util import logging
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, List  # NOQA
+    from sphinx.application import Sphinx
+
+
+logger = logging.getLogger(__name__)
 
 
 # List of all known core events. Maps name to arguments description.
@@ -42,24 +45,29 @@ core_events = {
     'html-collect-pages': 'builder',
     'html-page-context': 'pagename, context, doctree or None',
     'build-finished': 'exception',
-}  # type: Dict[unicode, unicode]
+}
 
 
-class EventManager(object):
-    def __init__(self):
-        # type: () -> None
+class EventManager:
+    """Event manager for Sphinx."""
+
+    def __init__(self, app: "Sphinx" = None) -> None:
+        if app is None:
+            warnings.warn('app argument is required for EventManager.',
+                          RemovedInSphinx40Warning)
+        self.app = app
         self.events = core_events.copy()
-        self.listeners = defaultdict(OrderedDict)  # type: Dict[unicode, Dict[int, Callable]]
+        self.listeners = defaultdict(OrderedDict)  # type: Dict[str, Dict[int, Callable]]
         self.next_listener_id = 0
 
-    def add(self, name):
-        # type: (unicode) -> None
+    def add(self, name: str) -> None:
+        """Register a custom Sphinx event."""
         if name in self.events:
             raise ExtensionError(__('Event %r already present') % name)
         self.events[name] = ''
 
-    def connect(self, name, callback):
-        # type: (unicode, Callable) -> int
+    def connect(self, name: str, callback: Callable) -> int:
+        """Connect a handler to specific event."""
         if name not in self.events:
             raise ExtensionError(__('Unknown event name: %s') % name)
 
@@ -68,20 +76,34 @@ class EventManager(object):
         self.listeners[name][listener_id] = callback
         return listener_id
 
-    def disconnect(self, listener_id):
-        # type: (int) -> None
-        for event in itervalues(self.listeners):
+    def disconnect(self, listener_id: int) -> None:
+        """Disconnect a handler."""
+        for event in self.listeners.values():
             event.pop(listener_id, None)
 
-    def emit(self, name, *args):
-        # type: (unicode, Any) -> List
+    def emit(self, name: str, *args: Any) -> List:
+        """Emit a Sphinx event."""
+        try:
+            logger.debug('[app] emitting event: %r%s', name, repr(args)[:100])
+        except Exception:
+            # not every object likes to be repr()'d (think
+            # random stuff coming via autodoc)
+            pass
+
         results = []
-        for callback in itervalues(self.listeners[name]):
-            results.append(callback(*args))
+        for callback in self.listeners[name].values():
+            if self.app is None:
+                # for compatibility; RemovedInSphinx40Warning
+                results.append(callback(*args))
+            else:
+                results.append(callback(self.app, *args))
         return results
 
-    def emit_firstresult(self, name, *args):
-        # type: (unicode, Any) -> Any
+    def emit_firstresult(self, name: str, *args: Any) -> Any:
+        """Emit a Sphinx event and returns first result.
+
+        This returns the result of the first handler that doesn't return ``None``.
+        """
         for result in self.emit(name, *args):
             if result is not None:
                 return result
