@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.doctest
     ~~~~~~~~~~~~~~~~~~
@@ -6,59 +5,53 @@
     Mimic doctest by automatically executing code snippets and checking
     their results.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
 
-import codecs
 import doctest
 import re
 import sys
 import time
+import warnings
+from io import StringIO
 from os import path
+from typing import Any, Callable, Dict, Iterable, List, Sequence, Set, Tuple
 
 from docutils import nodes
+from docutils.nodes import Element, Node, TextElement
 from docutils.parsers.rst import directives
 from packaging.specifiers import SpecifierSet, InvalidSpecifier
 from packaging.version import Version
-from six import itervalues, StringIO, binary_type, text_type, PY2
 
 import sphinx
 from sphinx.builders import Builder
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import __
-from sphinx.util import force_decode, logging
+from sphinx.util import logging
 from sphinx.util.console import bold  # type: ignore
 from sphinx.util.docutils import SphinxDirective
-from sphinx.util.nodes import set_source_info
-from sphinx.util.osutil import fs_encoding, relpath
+from sphinx.util.osutil import relpath
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterable, List, Optional, Sequence, Set, Tuple  # NOQA
-    from sphinx.application import Sphinx  # NOQA
+    from typing import Type  # for python3.5.1
+    from sphinx.application import Sphinx
+
 
 logger = logging.getLogger(__name__)
 
 blankline_re = re.compile(r'^\s*<BLANKLINE>', re.MULTILINE)
 doctestopt_re = re.compile(r'#\s*doctest:.+$', re.MULTILINE)
 
-if PY2:
-    def doctest_encode(text, encoding):
-        # type: (str, unicode) -> unicode
-        if isinstance(text, text_type):
-            text = text.encode(encoding)
-            if text.startswith(codecs.BOM_UTF8):
-                text = text[len(codecs.BOM_UTF8):]
-        return text
-else:
-    def doctest_encode(text, encoding):
-        # type: (unicode, unicode) -> unicode
-        return text
+
+def doctest_encode(text: str, encoding: str) -> str:
+    warnings.warn('doctest_encode() is deprecated.',
+                  RemovedInSphinx40Warning)
+    return text
 
 
-def is_allowed_version(spec, version):
-    # type: (unicode, unicode) -> bool
+def is_allowed_version(spec: str, version: str) -> bool:
     """Check `spec` satisfies `version` or not.
 
     This obeys PEP-440 specifiers:
@@ -88,8 +81,7 @@ class TestDirective(SphinxDirective):
     optional_arguments = 1
     final_argument_whitespace = True
 
-    def run(self):
-        # type: () -> List[nodes.Node]
+    def run(self) -> List[Node]:
         # use ordinary docutils nodes for test code: they get special attributes
         # so that our builder recognizes them, and the other builders are happy.
         code = '\n'.join(self.content)
@@ -103,7 +95,7 @@ class TestDirective(SphinxDirective):
                 if not test:
                     test = code
                 code = doctestopt_re.sub('', code)
-        nodetype = nodes.literal_block
+        nodetype = nodes.literal_block  # type: Type[TextElement]
         if self.name in ('testsetup', 'testcleanup') or 'hide' in self.options:
             nodetype = nodes.comment
         if self.arguments:
@@ -111,14 +103,20 @@ class TestDirective(SphinxDirective):
         else:
             groups = ['default']
         node = nodetype(code, code, testnodetype=self.name, groups=groups)
-        set_source_info(self, node)
+        self.set_source_info(node)
         if test is not None:
             # only save if it differs from code
             node['test'] = test
         if self.name == 'doctest':
-            node['language'] = 'pycon'
+            if self.config.highlight_language in ('py', 'python'):
+                node['language'] = 'pycon'
+            else:
+                node['language'] = 'pycon3'  # default
         elif self.name == 'testcode':
-            node['language'] = 'python'
+            if self.config.highlight_language in ('py', 'python'):
+                node['language'] = 'python'
+            else:
+                node['language'] = 'python3'  # default
         elif self.name == 'testoutput':
             # don't try to highlight output
             node['language'] = 'none'
@@ -195,16 +193,14 @@ parser = doctest.DocTestParser()
 
 # helper classes
 
-class TestGroup(object):
-    def __init__(self, name):
-        # type: (unicode) -> None
+class TestGroup:
+    def __init__(self, name: str) -> None:
         self.name = name
         self.setup = []     # type: List[TestCode]
         self.tests = []     # type: List[List[TestCode]]
         self.cleanup = []   # type: List[TestCode]
 
-    def add_code(self, code, prepend=False):
-        # type: (TestCode, bool) -> None
+    def add_code(self, code: "TestCode", prepend: bool = False) -> None:
         if code.type == 'testsetup':
             if prepend:
                 self.setup.insert(0, code)
@@ -222,43 +218,40 @@ class TestGroup(object):
         else:
             raise RuntimeError(__('invalid TestCode type'))
 
-    def __repr__(self):  # type: ignore
-        # type: () -> unicode
+    def __repr__(self) -> str:
         return 'TestGroup(name=%r, setup=%r, cleanup=%r, tests=%r)' % (
             self.name, self.setup, self.cleanup, self.tests)
 
 
-class TestCode(object):
-    def __init__(self, code, type, filename, lineno, options=None):
-        # type: (unicode, unicode, Optional[str], int, Optional[Dict]) -> None
+class TestCode:
+    def __init__(self, code: str, type: str, filename: str,
+                 lineno: int, options: Dict = None) -> None:
         self.code = code
         self.type = type
         self.filename = filename
         self.lineno = lineno
         self.options = options or {}
 
-    def __repr__(self):  # type: ignore
-        # type: () -> unicode
+    def __repr__(self) -> str:
         return 'TestCode(%r, %r, filename=%r, lineno=%r, options=%r)' % (
             self.code, self.type, self.filename, self.lineno, self.options)
 
 
 class SphinxDocTestRunner(doctest.DocTestRunner):
-    def summarize(self, out, verbose=None):  # type: ignore
-        # type: (Callable, bool) -> Tuple[int, int]
+    def summarize(self, out: Callable, verbose: bool = None  # type: ignore
+                  ) -> Tuple[int, int]:
         string_io = StringIO()
         old_stdout = sys.stdout
         sys.stdout = string_io
         try:
-            res = doctest.DocTestRunner.summarize(self, verbose)
+            res = super().summarize(verbose)
         finally:
             sys.stdout = old_stdout
         out(string_io.getvalue())
         return res
 
-    def _DocTestRunner__patched_linecache_getlines(self, filename,
-                                                   module_globals=None):
-        # type: (unicode, Any) -> Any
+    def _DocTestRunner__patched_linecache_getlines(self, filename: str,
+                                                   module_globals: Any = None) -> Any:
         # this is overridden from DocTestRunner adding the try-except below
         m = self._DocTestRunner__LINECACHE_FILENAME_RE.match(filename)  # type: ignore
         if m and m.group('name') == self.test.name:
@@ -284,8 +277,7 @@ class DocTestBuilder(Builder):
     epilog = __('Testing of doctests in the sources finished, look at the '
                 'results in %(outdir)s/output.txt.')
 
-    def init(self):
-        # type: () -> None
+    def init(self) -> None:
         # default options
         self.opt = self.config.doctest_default_flags
 
@@ -309,42 +301,32 @@ class DocTestBuilder(Builder):
 
         date = time.strftime('%Y-%m-%d %H:%M:%S')
 
-        self.outfile = None  # type: IO
-        self.outfile = codecs.open(path.join(self.outdir, 'output.txt'),  # type: ignore
-                                   'w', encoding='utf-8')
+        self.outfile = open(path.join(self.outdir, 'output.txt'), 'w', encoding='utf-8')
         self.outfile.write(('Results of doctest builder run on %s\n'
                             '==================================%s\n') %
                            (date, '=' * len(date)))
 
-    def _out(self, text):
-        # type: (unicode) -> None
+    def _out(self, text: str) -> None:
         logger.info(text, nonl=True)
         self.outfile.write(text)
 
-    def _warn_out(self, text):
-        # type: (unicode) -> None
+    def _warn_out(self, text: str) -> None:
         if self.app.quiet or self.app.warningiserror:
             logger.warning(text)
         else:
             logger.info(text, nonl=True)
-        if isinstance(text, binary_type):
-            text = force_decode(text, None)
         self.outfile.write(text)
 
-    def get_target_uri(self, docname, typ=None):
-        # type: (unicode, unicode) -> unicode
+    def get_target_uri(self, docname: str, typ: str = None) -> str:
         return ''
 
-    def get_outdated_docs(self):
-        # type: () -> Set[unicode]
+    def get_outdated_docs(self) -> Set[str]:
         return self.env.found_docs
 
-    def finish(self):
-        # type: () -> None
+    def finish(self) -> None:
         # write executive summary
-        def s(v):
-            # type: (int) -> unicode
-            return v != 1 and 's' or ''
+        def s(v: int) -> str:
+            return 's' if v != 1 else ''
         repl = (self.total_tries, s(self.total_tries),
                 self.total_failures, s(self.total_failures),
                 self.setup_failures, s(self.setup_failures),
@@ -362,8 +344,8 @@ Doctest summary
         if self.total_failures or self.setup_failures or self.cleanup_failures:
             self.app.statuscode = 1
 
-    def write(self, build_docnames, updated_docnames, method='update'):
-        # type: (Iterable[unicode], Sequence[unicode], unicode) -> None
+    def write(self, build_docnames: Iterable[str], updated_docnames: Sequence[str],
+              method: str = 'update') -> None:
         if build_docnames is None:
             build_docnames = sorted(self.env.all_docs)
 
@@ -373,8 +355,7 @@ Doctest summary
             doctree = self.env.get_doctree(docname)
             self.test_doc(docname, doctree)
 
-    def get_filename_for_node(self, node, docname):
-        # type: (nodes.Node, unicode) -> str
+    def get_filename_for_node(self, node: Node, docname: str) -> str:
         """Try to get the file which actually contains the doctest, not the
         filename of the document it's included in."""
         try:
@@ -382,13 +363,10 @@ Doctest summary
                 .rsplit(':docstring of ', maxsplit=1)[0]
         except Exception:
             filename = self.env.doc2path(docname, base=None)
-        if PY2:
-            return filename.encode(fs_encoding)
         return filename
 
     @staticmethod
-    def get_line_number(node):
-        # type: (nodes.Node) -> Optional[int]
+    def get_line_number(node: Node) -> int:
         """Get the real line number or admit we don't know."""
         # TODO:  Work out how to store or calculate real (file-relative)
         #       line numbers for doctest blocks in docstrings.
@@ -403,8 +381,7 @@ Doctest summary
             return node.line - 1
         return None
 
-    def skipped(self, node):
-        # type: (nodes.Element) -> bool
+    def skipped(self, node: Element) -> bool:
         if 'skipif' not in node:
             return False
         else:
@@ -417,9 +394,8 @@ Doctest summary
                 exec(self.config.doctest_global_cleanup, context)
             return should_skip
 
-    def test_doc(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
-        groups = {}  # type: Dict[unicode, TestGroup]
+    def test_doc(self, docname: str, doctree: Node) -> None:
+        groups = {}  # type: Dict[str, TestGroup]
         add_to_all_groups = []
         self.setup_runner = SphinxDocTestRunner(verbose=False,
                                                 optionflags=self.opt)
@@ -432,17 +408,15 @@ Doctest summary
         self.cleanup_runner._fakeout = self.setup_runner._fakeout  # type: ignore
 
         if self.config.doctest_test_doctest_blocks:
-            def condition(node):
-                # type: (nodes.Node) -> bool
+            def condition(node: Node) -> bool:
                 return (isinstance(node, (nodes.literal_block, nodes.comment)) and
                         'testnodetype' in node) or \
                     isinstance(node, nodes.doctest_block)
         else:
-            def condition(node):
-                # type: (nodes.Node) -> bool
+            def condition(node: Node) -> bool:
                 return isinstance(node, (nodes.literal_block, nodes.comment)) \
                     and 'testnodetype' in node
-        for node in doctree.traverse(condition):
+        for node in doctree.traverse(condition):  # type: Element
             if self.skipped(node):
                 continue
 
@@ -465,24 +439,24 @@ Doctest summary
                     groups[groupname] = TestGroup(groupname)
                 groups[groupname].add_code(code)
         for code in add_to_all_groups:
-            for group in itervalues(groups):
+            for group in groups.values():
                 group.add_code(code)
         if self.config.doctest_global_setup:
             code = TestCode(self.config.doctest_global_setup,
                             'testsetup', filename=None, lineno=0)
-            for group in itervalues(groups):
+            for group in groups.values():
                 group.add_code(code, prepend=True)
         if self.config.doctest_global_cleanup:
             code = TestCode(self.config.doctest_global_cleanup,
                             'testcleanup', filename=None, lineno=0)
-            for group in itervalues(groups):
+            for group in groups.values():
                 group.add_code(code)
         if not groups:
             return
 
         self._out('\nDocument: %s\n----------%s\n' %
                   (docname, '-' * len(docname)))
-        for group in itervalues(groups):
+        for group in groups.values():
             self.test_group(group)
         # Separately count results from setup code
         res_f, res_t = self.setup_runner.summarize(self._out, verbose=False)
@@ -498,21 +472,17 @@ Doctest summary
             self.cleanup_failures += res_f
             self.cleanup_tries += res_t
 
-    def compile(self, code, name, type, flags, dont_inherit):
-        # type: (unicode, unicode, unicode, Any, bool) -> Any
+    def compile(self, code: str, name: str, type: str, flags: Any, dont_inherit: bool) -> Any:
         return compile(code, name, self.type, flags, dont_inherit)
 
-    def test_group(self, group):
-        # type: (TestGroup) -> None
+    def test_group(self, group: TestGroup) -> None:
         ns = {}  # type: Dict
 
-        def run_setup_cleanup(runner, testcodes, what):
-            # type: (Any, List[TestCode], Any) -> bool
+        def run_setup_cleanup(runner: Any, testcodes: List[TestCode], what: Any) -> bool:
             examples = []
             for testcode in testcodes:
-                examples.append(doctest.Example(  # type: ignore
-                    doctest_encode(testcode.code, self.env.config.source_encoding), '',  # type: ignore  # NOQA
-                    lineno=testcode.lineno))
+                example = doctest.Example(testcode.code, '', lineno=testcode.lineno)
+                examples.append(example)
             if not examples:
                 return True
             # simulate a doctest with the code
@@ -537,9 +507,8 @@ Doctest summary
             if len(code) == 1:
                 # ordinary doctests (code/output interleaved)
                 try:
-                    test = parser.get_doctest(  # type: ignore
-                        doctest_encode(code[0].code, self.env.config.source_encoding), {},  # type: ignore  # NOQA
-                        group.name, code[0].filename, code[0].lineno)
+                    test = parser.get_doctest(code[0].code, {}, group.name,
+                                              code[0].filename, code[0].lineno)
                 except Exception:
                     logger.warning(__('ignoring invalid doctest code: %r'), code[0].code,
                                    location=(code[0].filename, code[0].lineno))
@@ -554,8 +523,8 @@ Doctest summary
                 self.type = 'single'  # as for ordinary doctests
             else:
                 # testcode and output separate
-                output = code[1] and code[1].code or ''
-                options = code[1] and code[1].options or {}
+                output = code[1].code if code[1] else ''
+                options = code[1].options if code[1] else {}
                 # disable <BLANKLINE> processing as it is not needed
                 options[doctest.DONT_ACCEPT_BLANKLINE] = True
                 # find out if we're testing an exception
@@ -564,12 +533,9 @@ Doctest summary
                     exc_msg = m.group('msg')
                 else:
                     exc_msg = None
-                example = doctest.Example(  # type: ignore
-                    doctest_encode(code[0].code, self.env.config.source_encoding), output,  # type: ignore  # NOQA
-                    exc_msg=exc_msg,
-                    lineno=code[0].lineno,
-                    options=options)
-                test = doctest.DocTest([example], {}, group.name,  # type: ignore
+                example = doctest.Example(code[0].code, output, exc_msg=exc_msg,
+                                          lineno=code[0].lineno, options=options)
+                test = doctest.DocTest([example], {}, group.name,
                                        code[0].filename, code[0].lineno, None)
                 self.type = 'exec'  # multiple statements again
             # DocTest.__init__ copies the globs namespace, which we don't want
@@ -581,8 +547,7 @@ Doctest summary
         run_setup_cleanup(self.cleanup_runner, group.cleanup, 'cleanup')
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+def setup(app: "Sphinx") -> Dict[str, Any]:
     app.add_directive('testsetup', TestsetupDirective)
     app.add_directive('testcleanup', TestcleanupDirective)
     app.add_directive('doctest', DoctestDirective)
