@@ -1,30 +1,29 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.extension
     ~~~~~~~~~~~~~~~~
 
     Utilities for Sphinx extensions.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from six import iteritems
+from typing import Any, Dict
 
+from sphinx.config import Config
 from sphinx.errors import VersionRequirementError
 from sphinx.locale import __
 from sphinx.util import logging
 
 if False:
     # For type annotation
-    from typing import Dict  # NOQA
-    from sphinx.application import Sphinx  # NOQA
+    from sphinx.application import Sphinx
 
 logger = logging.getLogger(__name__)
 
 
-class Extension(object):
-    def __init__(self, name, module, **kwargs):
+class Extension:
+    def __init__(self, name: str, module: Any, **kwargs: Any) -> None:
         self.name = name
         self.module = module
         self.metadata = kwargs
@@ -38,19 +37,18 @@ class Extension(object):
         # The extension supports parallel write or not.  The default value
         # is ``True``.  Sphinx writes parallelly documents even if
         # the extension does not tell its status.
-        self.parallel_write_safe = kwargs.pop('parallel_read_safe', True)
+        self.parallel_write_safe = kwargs.pop('parallel_write_safe', True)
 
 
-def verify_required_extensions(app, requirements):
-    # type: (Sphinx, Dict[unicode, unicode]) -> None
+def verify_needs_extensions(app: "Sphinx", config: Config) -> None:
     """Verify the required Sphinx extensions are loaded."""
-    if requirements is None:
+    if config.needs_extensions is None:
         return
 
-    for extname, reqversion in iteritems(requirements):
+    for extname, reqversion in config.needs_extensions.items():
         extension = app.extensions.get(extname)
         if extension is None:
-            logger.warning(__('The %s extension is required by needs_extensions settings,'
+            logger.warning(__('The %s extension is required by needs_extensions settings, '
                               'but it is not loaded.'), extname)
             continue
 
@@ -59,3 +57,13 @@ def verify_required_extensions(app, requirements):
                                              'version %s and therefore cannot be built with '
                                              'the loaded version (%s).') %
                                           (extname, reqversion, extension.version))
+
+
+def setup(app: "Sphinx") -> Dict[str, Any]:
+    app.connect('config-inited', verify_needs_extensions)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
