@@ -1,52 +1,46 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.jinja2glue
     ~~~~~~~~~~~~~~~~~
 
     Glue code for the jinja2 templating engine.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 from pprint import pformat
-from typing import Any, Callable, Iterator, Tuple  # NOQA
+from typing import Any, Callable, Dict, Iterator, List, Tuple, Union
 
-from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, \
-    contextfunction
+from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, contextfunction
+from jinja2.environment import Environment
 from jinja2.sandbox import SandboxedEnvironment
 from jinja2.utils import open_if_exists
-from six import string_types
 
 from sphinx.application import TemplateBridge
+from sphinx.theming import Theme
+from sphinx.util import logging
 from sphinx.util.osutil import mtimes_of_files
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, List, Iterator, Tuple  # NOQA
-    from jinja2.environment import Environment  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.theming import Theme  # NOQA
+    from sphinx.builders import Builder
 
 
-def _tobool(val):
-    # type: (unicode) -> bool
-    if isinstance(val, string_types):
+def _tobool(val: str) -> bool:
+    if isinstance(val, str):
         return val.lower() in ('true', '1', 'yes', 'on')
     return bool(val)
 
 
-def _toint(val):
-    # type: (unicode) -> int
+def _toint(val: str) -> int:
     try:
         return int(val)
     except ValueError:
         return 0
 
 
-def _todim(val):
-    # type (int or unicode) -> unicode
+def _todim(val: Union[int, str]) -> str:
     """
     Make val a css dimension. In particular the following transformations
     are performed:
@@ -61,11 +55,10 @@ def _todim(val):
         return 'initial'
     elif str(val).isdigit():
         return '0' if int(val) == 0 else '%spx' % val
-    return val
+    return val  # type: ignore
 
 
-def _slice_index(values, slices):
-    # type: (List, int) -> Iterator[List]
+def _slice_index(values: List, slices: int) -> Iterator[List]:
     seq = list(values)
     length = 0
     for value in values:
@@ -86,8 +79,7 @@ def _slice_index(values, slices):
         yield seq[start:offset]
 
 
-def accesskey(context, key):
-    # type: (Any, unicode) -> unicode
+def accesskey(context: Any, key: str) -> str:
     """Helper to output each access key only once."""
     if '_accesskeys' not in context:
         context.vars['_accesskeys'] = {}
@@ -97,30 +89,36 @@ def accesskey(context, key):
     return ''
 
 
-class idgen(object):
-    def __init__(self):
-        # type: () -> None
+class idgen:
+    def __init__(self) -> None:
         self.id = 0
 
-    def current(self):
-        # type: () -> int
+    def current(self) -> int:
         return self.id
 
-    def __next__(self):
-        # type: () -> int
+    def __next__(self) -> int:
         self.id += 1
         return self.id
     next = __next__  # Python 2/Jinja compatibility
 
 
+@contextfunction
+def warning(context: Dict, message: str, *args: Any, **kwargs: Any) -> str:
+    if 'pagename' in context:
+        filename = context.get('pagename') + context.get('file_suffix', '')
+        message = 'in rendering %s: %s' % (filename, message)
+    logger = logging.getLogger('sphinx.themes')
+    logger.warning(message, *args, **kwargs)
+    return ''  # return empty string not to output any values
+
+
 class SphinxFileSystemLoader(FileSystemLoader):
     """
     FileSystemLoader subclass that is not so strict about '..'  entries in
     template names.
     """
 
-    def get_source(self, environment, template):
-        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
+    def get_source(self, environment: Environment, template: str) -> Tuple[str, str, Callable]:
         for searchpath in self.searchpath:
             filename = path.join(searchpath, template)
             f = open_if_exists(filename)
@@ -131,8 +129,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
 
             mtime = path.getmtime(filename)
 
-            def uptodate():
-                # type: () -> bool
+            def uptodate() -> bool:
                 try:
                     return path.getmtime(filename) == mtime
                 except OSError:
@@ -148,8 +145,7 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
 
     # TemplateBridge interface
 
-    def init(self, builder, theme=None, dirs=None):
-        # type: (Builder, Theme, List[unicode]) -> None
+    def init(self, builder: "Builder", theme: Theme = None, dirs: List[str] = None) -> None:
         # create a chain of paths to search
         if theme:
             # the theme's own dir and its bases' dirs
@@ -178,7 +174,7 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
         self.loaders = [SphinxFileSystemLoader(x) for x in loaderchain]
 
         use_i18n = builder.app.translator is not None
-        extensions = use_i18n and ['jinja2.ext.i18n'] or []
+        extensions = ['jinja2.ext.i18n'] if use_i18n else []
         self.environment = SandboxedEnvironment(loader=self,
                                                 extensions=extensions)
         self.environment.filters['tobool'] = _tobool
@@ -186,27 +182,24 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
         self.environment.filters['todim'] = _todim
         self.environment.filters['slice_index'] = _slice_index
         self.environment.globals['debug'] = contextfunction(pformat)
+        self.environment.globals['warning'] = warning
         self.environment.globals['accesskey'] = contextfunction(accesskey)
         self.environment.globals['idgen'] = idgen
         if use_i18n:
             self.environment.install_gettext_translations(builder.app.translator)  # type: ignore  # NOQA
 
-    def render(self, template, context):  # type: ignore
-        # type: (unicode, Dict) -> unicode
+    def render(self, template: str, context: Dict) -> str:  # type: ignore
         return self.environment.get_template(template).render(context)
 
-    def render_string(self, source, context):
-        # type: (unicode, Dict) -> unicode
+    def render_string(self, source: str, context: Dict) -> str:
         return self.environment.from_string(source).render(context)
 
-    def newest_template_mtime(self):
-        # type: () -> float
+    def newest_template_mtime(self) -> float:
         return max(mtimes_of_files(self.pathchain, '.html'))
 
     # Loader interface
 
-    def get_source(self, environment, template):
-        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
+    def get_source(self, environment: Environment, template: str) -> Tuple[str, str, Callable]:
         loaders = self.loaders
         # exclamation mark starts search from theme
         if template.startswith('!'):
