@@ -1,26 +1,29 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.parsers
     ~~~~~~~~~~~~~~
 
     A Base class for additional parsers.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Any, Dict, List, Union
+
 import docutils.parsers
 import docutils.parsers.rst
+from docutils import nodes
 from docutils.parsers.rst import states
 from docutils.statemachine import StringList
 from docutils.transforms.universal import SmartQuotes
 
+from sphinx.util.rst import append_epilog, prepend_prolog
+
 if False:
     # For type annotation
-    from typing import Any, Dict, List, Type  # NOQA
-    from docutils import nodes  # NOQA
     from docutils.transforms import Transform  # NOQA
-    from sphinx.application import Sphinx  # NOQA
+    from typing import Type  # NOQA # for python3.5.1
+    from sphinx.application import Sphinx
 
 
 class Parser(docutils.parsers.Parser):
@@ -46,8 +49,7 @@ class Parser(docutils.parsers.Parser):
        ``warn()`` and ``info()`` is deprecated.  Use :mod:`sphinx.util.logging` instead.
     """
 
-    def set_application(self, app):
-        # type: (Sphinx) -> None
+    def set_application(self, app: "Sphinx") -> None:
         """set_application will be called from Sphinx to set app and other instance variables
 
         :param sphinx.application.Sphinx app: Sphinx application object
@@ -55,45 +57,49 @@ class Parser(docutils.parsers.Parser):
         self.app = app
         self.config = app.config
         self.env = app.env
-        self.warn = app.warn
-        self.info = app.info
 
 
-class RSTParser(docutils.parsers.rst.Parser):
+class RSTParser(docutils.parsers.rst.Parser, Parser):
     """A reST parser for Sphinx."""
 
-    def get_transforms(self):
-        # type: () -> List[Type[Transform]]
+    def get_transforms(self) -> List["Type[Transform]"]:
         """Sphinx's reST parser replaces a transform class for smart-quotes by own's
 
-        refs: sphinx.io.SphinxStandaloneReader"""
-        transforms = docutils.parsers.rst.Parser.get_transforms(self)
+        refs: sphinx.io.SphinxStandaloneReader
+        """
+        transforms = super().get_transforms()
         transforms.remove(SmartQuotes)
         return transforms
 
-    def parse(self, inputstring, document):
-        # type: (Any, nodes.document) -> None
-        """Parse text and generate a document tree.
+    def parse(self, inputstring: Union[str, StringList], document: nodes.document) -> None:
+        """Parse text and generate a document tree."""
+        self.setup_parse(inputstring, document)  # type: ignore
+        self.statemachine = states.RSTStateMachine(
+            state_classes=self.state_classes,
+            initial_state=self.initial_state,
+            debug=document.reporter.debug_flag)
 
-        This accepts StringList as an inputstring parameter.
-        It enables to handle mixed contents (cf. :confval:`rst_prolog`) correctly.
-        """
-        if isinstance(inputstring, StringList):
-            self.setup_parse(inputstring, document)
-            self.statemachine = states.RSTStateMachine(
-                state_classes=self.state_classes,
-                initial_state=self.initial_state,
-                debug=document.reporter.debug_flag)
-            # Give inputstring directly to statemachine.
-            self.statemachine.run(inputstring, document, inliner=self.inliner)
-            self.finish_parse()
+        # preprocess inputstring
+        if isinstance(inputstring, str):
+            lines = docutils.statemachine.string2lines(
+                inputstring, tab_width=document.settings.tab_width,
+                convert_whitespace=True)
+
+            inputlines = StringList(lines, document.current_source)
         else:
-            # otherwise, inputstring might be a string. It will be handled by superclass.
-            docutils.parsers.rst.Parser.parse(self, inputstring, document)
+            inputlines = inputstring
+
+        self.decorate(inputlines)
+        self.statemachine.run(inputlines, document, inliner=self.inliner)
+        self.finish_parse()
+
+    def decorate(self, content: StringList) -> None:
+        """Preprocess reST content before parsing."""
+        prepend_prolog(content, self.config.rst_prolog)
+        append_epilog(content, self.config.rst_epilog)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+def setup(app: "Sphinx") -> Dict[str, Any]:
     app.add_source_parser(RSTParser)
 
     return {
