@@ -1,34 +1,36 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.search
     ~~~~~~~~~~~~~
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+import html
+import pickle
 import re
+import warnings
+from importlib import import_module
 from os import path
+from typing import Any, Dict, IO, Iterable, List, Tuple, Set
 
-from six import iteritems, itervalues, text_type, string_types
-from six.moves import cPickle as pickle
+from docutils import nodes
+from docutils.nodes import Node
 
-from docutils.nodes import raw, comment, title, Text, NodeVisitor, SkipNode
-
-import sphinx
-from sphinx.util import jsdump, rpartition
-from sphinx.util.pycompat import htmlescape
+from sphinx import addnodes
+from sphinx import package_dir
+from sphinx.deprecation import RemovedInSphinx40Warning
+from sphinx.environment import BuildEnvironment
 from sphinx.search.jssplitter import splitter_code
+from sphinx.util import jsdump, rpartition
 
 if False:
     # For type annotation
-    from typing import Any, Dict, IO, Iterable, List, Tuple, Type, Set  # NOQA
-    from docutils import nodes  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from typing import Type  # for python3.5.1
 
 
-class SearchLanguage(object):
+class SearchLanguage:
     """
     This class is the base class for search natural language preprocessors.  If
     you want to add support for a new language, you should override the methods
@@ -50,10 +52,10 @@ class SearchLanguage(object):
        This class is used to preprocess search word which Sphinx HTML readers
        type, before searching index. Default implementation does nothing.
     """
-    lang = None                 # type: unicode
-    language_name = None        # type: unicode
-    stopwords = set()           # type: Set[unicode]
-    js_stemmer_rawcode = None   # type: unicode
+    lang = None                 # type: str
+    language_name = None        # type: str
+    stopwords = set()           # type: Set[str]
+    js_stemmer_rawcode = None   # type: str
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -63,23 +65,20 @@ var Stemmer = function() {
     return w;
   }
 }
-"""                             # type: unicode
+"""
 
     _word_re = re.compile(r'(?u)\w+')
 
-    def __init__(self, options):
-        # type: (Dict) -> None
+    def __init__(self, options: Dict) -> None:
         self.options = options
         self.init(options)
 
-    def init(self, options):
-        # type: (Dict) -> None
+    def init(self, options: Dict) -> None:
         """
         Initialize the class with the options the user has given.
         """
 
-    def split(self, input):
-        # type: (unicode) -> List[unicode]
+    def split(self, input: str) -> List[str]:
         """
         This method splits a sentence into words.  Default splitter splits input
         at white spaces, which should be enough for most languages except CJK
@@ -87,8 +86,7 @@ var Stemmer = function() {
         """
         return self._word_re.findall(input)
 
-    def stem(self, word):
-        # type: (unicode) -> unicode
+    def stem(self, word: str) -> str:
         """
         This method implements stemming algorithm of the Python version.
 
@@ -101,8 +99,7 @@ var Stemmer = function() {
         """
         return word
 
-    def word_filter(self, word):
-        # type: (unicode) -> bool
+    def word_filter(self, word: str) -> bool:
         """
         Return true if the target word should be registered in the search index.
         This method is called after stemming.
@@ -119,14 +116,13 @@ var Stemmer = function() {
 from sphinx.search.en import SearchEnglish
 
 
-def parse_stop_word(source):
-    # type: (unicode) -> Set[unicode]
+def parse_stop_word(source: str) -> Set[str]:
     """
     parse snowball style word list like this:
 
     * http://snowball.tartarus.org/algorithms/finnish/stop.txt
     """
-    result = set()
+    result = set()  # type: Set[str]
     for line in source.splitlines():
         line = line.split('|')[0]  # remove comment
         result.update(line.split())
@@ -152,10 +148,10 @@ languages = {
     'sv': 'sphinx.search.sv.SearchSwedish',
     'tr': 'sphinx.search.tr.SearchTurkish',
     'zh': 'sphinx.search.zh.SearchChinese',
-}   # type: Dict[unicode, Any]
+}  # type: Dict[str, Any]
 
 
-class _JavaScriptIndex(object):
+class _JavaScriptIndex:
     """
     The search index as javascript file that calls a function
     on the documentation search object to register the index.
@@ -164,45 +160,43 @@ class _JavaScriptIndex(object):
     PREFIX = 'Search.setIndex('
     SUFFIX = ')'
 
-    def dumps(self, data):
-        # type: (Any) -> unicode
+    def dumps(self, data: Any) -> str:
         return self.PREFIX + jsdump.dumps(data) + self.SUFFIX
 
-    def loads(self, s):
-        # type: (str) -> Any
+    def loads(self, s: str) -> Any:
         data = s[len(self.PREFIX):-len(self.SUFFIX)]
         if not data or not s.startswith(self.PREFIX) or not \
            s.endswith(self.SUFFIX):
             raise ValueError('invalid data')
         return jsdump.loads(data)
 
-    def dump(self, data, f):
-        # type: (Any, IO) -> None
+    def dump(self, data: Any, f: IO) -> None:
         f.write(self.dumps(data))
 
-    def load(self, f):
-        # type: (IO) -> Any
+    def load(self, f: IO) -> Any:
         return self.loads(f.read())
 
 
 js_index = _JavaScriptIndex()
 
 
-class WordCollector(NodeVisitor):
+class WordCollector(nodes.NodeVisitor):
     """
     A special visitor that collects words for the `IndexBuilder`.
     """
 
-    def __init__(self, document, lang):
-        # type: (nodes.Node, SearchLanguage) -> None
-        NodeVisitor.__init__(self, document)
-        self.found_words = []           # type: List[unicode]
-        self.found_title_words = []     # type: List[unicode]
+    def __init__(self, document: nodes.document, lang: SearchLanguage) -> None:
+        super().__init__(document)
+        self.found_words = []           # type: List[str]
+        self.found_title_words = []     # type: List[str]
         self.lang = lang
 
-    def is_meta_keywords(self, node, nodetype):
-        # type: (nodes.Node, Type) -> bool
-        if isinstance(node, sphinx.addnodes.meta) and node.get('name') == 'keywords':
+    def is_meta_keywords(self, node: addnodes.meta, nodetype: Any = None) -> bool:
+        if nodetype is not None:
+            warnings.warn('"nodetype" argument for WordCollector.is_meta_keywords() '
+                          'is deprecated.', RemovedInSphinx40Warning)
+
+        if isinstance(node, addnodes.meta) and node.get('name') == 'keywords':
             meta_lang = node.get('lang')
             if meta_lang is None:  # lang not specified
                 return True
@@ -211,12 +205,10 @@ class WordCollector(NodeVisitor):
 
         return False
 
-    def dispatch_visit(self, node):
-        # type: (nodes.Node) -> None
-        nodetype = type(node)
-        if issubclass(nodetype, comment):
-            raise SkipNode
-        if issubclass(nodetype, raw):
+    def dispatch_visit(self, node: Node) -> None:
+        if isinstance(node, nodes.comment):
+            raise nodes.SkipNode
+        elif isinstance(node, nodes.raw):
             if 'html' in node.get('format', '').split():
                 # Some people might put content in raw HTML that should be searched,
                 # so we just amateurishly strip HTML tags and index the remaining
@@ -225,18 +217,18 @@ class WordCollector(NodeVisitor):
                 nodetext = re.sub(r'(?is)<script.*?</script>', '', nodetext)
                 nodetext = re.sub(r'<[^<]+?>', '', nodetext)
                 self.found_words.extend(self.lang.split(nodetext))
-            raise SkipNode
-        if issubclass(nodetype, Text):
+            raise nodes.SkipNode
+        elif isinstance(node, nodes.Text):
             self.found_words.extend(self.lang.split(node.astext()))
-        elif issubclass(nodetype, title):
+        elif isinstance(node, nodes.title):
             self.found_title_words.extend(self.lang.split(node.astext()))
-        elif self.is_meta_keywords(node, nodetype):
+        elif isinstance(node, addnodes.meta) and self.is_meta_keywords(node):
             keywords = node['content']
             keywords = [keyword.strip() for keyword in keywords.split(',')]
             self.found_words.extend(keywords)
 
 
-class IndexBuilder(object):
+class IndexBuilder:
     """
     Helper class that creates a searchindex based on the doctrees
     passed to the `feed` method.
@@ -244,24 +236,23 @@ class IndexBuilder(object):
     formats = {
         'jsdump':   jsdump,
         'pickle':   pickle
-    }   # type: Dict[unicode, Any]
+    }
 
-    def __init__(self, env, lang, options, scoring):
-        # type: (BuildEnvironment, unicode, Dict, unicode) -> None
+    def __init__(self, env: BuildEnvironment, lang: str, options: Dict, scoring: str) -> None:
         self.env = env
-        self._titles = {}           # type: Dict[unicode, unicode]
+        self._titles = {}           # type: Dict[str, str]
                                     # docname -> title
-        self._filenames = {}        # type: Dict[unicode, unicode]
+        self._filenames = {}        # type: Dict[str, str]
                                     # docname -> filename
-        self._mapping = {}          # type: Dict[unicode, Set[unicode]]
+        self._mapping = {}          # type: Dict[str, Set[str]]
                                     # stemmed word -> set(docname)
-        self._title_mapping = {}    # type: Dict[unicode, Set[unicode]]
+        self._title_mapping = {}    # type: Dict[str, Set[str]]
                                     # stemmed words in titles -> set(docname)
-        self._stem_cache = {}       # type: Dict[unicode, unicode]
+        self._stem_cache = {}       # type: Dict[str, str]
                                     # word -> stemmed word
-        self._objtypes = {}         # type: Dict[Tuple[unicode, unicode], int]
+        self._objtypes = {}         # type: Dict[Tuple[str, str], int]
                                     # objtype -> index
-        self._objnames = {}         # type: Dict[int, Tuple[unicode, unicode, unicode]]
+        self._objnames = {}         # type: Dict[int, Tuple[str, str, str]]
                                     # objtype index -> (domain, type, objname (localized))
         lang_class = languages.get(lang)    # type: Type[SearchLanguage]
                                             # add language-specific SearchLanguage instance
@@ -274,8 +265,7 @@ class IndexBuilder(object):
             self.lang = SearchEnglish(options)  # type: SearchLanguage
         elif isinstance(lang_class, str):
             module, classname = lang_class.rsplit('.', 1)
-            lang_class = getattr(__import__(module, None, None, [classname]),
-                                 classname)
+            lang_class = getattr(import_module(module), classname)
             self.lang = lang_class(options)
         else:
             # it's directly a class (e.g. added by app.add_search_language)
@@ -283,15 +273,14 @@ class IndexBuilder(object):
 
         if scoring:
             with open(scoring, 'rb') as fp:
-                self.js_scorer_code = fp.read().decode('utf-8')
+                self.js_scorer_code = fp.read().decode()
         else:
-            self.js_scorer_code = u''
+            self.js_scorer_code = ''
         self.js_splitter_code = splitter_code
 
-    def load(self, stream, format):
-        # type: (IO, Any) -> None
+    def load(self, stream: IO, format: Any) -> None:
         """Reconstruct from frozen data."""
-        if isinstance(format, string_types):
+        if isinstance(format, str):
             format = self.formats[format]
         frozen = format.load(stream)
         # if an old index is present, we treat it as not existing.
@@ -302,41 +291,39 @@ class IndexBuilder(object):
         self._filenames = dict(zip(index2fn, frozen['filenames']))
         self._titles = dict(zip(index2fn, frozen['titles']))
 
-        def load_terms(mapping):
-            # type: (Dict[unicode, Any]) -> Dict[unicode, Set[unicode]]
+        def load_terms(mapping: Dict[str, Any]) -> Dict[str, Set[str]]:
             rv = {}
-            for k, v in iteritems(mapping):
+            for k, v in mapping.items():
                 if isinstance(v, int):
-                    rv[k] = set([index2fn[v]])
+                    rv[k] = {index2fn[v]}
                 else:
-                    rv[k] = set(index2fn[i] for i in v)
+                    rv[k] = {index2fn[i] for i in v}
             return rv
 
         self._mapping = load_terms(frozen['terms'])
         self._title_mapping = load_terms(frozen['titleterms'])
         # no need to load keywords/objtypes
 
-    def dump(self, stream, format):
-        # type: (IO, Any) -> None
+    def dump(self, stream: IO, format: Any) -> None:
         """Dump the frozen index to a stream."""
-        if isinstance(format, string_types):
+        if isinstance(format, str):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
-    def get_objects(self, fn2index):
-        # type: (Dict[unicode, int]) -> Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]  # NOQA
-        rv = {}  # type: Dict[unicode, Dict[unicode, Tuple[int, int, int, unicode]]]
+    def get_objects(self, fn2index: Dict[str, int]
+                    ) -> Dict[str, Dict[str, Tuple[int, int, int, str]]]:
+        rv = {}  # type: Dict[str, Dict[str, Tuple[int, int, int, str]]]
         otypes = self._objtypes
         onames = self._objnames
-        for domainname, domain in sorted(iteritems(self.env.domains)):
+        for domainname, domain in sorted(self.env.domains.items()):
             for fullname, dispname, type, docname, anchor, prio in \
                     sorted(domain.get_objects()):
                 if docname not in fn2index:
                     continue
                 if prio < 0:
                     continue
-                fullname = htmlescape(fullname)
-                dispname = htmlescape(dispname)
+                fullname = html.escape(fullname)
+                dispname = html.escape(dispname)
                 prefix, name = rpartition(dispname, '.')
                 pdict = rv.setdefault(prefix, {})
                 try:
@@ -346,13 +333,13 @@ class IndexBuilder(object):
                     otypes[domainname, type] = typeindex
                     otype = domain.object_types.get(type)
                     if otype:
-                        # use unicode() to fire translation proxies
+                        # use str() to fire translation proxies
                         onames[typeindex] = (domainname, type,
-                                             text_type(domain.get_type_name(otype)))
+                                             str(domain.get_type_name(otype)))
                     else:
                         onames[typeindex] = (domainname, type, type)
                 if anchor == fullname:
-                    shortanchor = ''  # type: unicode
+                    shortanchor = ''
                 elif anchor == type + '-' + fullname:
                     shortanchor = '-'
                 else:
@@ -360,11 +347,10 @@ class IndexBuilder(object):
                 pdict[name] = (fn2index[docname], typeindex, prio, shortanchor)
         return rv
 
-    def get_terms(self, fn2index):
-        # type: (Dict) -> Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
-        rvs = {}, {}  # type: Tuple[Dict[unicode, List[unicode]], Dict[unicode, List[unicode]]]
+    def get_terms(self, fn2index: Dict) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]:
+        rvs = {}, {}  # type: Tuple[Dict[str, List[str]], Dict[str, List[str]]]
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
-            for k, v in iteritems(mapping):
+            for k, v in mapping.items():
                 if len(v) == 1:
                     fn, = v
                     if fn in fn2index:
@@ -373,28 +359,24 @@ class IndexBuilder(object):
                     rv[k] = sorted([fn2index[fn] for fn in v if fn in fn2index])
         return rvs
 
-    def freeze(self):
-        # type: () -> Dict[unicode, Any]
+    def freeze(self) -> Dict[str, Any]:
         """Create a usable data structure for serializing."""
         docnames, titles = zip(*sorted(self._titles.items()))
         filenames = [self._filenames.get(docname) for docname in docnames]
-        fn2index = dict((f, i) for (i, f) in enumerate(docnames))
+        fn2index = {f: i for (i, f) in enumerate(docnames)}
         terms, title_terms = self.get_terms(fn2index)
 
         objects = self.get_objects(fn2index)  # populates _objtypes
-        objtypes = dict((v, k[0] + ':' + k[1])
-                        for (k, v) in iteritems(self._objtypes))
+        objtypes = {v: k[0] + ':' + k[1] for (k, v) in self._objtypes.items()}
         objnames = self._objnames
         return dict(docnames=docnames, filenames=filenames, titles=titles, terms=terms,
                     objects=objects, objtypes=objtypes, objnames=objnames,
                     titleterms=title_terms, envversion=self.env.version)
 
-    def label(self):
-        # type: () -> unicode
+    def label(self) -> str:
         return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
 
-    def prune(self, docnames):
-        # type: (Iterable[unicode]) -> None
+    def prune(self, docnames: Iterable[str]) -> None:
         """Remove data for all docnames not in the list."""
         new_titles = {}
         new_filenames = {}
@@ -404,13 +386,12 @@ class IndexBuilder(object):
                 new_filenames[docname] = self._filenames[docname]
         self._titles = new_titles
         self._filenames = new_filenames
-        for wordnames in itervalues(self._mapping):
+        for wordnames in self._mapping.values():
             wordnames.intersection_update(docnames)
-        for wordnames in itervalues(self._title_mapping):
+        for wordnames in self._title_mapping.values():
             wordnames.intersection_update(docnames)
 
-    def feed(self, docname, filename, title, doctree):
-        # type: (unicode, unicode, unicode, nodes.Node) -> None
+    def feed(self, docname: str, filename: str, title: str, doctree: nodes.document) -> None:
         """Feed a doctree to the index."""
         self._titles[docname] = title
         self._filenames[docname] = filename
@@ -419,8 +400,7 @@ class IndexBuilder(object):
         doctree.walk(visitor)
 
         # memoize self.lang.stem
-        def stem(word):
-            # type: (unicode) -> unicode
+        def stem(word: str) -> str:
             try:
                 return self._stem_cache[word]
             except KeyError:
@@ -444,22 +424,17 @@ class IndexBuilder(object):
             if _filter(stemmed_word) and not already_indexed:
                 self._mapping.setdefault(stemmed_word, set()).add(docname)
 
-    def context_for_searchtool(self):
-        # type: () -> Dict[unicode, Any]
-        return dict(
-            search_language_stemming_code = self.lang.js_stemmer_code,
-            search_language_stop_words = jsdump.dumps(sorted(self.lang.stopwords)),
-            search_scorer_tool = self.js_scorer_code,
-            search_word_splitter_code = self.js_splitter_code,
-        )
+    def context_for_searchtool(self) -> Dict[str, Any]:
+        return {
+            'search_language_stemming_code': self.lang.js_stemmer_code,
+            'search_language_stop_words': jsdump.dumps(sorted(self.lang.stopwords)),
+            'search_scorer_tool': self.js_scorer_code,
+            'search_word_splitter_code': self.js_splitter_code,
+        }
 
-    def get_js_stemmer_rawcode(self):
-        # type: () -> unicode
+    def get_js_stemmer_rawcode(self) -> str:
         if self.lang.js_stemmer_rawcode:
-            return path.join(
-                sphinx.package_dir, 'search',
-                'non-minified-js',
-                self.lang.js_stemmer_rawcode
-            )
+            return path.join(package_dir, 'search', 'non-minified-js',
+                             self.lang.js_stemmer_rawcode)
         else:
             return None
