@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.setup_command
     ~~~~~~~~~~~~~~~~~~~~
@@ -8,17 +7,15 @@
 
     :author: Sebastian Wiesner
     :contact: basti.wiesner@gmx.net
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
 import os
 import sys
 from distutils.cmd import Command
 from distutils.errors import DistutilsOptionError, DistutilsExecError
-
-from six import StringIO, string_types
+from io import StringIO
 
 from sphinx.application import Sphinx
 from sphinx.cmd.build import handle_exception
@@ -28,7 +25,7 @@ from sphinx.util.osutil import abspath
 
 if False:
     # For type annotation
-    from typing import Any, Dict, List, Tuple  # NOQA
+    from typing import Any, Dict  # NOQA
 
 
 class BuildDoc(Command):
@@ -88,6 +85,7 @@ class BuildDoc(Command):
         ('copyright', None, 'The copyright string'),
         ('pdb', None, 'Start pdb on exception'),
         ('nitpicky', 'n', 'nit-picky mode, warn about all missing references'),
+        ('keep-going', None, 'With -W, keep going when getting warnings'),
     ]
     boolean_options = ['fresh-env', 'all-files', 'warning-is-error',
                        'link-index', 'nitpicky']
@@ -96,22 +94,23 @@ class BuildDoc(Command):
         # type: () -> None
         self.fresh_env = self.all_files = False
         self.pdb = False
-        self.source_dir = self.build_dir = None  # type: unicode
+        self.source_dir = self.build_dir = None  # type: str
         self.builder = 'html'
         self.warning_is_error = False
         self.project = ''
         self.version = ''
         self.release = ''
         self.today = ''
-        self.config_dir = None  # type: unicode
+        self.config_dir = None  # type: str
         self.link_index = False
         self.copyright = ''
         self.verbosity = 0
         self.traceback = False
         self.nitpicky = False
+        self.keep_going = False
 
     def _guess_source_dir(self):
-        # type: () -> unicode
+        # type: () -> str
         for guess in ('doc', 'docs'):
             if not os.path.isdir(guess):
                 continue
@@ -124,12 +123,12 @@ class BuildDoc(Command):
     # unicode, causing finalize_options to fail if invoked again. Workaround
     # for https://bugs.python.org/issue19570
     def _ensure_stringlike(self, option, what, default=None):
-        # type: (unicode, unicode, Any) -> Any
+        # type: (str, str, Any) -> Any
         val = getattr(self, option)
         if val is None:
             setattr(self, option, default)
             return default
-        elif not isinstance(val, string_types):
+        elif not isinstance(val, str):
             raise DistutilsOptionError("'%s' must be a %s (got `%s`)"
                                        % (option, what, val))
         return val
@@ -155,7 +154,7 @@ class BuildDoc(Command):
 
         self.builder_target_dirs = [
             (builder, os.path.join(self.build_dir, builder))
-            for builder in self.builder]  # type: List[Tuple[str, unicode]]
+            for builder in self.builder]
 
     def run(self):
         # type: () -> None
@@ -165,7 +164,7 @@ class BuildDoc(Command):
             status_stream = StringIO()
         else:
             status_stream = sys.stdout  # type: ignore
-        confoverrides = {}  # type: Dict[unicode, Any]
+        confoverrides = {}  # type: Dict[str, Any]
         if self.project:
             confoverrides['project'] = self.project
         if self.version:
@@ -189,7 +188,8 @@ class BuildDoc(Command):
                                  builder_target_dir, self.doctree_dir,
                                  builder, confoverrides, status_stream,
                                  freshenv=self.fresh_env,
-                                 warningiserror=self.warning_is_error)
+                                 warningiserror=self.warning_is_error,
+                                 keep_going=self.keep_going)
                     app.build(force_all=self.all_files)
                     if app.statuscode:
                         raise DistutilsExecError(
