@@ -1,17 +1,18 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.transforms
     ~~~~~~~~~~~~~~~~~
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+from typing import Any, Dict, Generator, List, Tuple
 
 from docutils import nodes
+from docutils.nodes import Element, Node, Text
 from docutils.transforms import Transform, Transformer
 from docutils.transforms.parts import ContentsFilter
 from docutils.transforms.universal import SmartQuotes
@@ -19,60 +20,51 @@ from docutils.utils import normalize_language_tag
 from docutils.utils.smartquotes import smartchars
 
 from sphinx import addnodes
-from sphinx.locale import _
+from sphinx.config import Config
+from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
+from sphinx.locale import _, __
 from sphinx.util import logging
 from sphinx.util.docutils import new_document
 from sphinx.util.i18n import format_date
-from sphinx.util.nodes import apply_source_workaround, is_smartquotable
+from sphinx.util.nodes import NodeMatcher, apply_source_workaround, is_smartquotable
 
 if False:
     # For type annotation
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.config import Config  # NOQA
-    from sphinx.domain.std import StandardDomain  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.application import Sphinx
+    from sphinx.domain.std import StandardDomain
+    from sphinx.environment import BuildEnvironment
 
 
 logger = logging.getLogger(__name__)
 
-default_substitutions = set([
+default_substitutions = {
     'version',
     'release',
     'today',
-])
+}
 
 
 class SphinxTransform(Transform):
-    """
-    A base class of Transforms.
+    """A base class of Transforms.
 
     Compared with ``docutils.transforms.Transform``, this class improves accessibility to
     Sphinx APIs.
-
-    The subclasses can access following objects and functions:
-
-    self.app
-        The application object (:class:`sphinx.application.Sphinx`)
-    self.config
-        The config object (:class:`sphinx.config.Config`)
-    self.env
-        The environment object (:class:`sphinx.environment.BuildEnvironment`)
     """
 
     @property
-    def app(self):
-        # type: () -> Sphinx
-        return self.document.settings.env.app
+    def app(self) -> "Sphinx":
+        """Reference to the :class:`.Sphinx` object."""
+        return self.env.app
 
     @property
-    def env(self):
-        # type: () -> BuildEnvironment
+    def env(self) -> "BuildEnvironment":
+        """Reference to the :class:`.BuildEnvironment` object."""
         return self.document.settings.env
 
     @property
-    def config(self):
-        # type: () -> Config
-        return self.document.settings.env.config
+    def config(self) -> Config:
+        """Reference to the :class:`.Config` object."""
+        return self.env.config
 
 
 class SphinxTransformer(Transformer):
@@ -80,20 +72,18 @@ class SphinxTransformer(Transformer):
     A transformer for Sphinx.
     """
 
-    document = None  # type: nodes.Node
+    document = None  # type: nodes.document
     env = None  # type: BuildEnvironment
 
-    def set_environment(self, env):
-        # type: (BuildEnvironment) -> None
+    def set_environment(self, env: "BuildEnvironment") -> None:
         self.env = env
 
-    def apply_transforms(self):
-        # type: () -> None
+    def apply_transforms(self) -> None:
         if isinstance(self.document, nodes.document):
             if not hasattr(self.document.settings, 'env') and self.env:
                 self.document.settings.env = self.env
 
-            Transformer.apply_transforms(self)
+            super().apply_transforms()
         else:
             # wrap the target node by document node during transforming
             try:
@@ -102,7 +92,7 @@ class SphinxTransformer(Transformer):
                     document.settings.env = self.env
                 document += self.document
                 self.document = document
-                Transformer.apply_transforms(self)
+                super().apply_transforms()
             finally:
                 self.document = self.document[0]
 
@@ -114,8 +104,7 @@ class DefaultSubstitutions(SphinxTransform):
     # run before the default Substitutions
     default_priority = 210
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         # only handle those not otherwise defined in the document
         to_handle = default_substitutions - set(self.document.substitution_defs)
         for ref in self.document.traverse(nodes.substitution_reference):
@@ -124,7 +113,7 @@ class DefaultSubstitutions(SphinxTransform):
                 text = self.config[refname]
                 if refname == 'today' and not text:
                     # special handling: can also specify a strftime format
-                    text = format_date(self.config.today_fmt or _('%b %d, %Y'),  # type: ignore
+                    text = format_date(self.config.today_fmt or _('%b %d, %Y'),
                                        language=self.config.language)
                 ref.replace_self(nodes.Text(text, text))
 
@@ -138,8 +127,7 @@ class MoveModuleTargets(SphinxTransform):
     """
     default_priority = 210
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(nodes.target):
             if not node['ids']:
                 continue
@@ -157,8 +145,7 @@ class HandleCodeBlocks(SphinxTransform):
     """
     default_priority = 210
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         # move doctest blocks out of blockquotes
         for node in self.document.traverse(nodes.block_quote):
             if all(isinstance(child, nodes.doctest_block) for child
@@ -182,12 +169,13 @@ class AutoNumbering(SphinxTransform):
     """
     default_priority = 210
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         domain = self.env.get_domain('std')  # type: StandardDomain
 
         for node in self.document.traverse(nodes.Element):
-            if domain.is_enumerable_node(node) and domain.get_numfig_title(node) is not None:
+            if (domain.is_enumerable_node(node) and
+                    domain.get_numfig_title(node) is not None and
+                    node['ids'] == []):
                 self.document.note_implicit_target(node)
 
 
@@ -197,38 +185,12 @@ class SortIds(SphinxTransform):
     """
     default_priority = 261
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(nodes.section):
             if len(node['ids']) > 1 and node['ids'][0].startswith('id'):
                 node['ids'] = node['ids'][1:] + [node['ids'][0]]
 
 
-class CitationReferences(SphinxTransform):
-    """
-    Replace citation references by pending_xref nodes before the default
-    docutils transform tries to resolve them.
-    """
-    default_priority = 619
-
-    def apply(self):
-        # type: () -> None
-        # mark citation labels as not smartquoted
-        for citnode in self.document.traverse(nodes.citation):
-            citnode[0]['support_smartquotes'] = False
-
-        for citnode in self.document.traverse(nodes.citation_reference):
-            cittext = citnode.astext()
-            refnode = addnodes.pending_xref(cittext, refdomain='std', reftype='citation',
-                                            reftarget=cittext, refwarn=True,
-                                            support_smartquotes=False,
-                                            ids=citnode["ids"])
-            refnode.source = citnode.source or citnode.parent.source
-            refnode.line = citnode.line or citnode.parent.line
-            refnode += nodes.Text('[' + cittext + ']')
-            citnode.parent.replace(citnode, refnode)
-
-
 TRANSLATABLE_NODES = {
     'literal-block': nodes.literal_block,
     'doctest-block': nodes.doctest_block,
@@ -244,11 +206,10 @@ class ApplySourceWorkaround(SphinxTransform):
     """
     default_priority = 10
 
-    def apply(self):
-        # type: () -> None
-        for n in self.document.traverse():
-            if isinstance(n, (nodes.TextElement, nodes.image)):
-                apply_source_workaround(n)
+    def apply(self, **kwargs: Any) -> None:
+        for node in self.document.traverse():  # type: Node
+            if isinstance(node, (nodes.TextElement, nodes.image)):
+                apply_source_workaround(node)
 
 
 class AutoIndexUpgrader(SphinxTransform):
@@ -257,12 +218,11 @@ class AutoIndexUpgrader(SphinxTransform):
     """
     default_priority = 210
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.index):
             if 'entries' in node and any(len(entry) == 4 for entry in node['entries']):
-                msg = ('4 column based index found. '
-                       'It might be a bug of extensions you use: %r' % node['entries'])
+                msg = __('4 column based index found. '
+                         'It might be a bug of extensions you use: %r') % node['entries']
                 logger.warning(msg, location=node)
                 for i, entry in enumerate(node['entries']):
                     if len(entry) == 4:
@@ -275,18 +235,16 @@ class ExtraTranslatableNodes(SphinxTransform):
     """
     default_priority = 10
 
-    def apply(self):
-        # type: () -> None
+    def apply(self, **kwargs: Any) -> None:
         targets = self.config.gettext_additional_targets
         target_nodes = [v for k, v in TRANSLATABLE_NODES.items() if k in targets]
         if not target_nodes:
             return
 
-        def is_translatable_node(node):
-            # type: (nodes.Node) -> bool
+        def is_translatable_node(node: Node) -> bool:
             return isinstance(node, tuple(target_nodes))
 
-        for node in self.document.traverse(is_translatable_node):
+        for node in self.document.traverse(is_translatable_node):  # type: Element
             node['translatable'] = True
 
 
@@ -296,30 +254,50 @@ class UnreferencedFootnotesDetector(SphinxTransform):
     """
     default_priority = 200
 
-    def apply(self):
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.footnotes:
             if node['names'] == []:
                 # footnote having duplicated number.  It is already warned at parser.
                 pass
             elif node['names'][0] not in self.document.footnote_refs:
-                logger.warning('Footnote [%s] is not referenced.', node['names'][0],
+                logger.warning(__('Footnote [%s] is not referenced.'), node['names'][0],
                                type='ref', subtype='footnote',
                                location=node)
 
         for node in self.document.autofootnotes:
             if not any(ref['auto'] == node['auto'] for ref in self.document.autofootnote_refs):
-                logger.warning('Footnote [#] is not referenced.',
+                logger.warning(__('Footnote [#] is not referenced.'),
                                type='ref', subtype='footnote',
                                location=node)
 
 
+class DoctestTransform(SphinxTransform):
+    """Set "doctest" style to each doctest_block node"""
+    default_priority = 500
+
+    def apply(self, **kwargs: Any) -> None:
+        for node in self.document.traverse(nodes.doctest_block):
+            node['classes'].append('doctest')
+
+
+class FigureAligner(SphinxTransform):
+    """
+    Align figures to center by default.
+    """
+    default_priority = 700
+
+    def apply(self, **kwargs: Any) -> None:
+        matcher = NodeMatcher(nodes.table, nodes.figure)
+        for node in self.document.traverse(matcher):  # type: Element
+            node.setdefault('align', 'default')
+
+
 class FilterSystemMessages(SphinxTransform):
     """Filter system messages from a doctree."""
     default_priority = 999
 
-    def apply(self):
-        # type: () -> None
-        filterlevel = self.config.keep_warnings and 2 or 5
+    def apply(self, **kwargs: Any) -> None:
+        filterlevel = 2 if self.config.keep_warnings else 5
         for node in self.document.traverse(nodes.system_message):
             if node['level'] < filterlevel:
                 logger.debug('%s [filtered system message]', node.astext())
@@ -331,14 +309,9 @@ class SphinxContentsFilter(ContentsFilter):
     Used with BuildEnvironment.add_toc_from() to discard cross-file links
     within table-of-contents link nodes.
     """
-    def visit_pending_xref(self, node):
-        # type: (nodes.Node) -> None
-        text = node.astext()
-        self.parent.append(nodes.literal(text, text))
-        raise nodes.SkipNode
+    visit_pending_xref = ContentsFilter.ignore_node_but_process_children
 
-    def visit_image(self, node):
-        # type: (nodes.Node) -> None
+    def visit_image(self, node: nodes.image) -> None:
         raise nodes.SkipNode
 
 
@@ -348,15 +321,18 @@ class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
 
     refs: sphinx.parsers.RSTParser
     """
-    def apply(self):
-        # type: () -> None
+    default_priority = 750
+
+    def apply(self, **kwargs: Any) -> None:
         if not self.is_available():
             return
 
-        SmartQuotes.apply(self)
+        # override default settings with :confval:`smartquotes_action`
+        self.smartquotes_action = self.config.smartquotes_action
 
-    def is_available(self):
-        # type: () -> bool
+        super().apply()
+
+    def is_available(self) -> bool:
         builders = self.config.smartquotes_excludes.get('builders', [])
         languages = self.config.smartquotes_excludes.get('languages', [])
 
@@ -381,16 +357,7 @@ class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
         else:
             return False
 
-    @property
-    def smartquotes_action(self):
-        # type: () -> unicode
-        """A smartquotes_action setting for SmartQuotes.
-
-        Users can change this setting through :confval:`smartquotes_action`.
-        """
-        return self.config.smartquotes_action
-
-    def get_tokens(self, txtnodes):
+    def get_tokens(self, txtnodes: List[Text]) -> Generator[Tuple[str, str], None, None]:
         # A generator that yields ``(texttype, nodetext)`` tuples for a list
         # of "Text" nodes (interface to ``smartquotes.educate_tokens()``).
 
@@ -401,11 +368,19 @@ class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
             yield (texttype[notsmartquotable], txtnode.astext())
 
 
+class DoctreeReadEvent(SphinxTransform):
+    """Emit :event:`doctree-read` event."""
+    default_priority = 880
+
+    def apply(self, **kwargs: Any) -> None:
+        self.app.emit('doctree-read', self.document)
+
+
 class ManpageLink(SphinxTransform):
     """Find manpage section numbers and names"""
     default_priority = 999
 
-    def apply(self):
+    def apply(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.manpage):
             manpage = ' '.join([str(x) for x in node.children
                                 if isinstance(x, nodes.Text)])
@@ -417,3 +392,39 @@ class ManpageLink(SphinxTransform):
             if r:
                 info = r.groupdict()
             node.attributes.update(info)
+
+
+from sphinx.domains.citation import (  # NOQA
+    CitationDefinitionTransform, CitationReferenceTransform
+)
+
+deprecated_alias('sphinx.transforms',
+                 {
+                     'CitationReferences': CitationReferenceTransform,
+                     'SmartQuotesSkipper': CitationDefinitionTransform,
+                 },
+                 RemovedInSphinx40Warning)
+
+
+def setup(app: "Sphinx") -> Dict[str, Any]:
+    app.add_transform(ApplySourceWorkaround)
+    app.add_transform(ExtraTranslatableNodes)
+    app.add_transform(DefaultSubstitutions)
+    app.add_transform(MoveModuleTargets)
+    app.add_transform(HandleCodeBlocks)
+    app.add_transform(SortIds)
+    app.add_transform(DoctestTransform)
+    app.add_transform(FigureAligner)
+    app.add_transform(AutoNumbering)
+    app.add_transform(AutoIndexUpgrader)
+    app.add_transform(FilterSystemMessages)
+    app.add_transform(UnreferencedFootnotesDetector)
+    app.add_transform(SphinxSmartQuotes)
+    app.add_transform(DoctreeReadEvent)
+    app.add_transform(ManpageLink)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
