@@ -1,16 +1,15 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util
     ~~~~~~~~~~~
 
     Utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
 
 import fnmatch
+import functools
 import os
 import posixpath
 import re
@@ -23,20 +22,23 @@ from codecs import BOM_UTF8
 from collections import deque
 from datetime import datetime
 from hashlib import md5
+from importlib import import_module
 from os import path
 from time import mktime, strptime
+from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Set, Tuple
+from urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
 
 from docutils.utils import relative_path
-from six import text_type, binary_type, itervalues
-from six.moves import range
-from six.moves.urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
 
-from sphinx.deprecation import RemovedInSphinx30Warning
-from sphinx.errors import PycodeError, SphinxParallelError, ExtensionError
+from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
+from sphinx.errors import (
+    PycodeError, SphinxParallelError, ExtensionError, FiletypeNotFoundError
+)
+from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.console import strip_colors, colorize, bold, term_width_line  # type: ignore
 from sphinx.util.fileutil import copy_asset_file
-from sphinx.util.osutil import fs_encoding
+from sphinx.util.typing import PathMatcher
 from sphinx.util import smartypants  # noqa
 
 # import other utilities; partly for backwards compatibility, so don't
@@ -49,10 +51,12 @@ from sphinx.util.nodes import (   # noqa
     caption_ref_re)
 from sphinx.util.matching import patfilter  # noqa
 
+
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Sequence, Set, Tuple, Union  # NOQA
-
+    from typing import Type  # for python3.5.1
+    from sphinx.application import Sphinx
+    from sphinx.builders import Builder
 
 logger = logging.getLogger(__name__)
 
@@ -63,23 +67,19 @@ url_re = re.compile(r'(?P<schema>.+)://.*')     # type: Pattern
 
 # High-level utility functions.
 
-def docname_join(basedocname, docname):
-    # type: (unicode, unicode) -> unicode
+def docname_join(basedocname: str, docname: str) -> str:
     return posixpath.normpath(
         posixpath.join('/' + basedocname, '..', docname))[1:]
 
 
-def path_stabilize(filepath):
-    # type: (unicode) -> unicode
+def path_stabilize(filepath: str) -> str:
     "normalize path separater and unicode string"
     newpath = filepath.replace(os.path.sep, SEP)
-    if isinstance(newpath, text_type):
-        newpath = unicodedata.normalize('NFC', newpath)
-    return newpath
+    return unicodedata.normalize('NFC', newpath)
 
 
-def get_matching_files(dirname, exclude_matchers=()):
-    # type: (unicode, Tuple[Callable[[unicode], bool], ...]) -> Iterable[unicode]
+def get_matching_files(dirname: str,
+                       exclude_matchers: Tuple[PathMatcher, ...] = ()) -> Iterable[str]:  # NOQA
     """Get all file names in a directory, recursively.
 
     Exclude files and dirs matching some matcher in *exclude_matchers*.
@@ -88,13 +88,13 @@ def get_matching_files(dirname, exclude_matchers=()):
     dirname = path.normpath(path.abspath(dirname))
     dirlen = len(dirname) + 1    # exclude final os.path.sep
 
-    for root, dirs, files in walk(dirname, followlinks=True):
+    for root, dirs, files in os.walk(dirname, followlinks=True):
         relativeroot = root[dirlen:]
 
         qdirs = enumerate(path_stabilize(path.join(relativeroot, dn))
-                          for dn in dirs)  # type: Iterable[Tuple[int, unicode]]
+                          for dn in dirs)  # type: Iterable[Tuple[int, str]]
         qfiles = enumerate(path_stabilize(path.join(relativeroot, fn))
-                           for fn in files)  # type: Iterable[Tuple[int, unicode]]
+                           for fn in files)  # type: Iterable[Tuple[int, str]]
         for matcher in exclude_matchers:
             qdirs = [entry for entry in qdirs if not matcher(entry[1])]
             qfiles = [entry for entry in qfiles if not matcher(entry[1])]
@@ -105,13 +105,15 @@ def get_matching_files(dirname, exclude_matchers=()):
             yield filename
 
 
-def get_matching_docs(dirname, suffixes, exclude_matchers=()):
-    # type: (unicode, List[unicode], Tuple[Callable[[unicode], bool], ...]) -> Iterable[unicode]  # NOQA
+def get_matching_docs(dirname: str, suffixes: List[str],
+                      exclude_matchers: Tuple[PathMatcher, ...] = ()) -> Iterable[str]:
     """Get all file names (without suffixes) matching a suffix in a directory,
     recursively.
 
     Exclude files and dirs matching a pattern in *exclude_patterns*.
     """
+    warnings.warn('get_matching_docs() is now deprecated. Use get_matching_files() instead.',
+                  RemovedInSphinx40Warning)
     suffixpatterns = ['*' + s for s in suffixes]
     for filename in get_matching_files(dirname, exclude_matchers):
         for suffixpattern in suffixpatterns:
@@ -120,18 +122,25 @@ def get_matching_docs(dirname, suffixes, exclude_matchers=()):
                 break
 
 
+def get_filetype(source_suffix: Dict[str, str], filename: str) -> str:
+    for suffix, filetype in source_suffix.items():
+        if filename.endswith(suffix):
+            # If default filetype (None), considered as restructuredtext.
+            return filetype or 'restructuredtext'
+    else:
+        raise FiletypeNotFoundError
+
+
 class FilenameUniqDict(dict):
     """
     A dictionary that automatically generates unique names for its keys,
     interpreted as filenames, and keeps track of a set of docnames they
     appear in.  Used for images and downloadable files in the environment.
     """
-    def __init__(self):
-        # type: () -> None
-        self._existing = set()  # type: Set[unicode]
+    def __init__(self) -> None:
+        self._existing = set()  # type: Set[str]
 
-    def add_file(self, docname, newfile):
-        # type: (unicode, unicode) -> unicode
+    def add_file(self, docname: str, newfile: str) -> str:
         if newfile in self:
             self[newfile][0].add(docname)
             return self[newfile][1]
@@ -141,30 +150,26 @@ class FilenameUniqDict(dict):
         while uniquename in self._existing:
             i += 1
             uniquename = '%s%s%s' % (base, i, ext)
-        self[newfile] = (set([docname]), uniquename)
+        self[newfile] = ({docname}, uniquename)
         self._existing.add(uniquename)
         return uniquename
 
-    def purge_doc(self, docname):
-        # type: (unicode) -> None
+    def purge_doc(self, docname: str) -> None:
         for filename, (docs, unique) in list(self.items()):
             docs.discard(docname)
             if not docs:
                 del self[filename]
                 self._existing.discard(unique)
 
-    def merge_other(self, docnames, other):
-        # type: (Set[unicode], Dict[unicode, Tuple[Set[unicode], Any]]) -> None
+    def merge_other(self, docnames: Set[str], other: Dict[str, Tuple[Set[str], Any]]) -> None:
         for filename, (docs, unique) in other.items():
             for doc in docs & set(docnames):
                 self.add_file(doc, filename)
 
-    def __getstate__(self):
-        # type: () -> Set[unicode]
+    def __getstate__(self) -> Set[str]:
         return self._existing
 
-    def __setstate__(self, state):
-        # type: (Set[unicode]) -> None
+    def __setstate__(self, state: Set[str]) -> None:
         self._existing = state
 
 
@@ -175,33 +180,29 @@ class DownloadFiles(dict):
                    Hence don't hack this directly.
     """
 
-    def add_file(self, docname, filename):
-        # type: (unicode, unicode) -> None
+    def add_file(self, docname: str, filename: str) -> str:
         if filename not in self:
-            digest = md5(filename.encode('utf-8')).hexdigest()
+            digest = md5(filename.encode()).hexdigest()
             dest = '%s/%s' % (digest, os.path.basename(filename))
             self[filename] = (set(), dest)
 
         self[filename][0].add(docname)
         return self[filename][1]
 
-    def purge_doc(self, docname):
-        # type: (unicode) -> None
+    def purge_doc(self, docname: str) -> None:
         for filename, (docs, dest) in list(self.items()):
             docs.discard(docname)
             if not docs:
                 del self[filename]
 
-    def merge_other(self, docnames, other):
-        # type: (Set[unicode], Dict[unicode, Tuple[Set[unicode], Any]]) -> None
+    def merge_other(self, docnames: Set[str], other: Dict[str, Tuple[Set[str], Any]]) -> None:
         for filename, (docs, dest) in other.items():
             for docname in docs & set(docnames):
                 self.add_file(docname, filename)
 
 
-def copy_static_entry(source, targetdir, builder, context={},
-                      exclude_matchers=(), level=0):
-    # type: (unicode, unicode, Any, Dict, Tuple[Callable, ...], int) -> None
+def copy_static_entry(source: str, targetdir: str, builder: "Builder", context: Dict = {},
+                      exclude_matchers: Tuple[PathMatcher, ...] = (), level: int = 0) -> None:
     """[DEPRECATED] Copy a HTML builder static_path entry from source to targetdir.
 
     Handles all possible cases of files, directories and subdirectories.
@@ -217,8 +218,7 @@ def copy_static_entry(source, targetdir, builder, context={},
     if path.isfile(source):
         copy_asset_file(source, targetdir, context, builder.templates)
     elif path.isdir(source):
-        if not path.isdir(targetdir):
-            os.mkdir(targetdir)
+        ensuredir(targetdir)
         for entry in os.listdir(source):
             if entry.startswith('.'):
                 continue
@@ -241,8 +241,7 @@ _DEBUG_HEADER = '''\
 '''
 
 
-def save_traceback(app):
-    # type: (Any) -> unicode
+def save_traceback(app: "Sphinx") -> str:
     """Save the current exception's traceback in a temporary file."""
     import sphinx
     import jinja2
@@ -257,7 +256,7 @@ def save_traceback(app):
     last_msgs = ''
     if app is not None:
         last_msgs = '\n'.join(
-            '#   %s' % strip_colors(force_decode(s, 'utf-8')).strip()  # type: ignore
+            '#   %s' % strip_colors(s).strip()
             for s in app.messagelog)
     os.write(fd, (_DEBUG_HEADER %
                   (sphinx.__display_version__,
@@ -265,33 +264,30 @@ def save_traceback(app):
                    platform.python_implementation(),
                    docutils.__version__, docutils.__version_details__,
                    jinja2.__version__,  # type: ignore
-                   last_msgs)).encode('utf-8'))
+                   last_msgs)).encode())
     if app is not None:
-        for ext in itervalues(app.extensions):
+        for ext in app.extensions.values():
             modfile = getattr(ext.module, '__file__', 'unknown')
-            if isinstance(modfile, bytes):
-                modfile = modfile.decode(fs_encoding, 'replace')
             if ext.version != 'builtin':
                 os.write(fd, ('#   %s (%s) from %s\n' %
-                              (ext.name, ext.version, modfile)).encode('utf-8'))
-    os.write(fd, exc_format.encode('utf-8'))
+                              (ext.name, ext.version, modfile)).encode())
+    os.write(fd, exc_format.encode())
     os.close(fd)
     return path
 
 
-def get_module_source(modname):
-    # type: (str) -> Tuple[unicode, unicode]
+def get_module_source(modname: str) -> Tuple[str, str]:
     """Try to find the source code for a module.
 
     Can return ('file', 'filename') in which case the source is in the given
     file, or ('string', 'source') which which case the source is the string.
     """
-    if modname not in sys.modules:
-        try:
-            __import__(modname)
-        except Exception as err:
-            raise PycodeError('error importing %r' % modname, err)
-    mod = sys.modules[modname]
+    warnings.warn('get_module_source() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
+    try:
+        mod = import_module(modname)
+    except Exception as err:
+        raise PycodeError('error importing %r' % modname, err)
     filename = getattr(mod, '__file__', None)
     loader = getattr(mod, '__loader__', None)
     if loader and getattr(loader, 'get_filename', None):
@@ -327,14 +323,12 @@ def get_module_source(modname):
     return 'file', filename
 
 
-def get_full_modname(modname, attribute):
-    # type: (str, unicode) -> unicode
+def get_full_modname(modname: str, attribute: str) -> str:
     if modname is None:
         # Prevents a TypeError: if the last getattr() call will return None
         # then it's better to return it directly
         return None
-    __import__(modname)
-    module = sys.modules[modname]
+    module = import_module(modname)
 
     # Allow an attribute to have multiple parts and incidentially allow
     # repeated .s in the attribute.
@@ -350,19 +344,18 @@ def get_full_modname(modname, attribute):
 _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 
 
-def detect_encoding(readline):
-    # type: (Callable) -> unicode
+def detect_encoding(readline: Callable[[], bytes]) -> str:
     """Like tokenize.detect_encoding() from Py3k, but a bit simplified."""
+    warnings.warn('sphinx.util.detect_encoding() is deprecated',
+                  RemovedInSphinx40Warning)
 
-    def read_or_stop():
-        # type: () -> unicode
+    def read_or_stop() -> bytes:
         try:
             return readline()
         except StopIteration:
             return None
 
-    def get_normal_name(orig_enc):
-        # type: (str) -> str
+    def get_normal_name(orig_enc: str) -> str:
         """Imitates get_normal_name in tokenizer.c."""
         # Only care about the first 12 characters.
         enc = orig_enc[:12].lower().replace('_', '-')
@@ -373,8 +366,7 @@ def detect_encoding(readline):
             return 'iso-8859-1'
         return orig_enc
 
-    def find_cookie(line):
-        # type: (unicode) -> unicode
+    def find_cookie(line: bytes) -> str:
         try:
             line_string = line.decode('ascii')
         except UnicodeDecodeError:
@@ -404,32 +396,48 @@ def detect_encoding(readline):
     return default
 
 
+class UnicodeDecodeErrorHandler:
+    """Custom error handler for open() that warns and replaces."""
+
+    def __init__(self, docname: str) -> None:
+        self.docname = docname
+
+    def __call__(self, error: UnicodeDecodeError) -> Tuple[str, int]:
+        linestart = error.object.rfind(b'\n', 0, error.start)
+        lineend = error.object.find(b'\n', error.start)
+        if lineend == -1:
+            lineend = len(error.object)
+        lineno = error.object.count(b'\n', 0, error.start) + 1
+        logger.warning(__('undecodable source characters, replacing with "?": %r'),
+                       (error.object[linestart + 1:error.start] + b'>>>' +
+                        error.object[error.start:error.end] + b'<<<' +
+                        error.object[error.end:lineend]),
+                       location=(self.docname, lineno))
+        return ('?', error.end)
+
+
 # Low-level utility functions and classes.
 
-class Tee(object):
+class Tee:
     """
     File-like object writing to two streams.
     """
-    def __init__(self, stream1, stream2):
-        # type: (IO, IO) -> None
+    def __init__(self, stream1: IO, stream2: IO) -> None:
         self.stream1 = stream1
         self.stream2 = stream2
 
-    def write(self, text):
-        # type: (unicode) -> None
+    def write(self, text: str) -> None:
         self.stream1.write(text)
         self.stream2.write(text)
 
-    def flush(self):
-        # type: () -> None
+    def flush(self) -> None:
         if hasattr(self.stream1, 'flush'):
             self.stream1.flush()
         if hasattr(self.stream2, 'flush'):
             self.stream2.flush()
 
 
-def parselinenos(spec, total):
-    # type: (unicode, int) -> List[int]
+def parselinenos(spec: str, total: int) -> List[int]:
     """Parse a line number spec (such as "1,2,4-6") and return a list of
     wanted line numbers.
     """
@@ -456,16 +464,17 @@ def parselinenos(spec, total):
     return items
 
 
-def force_decode(string, encoding):
-    # type: (unicode, unicode) -> unicode
+def force_decode(string: str, encoding: str) -> str:
     """Forcibly get a unicode string out of a bytestring."""
-    if isinstance(string, binary_type):
+    warnings.warn('force_decode() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
+    if isinstance(string, bytes):
         try:
             if encoding:
                 string = string.decode(encoding)
             else:
                 # try decoding with utf-8, should only work for real UTF-8
-                string = string.decode('utf-8')
+                string = string.decode()
         except UnicodeError:
             # last resort -- can't fail
             string = string.decode('latin1')
@@ -473,21 +482,22 @@ def force_decode(string, encoding):
 
 
 class attrdict(dict):
-    def __getattr__(self, key):
-        # type: (unicode) -> unicode
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+        warnings.warn('The attrdict class is deprecated.',
+                      RemovedInSphinx40Warning, stacklevel=2)
+
+    def __getattr__(self, key: str) -> str:
         return self[key]
 
-    def __setattr__(self, key, val):
-        # type: (unicode, unicode) -> None
+    def __setattr__(self, key: str, val: str) -> None:
         self[key] = val
 
-    def __delattr__(self, key):
-        # type: (unicode) -> None
+    def __delattr__(self, key: str) -> None:
         del self[key]
 
 
-def rpartition(s, t):
-    # type: (unicode, unicode) -> Tuple[unicode, unicode]
+def rpartition(s: str, t: str) -> Tuple[str, str]:
     """Similar to str.rpartition from 2.5, but doesn't return the separator."""
     i = s.rfind(t)
     if i != -1:
@@ -495,8 +505,7 @@ def rpartition(s, t):
     return '', s
 
 
-def split_into(n, type, value):
-    # type: (int, unicode, unicode) -> List[unicode]
+def split_into(n: int, type: str, value: str) -> List[str]:
     """Split an index entry into a given number of parts at semicolons."""
     parts = [x.strip() for x in value.split(';', n - 1)]
     if sum(1 for part in parts if part) < n:
@@ -504,8 +513,7 @@ def split_into(n, type, value):
     return parts
 
 
-def split_index_msg(type, value):
-    # type: (unicode, unicode) -> List[unicode]
+def split_index_msg(type: str, value: str) -> List[str]:
     # new entry types must be listed in directives/other.py!
     if type == 'single':
         try:
@@ -526,34 +534,32 @@ def split_index_msg(type, value):
     return result
 
 
-def format_exception_cut_frames(x=1):
-    # type: (int) -> unicode
+def format_exception_cut_frames(x: int = 1) -> str:
     """Format an exception with traceback, but only the last x frames."""
     typ, val, tb = sys.exc_info()
     # res = ['Traceback (most recent call last):\n']
-    res = []  # type: List[unicode]
+    res = []  # type: List[str]
     tbres = traceback.format_tb(tb)
     res += tbres[-x:]
     res += traceback.format_exception_only(typ, val)
     return ''.join(res)
 
 
-class PeekableIterator(object):
+class PeekableIterator:
     """
     An iterator which wraps any iterable and makes it possible to peek to see
     what's the next item.
     """
-    def __init__(self, iterable):
-        # type: (Iterable) -> None
+    def __init__(self, iterable: Iterable) -> None:
         self.remaining = deque()  # type: deque
         self._iterator = iter(iterable)
+        warnings.warn('PeekableIterator is deprecated.',
+                      RemovedInSphinx40Warning, stacklevel=2)
 
-    def __iter__(self):
-        # type: () -> PeekableIterator
+    def __iter__(self) -> "PeekableIterator":
         return self
 
-    def __next__(self):
-        # type: () -> Any
+    def __next__(self) -> Any:
         """Return the next item from the iterator."""
         if self.remaining:
             return self.remaining.popleft()
@@ -561,62 +567,60 @@ class PeekableIterator(object):
 
     next = __next__  # Python 2 compatibility
 
-    def push(self, item):
-        # type: (Any) -> None
+    def push(self, item: Any) -> None:
         """Push the `item` on the internal stack, it will be returned on the
         next :meth:`next` call.
         """
         self.remaining.append(item)
 
-    def peek(self):
-        # type: () -> Any
+    def peek(self) -> Any:
         """Return the next item without changing the state of the iterator."""
         item = next(self)
         self.push(item)
         return item
 
 
-def import_object(objname, source=None):
-    # type: (str, unicode) -> Any
+def import_object(objname: str, source: str = None) -> Any:
+    """Import python object by qualname."""
     try:
-        module, name = objname.rsplit('.', 1)
-    except ValueError as err:
-        raise ExtensionError('Invalid full object name %s' % objname +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
-    try:
-        return getattr(__import__(module, None, None, [name]), name)
-    except ImportError as err:
-        raise ExtensionError('Could not import %s' % module +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
-    except AttributeError as err:
-        raise ExtensionError('Could not find %s' % objname +
-                             (source and ' (needed for %s)' % source or ''),
-                             err)
+        objpath = objname.split('.')
+        modname = objpath.pop(0)
+        obj = import_module(modname)
+        for name in objpath:
+            modname += '.' + name
+            try:
+                obj = getattr(obj, name)
+            except AttributeError:
+                obj = import_module(modname)
+
+        return obj
+    except (AttributeError, ImportError) as exc:
+        if source:
+            raise ExtensionError('Could not import %s (needed for %s)' %
+                                 (objname, source), exc)
+        else:
+            raise ExtensionError('Could not import %s' % objname, exc)
 
 
-def encode_uri(uri):
-    # type: (unicode) -> unicode
-    split = list(urlsplit(uri))  # type: List[unicode]
+def encode_uri(uri: str) -> str:
+    split = list(urlsplit(uri))
     split[1] = split[1].encode('idna').decode('ascii')
-    split[2] = quote_plus(split[2].encode('utf-8'), '/')
-    query = list((q, v.encode('utf-8')) for (q, v) in parse_qsl(split[3]))
+    split[2] = quote_plus(split[2].encode(), '/')
+    query = list((q, v.encode()) for (q, v) in parse_qsl(split[3]))
     split[3] = urlencode(query)
     return urlunsplit(split)
 
 
-def display_chunk(chunk):
-    # type: (Any) -> unicode
+def display_chunk(chunk: Any) -> str:
     if isinstance(chunk, (list, tuple)):
         if len(chunk) == 1:
-            return text_type(chunk[0])
+            return str(chunk[0])
         return '%s .. %s' % (chunk[0], chunk[-1])
-    return text_type(chunk)
+    return str(chunk)
 
 
-def old_status_iterator(iterable, summary, color="darkgreen", stringify_func=display_chunk):
-    # type: (Iterable, unicode, str, Callable[[Any], unicode]) -> Iterator
+def old_status_iterator(iterable: Iterable, summary: str, color: str = "darkgreen",
+                        stringify_func: Callable[[Any], str] = display_chunk) -> Iterator:
     l = 0
     for item in iterable:
         if l == 0:
@@ -630,12 +634,11 @@ def old_status_iterator(iterable, summary, color="darkgreen", stringify_func=dis
 
 
 # new version with progress info
-def status_iterator(iterable, summary, color="darkgreen", length=0, verbosity=0,
-                    stringify_func=display_chunk):
-    # type: (Iterable, unicode, str, int, int, Callable[[Any], unicode]) -> Iterable  # NOQA
+def status_iterator(iterable: Iterable, summary: str, color: str = "darkgreen",
+                    length: int = 0, verbosity: int = 0,
+                    stringify_func: Callable[[Any], str] = display_chunk) -> Iterable:
     if length == 0:
-        for item in old_status_iterator(iterable, summary, color, stringify_func):
-            yield item
+        yield from old_status_iterator(iterable, summary, color, stringify_func)
         return
     l = 0
     summary = bold(summary)
@@ -652,8 +655,40 @@ def status_iterator(iterable, summary, color="darkgreen", length=0, verbosity=0,
         logger.info('')
 
 
-def epoch_to_rfc1123(epoch):
-    # type: (float) -> unicode
+class SkipProgressMessage(Exception):
+    pass
+
+
+class progress_message:
+    def __init__(self, message: str) -> None:
+        self.message = message
+
+    def __enter__(self) -> None:
+        logger.info(bold(self.message + '... '), nonl=True)
+
+    def __exit__(self, exc_type: "Type[Exception]", exc_value: Exception, traceback: Any) -> bool:  # NOQA
+        if isinstance(exc_value, SkipProgressMessage):
+            logger.info(__('skipped'))
+            if exc_value.args:
+                logger.info(*exc_value.args)
+            return True
+        elif exc_type:
+            logger.info(__('failed'))
+        else:
+            logger.info(__('done'))
+
+        return False
+
+    def __call__(self, f: Callable) -> Callable:
+        @functools.wraps(f)
+        def wrapper(*args, **kwargs):
+            with self:
+                return f(*args, **kwargs)
+
+        return wrapper
+
+
+def epoch_to_rfc1123(epoch: float) -> str:
     """Convert datetime format epoch to RFC1123."""
     from babel.dates import format_datetime
 
@@ -662,41 +697,34 @@ def epoch_to_rfc1123(epoch):
     return format_datetime(dt, fmt, locale='en') + ' GMT'
 
 
-def rfc1123_to_epoch(rfc1123):
-    # type: (str) -> float
+def rfc1123_to_epoch(rfc1123: str) -> float:
     return mktime(strptime(rfc1123, '%a, %d %b %Y %H:%M:%S %Z'))
 
 
-def xmlname_checker():
-    # type: () -> Pattern
+def xmlname_checker() -> Pattern:
     # https://www.w3.org/TR/REC-xml/#NT-Name
-    # Only Python 3.3 or newer support character code in regular expression
     name_start_chars = [
-        u':', [u'A', u'Z'], u'_',  [u'a', u'z'], [u'\u00C0', u'\u00D6'],
-        [u'\u00D8', u'\u00F6'], [u'\u00F8', u'\u02FF'], [u'\u0370', u'\u037D'],
-        [u'\u037F', u'\u1FFF'], [u'\u200C', u'\u200D'], [u'\u2070', u'\u218F'],
-        [u'\u2C00', u'\u2FEF'], [u'\u3001', u'\uD7FF'], [u'\uF900', u'\uFDCF'],
-        [u'\uFDF0', u'\uFFFD']]
-
-    if sys.version_info.major == 3:
-        name_start_chars.append([u'\U00010000', u'\U000EFFFF'])
+        ':', ['A', 'Z'], '_', ['a', 'z'], ['\u00C0', '\u00D6'],
+        ['\u00D8', '\u00F6'], ['\u00F8', '\u02FF'], ['\u0370', '\u037D'],
+        ['\u037F', '\u1FFF'], ['\u200C', '\u200D'], ['\u2070', '\u218F'],
+        ['\u2C00', '\u2FEF'], ['\u3001', '\uD7FF'], ['\uF900', '\uFDCF'],
+        ['\uFDF0', '\uFFFD'], ['\U00010000', '\U000EFFFF']]
 
     name_chars = [
-        u"\\-", u"\\.", [u'0', u'9'], u'\u00B7', [u'\u0300', u'\u036F'],
-        [u'\u203F', u'\u2040']
+        "\\-", "\\.", ['0', '9'], '\u00B7', ['\u0300', '\u036F'],
+        ['\u203F', '\u2040']
     ]
 
-    def convert(entries, splitter=u'|'):
-        # type: (Any, unicode) -> unicode
+    def convert(entries: Any, splitter: str = '|') -> str:
         results = []
         for entry in entries:
             if isinstance(entry, list):
-                results.append(u'[%s]' % convert(entry, u'-'))
+                results.append('[%s]' % convert(entry, '-'))
             else:
                 results.append(entry)
         return splitter.join(results)
 
     start_chars_regex = convert(name_start_chars)
     name_chars_regex = convert(name_chars)
-    return re.compile(u'(%s)(%s|%s)*' % (
+    return re.compile('(%s)(%s|%s)*' % (
         start_chars_regex, start_chars_regex, name_chars_regex))
