@@ -1,18 +1,18 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.docstrings
     ~~~~~~~~~~~~~~~~~~~~~~
 
     Utilities for docstring processing.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
+from typing import List
 
 
-def prepare_docstring(s, ignore=1):
+def prepare_docstring(s: str, ignore: int = 1, tabsize: int = 8) -> List[str]:
     """Convert a docstring into lines of parseable reST.  Remove common leading
     indentation, where the indentation of a given number of lines (usually just
     one) is ignored.
@@ -21,7 +21,7 @@ def prepare_docstring(s, ignore=1):
     ViewList (used as argument of nested_parse().)  An empty line is added to
     act as a separator between this docstring and following content.
     """
-    lines = s.expandtabs().splitlines()
+    lines = s.expandtabs(tabsize).splitlines()
     # Find minimum indentation of any non-blank lines after ignored lines.
     margin = sys.maxsize
     for line in lines[ignore:]:
@@ -45,7 +45,7 @@ def prepare_docstring(s, ignore=1):
     return lines
 
 
-def prepare_commentdoc(s):
+def prepare_commentdoc(s: str) -> List[str]:
     """Extract documentation comment lines (starting with #:) and return them
     as a list of lines.  Returns an empty list if there is no documentation.
     """
