@@ -1,40 +1,38 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.i18n
     ~~~~~~~~~~~~~~~~
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import gettext
-import io
 import os
 import re
 import warnings
 from collections import namedtuple
-from datetime import datetime
+from datetime import datetime, timezone
 from os import path
+from typing import Callable, Generator, List, Set, Tuple
 
 import babel.dates
 from babel.messages.mofile import write_mo
 from babel.messages.pofile import read_po
 
-from sphinx.deprecation import RemovedInSphinx30Warning
+from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
 from sphinx.errors import SphinxError
 from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.matching import Matcher
-from sphinx.util.osutil import SEP, relpath, walk
-
-
-logger = logging.getLogger(__name__)
+from sphinx.util.osutil import SEP, canon_path, relpath
 
 if False:
     # For type annotation
-    from typing import Callable, List, Set  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.environment import BuildEnvironment
+
+
+logger = logging.getLogger(__name__)
 
 LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
 
@@ -42,49 +40,86 @@ LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
 class CatalogInfo(LocaleFileInfoBase):
 
     @property
-    def po_file(self):
-        # type: () -> unicode
+    def po_file(self) -> str:
         return self.domain + '.po'
 
     @property
-    def mo_file(self):
-        # type: () -> unicode
+    def mo_file(self) -> str:
         return self.domain + '.mo'
 
     @property
-    def po_path(self):
-        # type: () -> unicode
+    def po_path(self) -> str:
         return path.join(self.base_dir, self.po_file)
 
     @property
-    def mo_path(self):
-        # type: () -> unicode
+    def mo_path(self) -> str:
         return path.join(self.base_dir, self.mo_file)
 
-    def is_outdated(self):
-        # type: () -> bool
+    def is_outdated(self) -> bool:
         return (
             not path.exists(self.mo_path) or
             path.getmtime(self.mo_path) < path.getmtime(self.po_path))
 
-    def write_mo(self, locale):
-        # type: (unicode) -> None
-        with io.open(self.po_path, 'rt', encoding=self.charset) as file_po:
+    def write_mo(self, locale: str) -> None:
+        with open(self.po_path, encoding=self.charset) as file_po:
             try:
                 po = read_po(file_po, locale)
             except Exception as exc:
                 logger.warning(__('reading error: %s, %s'), self.po_path, exc)
                 return
 
-        with io.open(self.mo_path, 'wb') as file_mo:
+        with open(self.mo_path, 'wb') as file_mo:
             try:
                 write_mo(file_mo, po)
             except Exception as exc:
                 logger.warning(__('writing error: %s, %s'), self.mo_path, exc)
 
 
-def find_catalog(docname, compaction):
-    # type: (unicode, bool) -> unicode
+class CatalogRepository:
+    """A repository for message catalogs."""
+
+    def __init__(self, basedir: str, locale_dirs: List[str],
+                 language: str, encoding: str) -> None:
+        self.basedir = basedir
+        self._locale_dirs = locale_dirs
+        self.language = language
+        self.encoding = encoding
+
+    @property
+    def locale_dirs(self) -> Generator[str, None, None]:
+        if not self.language:
+            return
+
+        for locale_dir in self._locale_dirs:
+            locale_dir = path.join(self.basedir, locale_dir)
+            if path.exists(path.join(locale_dir, self.language, 'LC_MESSAGES')):
+                yield locale_dir
+
+    @property
+    def pofiles(self) -> Generator[Tuple[str, str], None, None]:
+        for locale_dir in self.locale_dirs:
+            basedir = path.join(locale_dir, self.language, 'LC_MESSAGES')
+            for root, dirnames, filenames in os.walk(basedir):
+                # skip dot-directories
+                for dirname in dirnames:
+                    if dirname.startswith('.'):
+                        dirnames.remove(dirname)
+
+                for filename in filenames:
+                    if filename.endswith('.po'):
+                        fullpath = path.join(root, filename)
+                        yield basedir, relpath(fullpath, basedir)
+
+    @property
+    def catalogs(self) -> Generator[CatalogInfo, None, None]:
+        for basedir, filename in self.pofiles:
+            domain = canon_path(path.splitext(filename)[0])
+            yield CatalogInfo(basedir, domain, self.encoding)
+
+
+def find_catalog(docname: str, compaction: bool) -> str:
+    warnings.warn('find_catalog() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if compaction:
         ret = docname.split(SEP, 1)[0]
     else:
@@ -93,22 +128,32 @@ def find_catalog(docname, compaction):
     return ret
 
 
-def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
-    # type: (unicode, unicode, List[unicode], unicode, bool) -> List[unicode]
+def docname_to_domain(docname: str, compation: bool) -> str:
+    """Convert docname to domain for catalogs."""
+    if compation:
+        return docname.split(SEP, 1)[0]
+    else:
+        return docname
+
+
+def find_catalog_files(docname: str, srcdir: str, locale_dirs: List[str],
+                       lang: str, compaction: bool) -> List[str]:
+    warnings.warn('find_catalog_files() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if not(lang and locale_dirs):
         return []
 
     domain = find_catalog(docname, compaction)
-    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])  # type: ignore
+    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])
              for dir_ in locale_dirs]
-    files = [relpath(f, srcdir) for f in files if f]  # type: ignore
-    return files  # type: ignore
+    files = [relpath(f, srcdir) for f in files if f]
+    return files
 
 
-def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact=None,
-                              charset='utf-8', force_all=False,
-                              excluded=Matcher([])):
-    # type: (List[unicode], unicode, List[unicode], bool, unicode, bool, Matcher) -> Set[CatalogInfo]  # NOQA
+def find_catalog_source_files(locale_dirs: List[str], locale: str, domains: List[str] = None,
+                              gettext_compact: bool = None, charset: str = 'utf-8',
+                              force_all: bool = False, excluded: Matcher = Matcher([])
+                              ) -> Set[CatalogInfo]:
     """
     :param list locale_dirs:
        list of path as `['locale_dir1', 'locale_dir2', ...]` to find
@@ -122,6 +167,8 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
        default is False.
     :return: [CatalogInfo(), ...]
     """
+    warnings.warn('find_catalog_source_files() is deprecated.',
+                  RemovedInSphinx40Warning, stacklevel=2)
     if gettext_compact is not None:
         warnings.warn('gettext_compact argument for find_catalog_source_files() '
                       'is deprecated.', RemovedInSphinx30Warning, stacklevel=2)
@@ -140,7 +187,7 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
         if not path.exists(base_dir):
             continue  # locale path is not found
 
-        for dirpath, dirnames, filenames in walk(base_dir, followlinks=True):
+        for dirpath, dirnames, filenames in os.walk(base_dir, followlinks=True):
             filenames = [f for f in filenames if f.endswith('.po')]
             for filename in filenames:
                 if excluded(path.join(relpath(dirpath, base_dir), filename)):
@@ -190,7 +237,7 @@ date_format_mappings = {
     '%x':  'medium',  # Locale’s appropriate date representation.
     '%X':  'medium',  # Locale’s appropriate time representation.
     '%y':  'YY',      # Year without century as a zero-padded decimal number.
-    '%Y':  'YYYY',    # Year with century as a decimal number.
+    '%Y':  'yyyy',    # Year with century as a decimal number.
     '%Z':  'zzzz',    # Time zone name (no characters if no time zone exists).
     '%%':  '%',
 }
@@ -198,8 +245,8 @@ date_format_mappings = {
 date_format_re = re.compile('(%s)' % '|'.join(date_format_mappings))
 
 
-def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
-    # type: (datetime, unicode, unicode, Callable) -> unicode
+def babel_format_date(date: datetime, format: str, locale: str,
+                      formatter: Callable = babel.dates.format_date) -> str:
     if locale is None:
         locale = 'en'
 
@@ -219,8 +266,7 @@ def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
         return format
 
 
-def format_date(format, date=None, language=None):
-    # type: (str, datetime, unicode) -> unicode
+def format_date(format: str, date: datetime = None, language: str = None) -> str:
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
         # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
@@ -228,7 +274,7 @@ def format_date(format, date=None, language=None):
         if source_date_epoch is not None:
             date = datetime.utcfromtimestamp(float(source_date_epoch))
         else:
-            date = datetime.now()
+            date = datetime.now(timezone.utc).astimezone()
 
     result = []
     tokens = date_format_re.split(format)
@@ -254,8 +300,7 @@ def format_date(format, date=None, language=None):
     return "".join(result)
 
 
-def get_image_filename_for_language(filename, env):
-    # type: (unicode, BuildEnvironment) -> unicode
+def get_image_filename_for_language(filename: str, env: "BuildEnvironment") -> str:
     if not env.config.language:
         return filename
 
@@ -274,8 +319,7 @@ def get_image_filename_for_language(filename, env):
         raise SphinxError('Invalid figure_language_filename: %r' % exc)
 
 
-def search_image_for_language(filename, env):
-    # type: (unicode, BuildEnvironment) -> unicode
+def search_image_for_language(filename: str, env: "BuildEnvironment") -> str:
     if not env.config.language:
         return filename
 
