@@ -1,64 +1,54 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.inventory
     ~~~~~~~~~~~~~~~~~~~~~
 
     Inventory utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import os
 import re
 import zlib
-
-from six import PY3
+from typing import Callable, IO, Iterator
 
 from sphinx.util import logging
-
-if False:
-    # For type annotation
-    from typing import Callable, Dict, IO, Iterator, Tuple  # NOQA
-    from sphinx.builders import Builder  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
-
-    if PY3:
-        unicode = str
-
-    Inventory = Dict[unicode, Dict[unicode, Tuple[unicode, unicode, unicode, unicode]]]
+from sphinx.util.typing import Inventory
 
 
 BUFSIZE = 16 * 1024
 logger = logging.getLogger(__name__)
 
+if False:
+    # For type annotation
+    from sphinx.builders import Builder
+    from sphinx.environment import BuildEnvironment
 
-class InventoryFileReader(object):
+
+class InventoryFileReader:
     """A file reader for inventory file.
 
     This reader supports mixture of texts and compressed texts.
     """
 
-    def __init__(self, stream):
-        # type: (IO) -> None
+    def __init__(self, stream: IO) -> None:
         self.stream = stream
         self.buffer = b''
         self.eof = False
 
-    def read_buffer(self):
-        # type: () -> None
+    def read_buffer(self) -> None:
         chunk = self.stream.read(BUFSIZE)
         if chunk == b'':
             self.eof = True
         self.buffer += chunk
 
-    def readline(self):
-        # type: () -> unicode
+    def readline(self) -> str:
         pos = self.buffer.find(b'\n')
         if pos != -1:
-            line = self.buffer[:pos].decode('utf-8')
+            line = self.buffer[:pos].decode()
             self.buffer = self.buffer[pos + 1:]
         elif self.eof:
-            line = self.buffer.decode('utf-8')
+            line = self.buffer.decode()
             self.buffer = b''
         else:
             self.read_buffer()
@@ -66,15 +56,13 @@ class InventoryFileReader(object):
 
         return line
 
-    def readlines(self):
-        # type: () -> Iterator[unicode]
+    def readlines(self) -> Iterator[str]:
         while not self.eof:
             line = self.readline()
             if line:
                 yield line
 
-    def read_compressed_chunks(self):
-        # type: () -> Iterator[bytes]
+    def read_compressed_chunks(self) -> Iterator[bytes]:
         decompressor = zlib.decompressobj()
         while not self.eof:
             self.read_buffer()
@@ -82,22 +70,20 @@ class InventoryFileReader(object):
             self.buffer = b''
         yield decompressor.flush()
 
-    def read_compressed_lines(self):
-        # type: () -> Iterator[unicode]
+    def read_compressed_lines(self) -> Iterator[str]:
         buf = b''
         for chunk in self.read_compressed_chunks():
             buf += chunk
             pos = buf.find(b'\n')
             while pos != -1:
-                yield buf[:pos].decode('utf-8')
+                yield buf[:pos].decode()
                 buf = buf[pos + 1:]
                 pos = buf.find(b'\n')
 
 
-class InventoryFile(object):
+class InventoryFile:
     @classmethod
-    def load(cls, stream, uri, joinfunc):
-        # type: (IO, unicode, Callable) -> Inventory
+    def load(cls, stream: IO, uri: str, joinfunc: Callable) -> Inventory:
         reader = InventoryFileReader(stream)
         line = reader.readline().rstrip()
         if line == '# Sphinx inventory version 1':
@@ -108,8 +94,7 @@ class InventoryFile(object):
             raise ValueError('invalid inventory header: %s' % line)
 
     @classmethod
-    def load_v1(cls, stream, uri, join):
-        # type: (InventoryFileReader, unicode, Callable) -> Inventory
+    def load_v1(cls, stream: InventoryFileReader, uri: str, join: Callable) -> Inventory:
         invdata = {}  # type: Inventory
         projname = stream.readline().rstrip()[11:]
         version = stream.readline().rstrip()[11:]
@@ -127,8 +112,7 @@ class InventoryFile(object):
         return invdata
 
     @classmethod
-    def load_v2(cls, stream, uri, join):
-        # type: (InventoryFileReader, unicode, Callable) -> Inventory
+    def load_v2(cls, stream: InventoryFileReader, uri: str, join: Callable) -> Inventory:
         invdata = {}  # type: Inventory
         projname = stream.readline().rstrip()[11:]
         version = stream.readline().rstrip()[11:]
@@ -138,7 +122,7 @@ class InventoryFile(object):
 
         for line in stream.read_compressed_lines():
             # be careful to handle names with embedded spaces correctly
-            m = re.match(r'(?x)(.+?)\s+(\S*:\S*)\s+(-?\d+)\s+(\S+)\s+(.*)',
+            m = re.match(r'(?x)(.+?)\s+(\S*:\S*)\s+(-?\d+)\s+?(\S*)\s+(.*)',
                          line.rstrip())
             if not m:
                 continue
@@ -149,7 +133,7 @@ class InventoryFile(object):
                 # for Python modules, and the first
                 # one is correct
                 continue
-            if location.endswith(u'$'):
+            if location.endswith('$'):
                 location = location[:-1] + name
             location = join(uri, location)
             invdata.setdefault(type, {})[name] = (projname, version,
@@ -157,20 +141,18 @@ class InventoryFile(object):
         return invdata
 
     @classmethod
-    def dump(cls, filename, env, builder):
-        # type: (unicode, BuildEnvironment, Builder) -> None
-        def escape(string):
-            # type: (unicode) -> unicode
+    def dump(cls, filename: str, env: "BuildEnvironment", builder: "Builder") -> None:
+        def escape(string: str) -> str:
             return re.sub("\\s+", " ", string)
 
         with open(os.path.join(filename), 'wb') as f:
             # header
-            f.write((u'# Sphinx inventory version 2\n'
-                     u'# Project: %s\n'
-                     u'# Version: %s\n'
-                     u'# The remainder of this file is compressed using zlib.\n' %
+            f.write(('# Sphinx inventory version 2\n'
+                     '# Project: %s\n'
+                     '# Version: %s\n'
+                     '# The remainder of this file is compressed using zlib.\n' %
                      (escape(env.config.project),
-                      escape(env.config.version))).encode('utf-8'))
+                      escape(env.config.version))).encode())
 
             # body
             compressor = zlib.compressobj(9)
@@ -184,8 +166,8 @@ class InventoryFile(object):
                     if anchor:
                         uri += '#' + anchor
                     if dispname == name:
-                        dispname = u'-'
-                    entry = (u'%s %s:%s %s %s %s\n' %
+                        dispname = '-'
+                    entry = ('%s %s:%s %s %s %s\n' %
                              (name, domainname, typ, prio, uri, dispname))
-                    f.write(compressor.compress(entry.encode('utf-8')))
+                    f.write(compressor.compress(entry.encode()))
             f.write(compressor.flush())
