@@ -1,32 +1,30 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.util.logging
     ~~~~~~~~~~~~~~~~~~~
 
     Logging utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import absolute_import
 
 import logging
 import logging.handlers
 from collections import defaultdict
 from contextlib import contextmanager
+from typing import Any, Dict, Generator, IO, List, Tuple, Union
 
 from docutils import nodes
+from docutils.nodes import Node
 from docutils.utils import get_source_line
-from six import PY2, StringIO
 
 from sphinx.errors import SphinxWarning
 from sphinx.util.console import colorize
 
 if False:
     # For type annotation
-    from typing import Any, Dict, Generator, IO, List, Tuple, Union  # NOQA
-    from docutils import nodes  # NOQA
-    from sphinx.application import Sphinx  # NOQA
+    from typing import Type  # for python3.5.1
+    from sphinx.application import Sphinx
 
 
 NAMESPACE = 'sphinx'
@@ -50,21 +48,27 @@ VERBOSITY_MAP.update({
     2: logging.DEBUG,
 })
 
-COLOR_MAP = defaultdict(lambda: 'blue')  # type: Dict[int, unicode]
-COLOR_MAP.update({
-    logging.ERROR: 'darkred',
-    logging.WARNING: 'red',
-    logging.DEBUG: 'darkgray',
-})
+COLOR_MAP = defaultdict(lambda: 'blue',
+                        {
+                            logging.ERROR: 'darkred',
+                            logging.WARNING: 'red',
+                            logging.DEBUG: 'darkgray'
+                        })
 
 
-def getLogger(name):
-    # type: (str) -> SphinxLoggerAdapter
-    """Get logger wrapped by SphinxLoggerAdapter.
+def getLogger(name: str) -> "SphinxLoggerAdapter":
+    """Get logger wrapped by :class:`sphinx.util.logging.SphinxLoggerAdapter`.
 
-    Sphinx logger always uses ``sphinx.*`` namesapce to be independent from
-    settings of root logger.  It enables to log stably even if 3rd party
-    extension or imported application resets logger settings.
+    Sphinx logger always uses ``sphinx.*`` namespace to be independent from
+    settings of root logger.  It ensures logging is consistent even if a
+    third-party extension or imported application resets logger settings.
+
+    Example usage::
+
+        >>> from sphinx.util import logging
+        >>> logger = logging.getLogger(__name__)
+        >>> logger.info('Hello, this is an extension!')
+        Hello, this is an extension!
     """
     # add sphinx prefix to name forcely
     logger = logging.getLogger(NAMESPACE + '.' + name)
@@ -74,8 +78,7 @@ def getLogger(name):
     return SphinxLoggerAdapter(logger, {})
 
 
-def convert_serializable(records):
-    # type: (List[logging.LogRecord]) -> None
+def convert_serializable(records: List[logging.LogRecord]) -> None:
     """Convert LogRecord serializable."""
     for r in records:
         # extract arguments to a message and clear them
@@ -87,39 +90,46 @@ def convert_serializable(records):
             r.location = get_node_location(location)  # type: ignore
 
 
-class SphinxWarningLogRecord(logging.LogRecord):
+class SphinxLogRecord(logging.LogRecord):
     """Log record class supporting location"""
+    prefix = ''
     location = None  # type: Any
 
-    def getMessage(self):
-        # type: () -> str
-        message = super(SphinxWarningLogRecord, self).getMessage()
+    def getMessage(self) -> str:
+        message = super().getMessage()
         location = getattr(self, 'location', None)
         if location:
-            message = '%s: WARNING: %s' % (location, message)
-        elif 'WARNING:' not in message:
-            message = 'WARNING: %s' % message
+            message = '%s: %s%s' % (location, self.prefix, message)
+        elif self.prefix not in message:
+            message = self.prefix + message
 
         return message
 
 
+class SphinxInfoLogRecord(SphinxLogRecord):
+    """Info log record class supporting location"""
+    prefix = ''  # do not show any prefix for INFO messages
+
+
+class SphinxWarningLogRecord(SphinxLogRecord):
+    """Warning log record class supporting location"""
+    prefix = 'WARNING: '
+
+
 class SphinxLoggerAdapter(logging.LoggerAdapter):
     """LoggerAdapter allowing ``type`` and ``subtype`` keywords."""
 
-    def log(self, level, msg, *args, **kwargs):  # type: ignore
-        # type: (Union[int, str], unicode, Any, Any) -> None
+    def log(self, level: Union[int, str], msg: str, *args: Any, **kwargs: Any) -> None:
         if isinstance(level, int):
-            super(SphinxLoggerAdapter, self).log(level, msg, *args, **kwargs)
+            super().log(level, msg, *args, **kwargs)
         else:
             levelno = LEVEL_NAMES[level]
-            super(SphinxLoggerAdapter, self).log(levelno, msg, *args, **kwargs)
+            super().log(levelno, msg, *args, **kwargs)
 
-    def verbose(self, msg, *args, **kwargs):
-        # type: (unicode, Any, Any) -> None
+    def verbose(self, msg: str, *args: Any, **kwargs: Any) -> None:
         self.log(VERBOSE, msg, *args, **kwargs)
 
-    def process(self, msg, kwargs):  # type: ignore
-        # type: (unicode, Dict) -> Tuple[unicode, Dict]
+    def process(self, msg: str, kwargs: Dict) -> Tuple[str, Dict]:  # type: ignore
         extra = kwargs.setdefault('extra', {})
         if 'type' in kwargs:
             extra['type'] = kwargs.pop('type')
@@ -134,9 +144,8 @@ class SphinxLoggerAdapter(logging.LoggerAdapter):
 
         return msg, kwargs
 
-    def handle(self, record):
-        # type: (logging.LogRecord) -> None
-        self.logger.handle(record)  # type: ignore
+    def handle(self, record: logging.LogRecord) -> None:
+        self.logger.handle(record)
 
 
 class WarningStreamHandler(logging.StreamHandler):
@@ -144,62 +153,31 @@ class WarningStreamHandler(logging.StreamHandler):
     pass
 
 
-class NewLineStreamHandlerPY2(logging.StreamHandler):
+class NewLineStreamHandler(logging.StreamHandler):
     """StreamHandler which switches line terminator by record.nonl flag."""
 
-    def emit(self, record):
-        # type: (logging.LogRecord) -> None
-        try:
-            self.acquire()
-            stream = self.stream
-            if getattr(record, 'nonl', False):
-                # remove return code forcely when nonl=True
-                self.stream = StringIO()
-                super(NewLineStreamHandlerPY2, self).emit(record)
-                stream.write(self.stream.getvalue()[:-1])
-                stream.flush()
-            else:
-                super(NewLineStreamHandlerPY2, self).emit(record)
-        finally:
-            self.stream = stream
-            self.release()
-
-
-class NewLineStreamHandlerPY3(logging.StreamHandler):
-    """StreamHandler which switches line terminator by record.nonl flag."""
-
-    def emit(self, record):
-        # type: (logging.LogRecord) -> None
+    def emit(self, record: logging.LogRecord) -> None:
         try:
             self.acquire()
             if getattr(record, 'nonl', False):
                 # skip appending terminator when nonl=True
                 self.terminator = ''
-            super(NewLineStreamHandlerPY3, self).emit(record)
+            super().emit(record)
         finally:
             self.terminator = '\n'
             self.release()
 
 
-if PY2:
-    NewLineStreamHandler = NewLineStreamHandlerPY2
-else:
-    NewLineStreamHandler = NewLineStreamHandlerPY3
-
-
 class MemoryHandler(logging.handlers.BufferingHandler):
     """Handler buffering all logs."""
 
-    def __init__(self):
-        # type: () -> None
-        super(MemoryHandler, self).__init__(-1)
+    def __init__(self) -> None:
+        super().__init__(-1)
 
-    def shouldFlush(self, record):
-        # type: (logging.LogRecord) -> bool
+    def shouldFlush(self, record: logging.LogRecord) -> bool:
         return False  # never flush
 
-    def flushTo(self, logger):
-        # type: (logging.Logger) -> None
+    def flushTo(self, logger: logging.Logger) -> None:
         self.acquire()
         try:
             for record in self.buffer:
@@ -208,16 +186,17 @@ class MemoryHandler(logging.handlers.BufferingHandler):
         finally:
             self.release()
 
-    def clear(self):
-        # type: () -> List[logging.LogRecord]
+    def clear(self) -> List[logging.LogRecord]:
         buffer, self.buffer = self.buffer, []
         return buffer
 
 
 @contextmanager
-def pending_warnings():
-    # type: () -> Generator
-    """contextmanager to pend logging warnings temporary."""
+def pending_warnings() -> Generator[logging.Handler, None, None]:
+    """Contextmanager to pend logging warnings temporary.
+
+    Similar to :func:`pending_logging`.
+    """
     logger = logging.getLogger(NAMESPACE)
     memhandler = MemoryHandler()
     memhandler.setLevel(logging.WARNING)
@@ -241,9 +220,17 @@ def pending_warnings():
 
 
 @contextmanager
-def pending_logging():
-    # type: () -> Generator
-    """contextmanager to pend logging all logs temporary."""
+def pending_logging() -> Generator[MemoryHandler, None, None]:
+    """Contextmanager to pend logging all logs temporary.
+
+    For example::
+
+        >>> with pending_logging():
+        >>>     logger.warning('Warning message!')  # not flushed yet
+        >>>     some_long_process()
+        >>>
+        Warning message!  # the warning is flushed here
+    """
     logger = logging.getLogger(NAMESPACE)
     memhandler = MemoryHandler()
 
@@ -265,8 +252,7 @@ def pending_logging():
 
 
 @contextmanager
-def skip_warningiserror(skip=True):
-    # type: (bool) -> Generator
+def skip_warningiserror(skip: bool = True) -> Generator[None, None, None]:
     """contextmanager to skip WarningIsErrorFilter for a while."""
     logger = logging.getLogger(NAMESPACE)
 
@@ -285,14 +271,58 @@ def skip_warningiserror(skip=True):
                 handler.removeFilter(disabler)
 
 
-class LogCollector(object):
-    def __init__(self):
-        # type: () -> None
+@contextmanager
+def prefixed_warnings(prefix: str) -> Generator[None, None, None]:
+    """Prepend prefix to all records for a while.
+
+    For example::
+
+        >>> with prefixed_warnings("prefix:"):
+        >>>     logger.warning('Warning message!')  # => prefix: Warning message!
+
+    .. versionadded:: 2.0
+    """
+    logger = logging.getLogger(NAMESPACE)
+    warning_handler = None
+    for handler in logger.handlers:
+        if isinstance(handler, WarningStreamHandler):
+            warning_handler = handler
+            break
+    else:
+        # warning stream not found
+        yield
+        return
+
+    prefix_filter = None
+    for _filter in warning_handler.filters:
+        if isinstance(_filter, MessagePrefixFilter):
+            prefix_filter = _filter
+            break
+
+    if prefix_filter:
+        # already prefixed
+        try:
+            previous = prefix_filter.prefix
+            prefix_filter.prefix = prefix
+            yield
+        finally:
+            prefix_filter.prefix = previous
+    else:
+        # not prefixed yet
+        try:
+            prefix_filter = MessagePrefixFilter(prefix)
+            warning_handler.addFilter(prefix_filter)
+            yield
+        finally:
+            warning_handler.removeFilter(prefix_filter)
+
+
+class LogCollector:
+    def __init__(self) -> None:
         self.logs = []  # type: List[logging.LogRecord]
 
     @contextmanager
-    def collect(self):
-        # type: () -> Generator
+    def collect(self) -> Generator[None, None, None]:
         with pending_logging() as memhandler:
             yield
 
@@ -302,16 +332,14 @@ class LogCollector(object):
 class InfoFilter(logging.Filter):
     """Filter error and warning messages."""
 
-    def filter(self, record):
-        # type: (logging.LogRecord) -> bool
+    def filter(self, record: logging.LogRecord) -> bool:
         if record.levelno < logging.WARNING:
             return True
         else:
             return False
 
 
-def is_suppressed_warning(type, subtype, suppress_warnings):
-    # type: (unicode, unicode, List[unicode]) -> bool
+def is_suppressed_warning(type: str, subtype: str, suppress_warnings: List[str]) -> bool:
     """Check the warning is suppressed or not."""
     if type is None:
         return False
@@ -333,13 +361,11 @@ def is_suppressed_warning(type, subtype, suppress_warnings):
 class WarningSuppressor(logging.Filter):
     """Filter logs by `suppress_warnings`."""
 
-    def __init__(self, app):
-        # type: (Sphinx) -> None
+    def __init__(self, app: "Sphinx") -> None:
         self.app = app
-        super(WarningSuppressor, self).__init__()
+        super().__init__()
 
-    def filter(self, record):
-        # type: (logging.LogRecord) -> bool
+    def filter(self, record: logging.LogRecord) -> bool:
         type = getattr(record, 'type', None)
         subtype = getattr(record, 'subtype', None)
 
@@ -359,13 +385,11 @@ class WarningSuppressor(logging.Filter):
 class WarningIsErrorFilter(logging.Filter):
     """Raise exception if warning emitted."""
 
-    def __init__(self, app):
-        # type: (Sphinx) -> None
+    def __init__(self, app: "Sphinx") -> None:
         self.app = app
-        super(WarningIsErrorFilter, self).__init__()
+        super().__init__()
 
-    def filter(self, record):
-        # type: (logging.LogRecord) -> bool
+    def filter(self, record: logging.LogRecord) -> bool:
         if getattr(record, 'skip_warningsiserror', False):
             # disabled by DisableWarningIsErrorFilter
             return True
@@ -387,27 +411,40 @@ class WarningIsErrorFilter(logging.Filter):
 class DisableWarningIsErrorFilter(logging.Filter):
     """Disable WarningIsErrorFilter if this filter installed."""
 
-    def filter(self, record):
-        # type: (logging.LogRecord) -> bool
+    def filter(self, record: logging.LogRecord) -> bool:
         record.skip_warningsiserror = True  # type: ignore
         return True
 
 
-class WarningLogRecordTranslator(logging.Filter):
+class MessagePrefixFilter(logging.Filter):
+    """Prepend prefix to all records."""
+
+    def __init__(self, prefix: str) -> None:
+        self.prefix = prefix
+        super().__init__()
+
+    def filter(self, record: logging.LogRecord) -> bool:
+        if self.prefix:
+            record.msg = self.prefix + ' ' + record.msg
+        return True
+
+
+class SphinxLogRecordTranslator(logging.Filter):
     """Converts a log record to one Sphinx expects
 
-    * Make a instance of SphinxWarningLogRecord
+    * Make a instance of SphinxLogRecord
     * docname to path if location given
     """
-    def __init__(self, app):
-        # type: (Sphinx) -> None
-        self.app = app
-        super(WarningLogRecordTranslator, self).__init__()
+    LogRecordClass = None  # type: Type[logging.LogRecord]
 
-    def filter(self, record):  # type: ignore
-        # type: (SphinxWarningLogRecord) -> bool
+    def __init__(self, app: "Sphinx") -> None:
+        self.app = app
+        super().__init__()
+
+    def filter(self, record: SphinxWarningLogRecord) -> bool:  # type: ignore
         if isinstance(record, logging.LogRecord):
-            record.__class__ = SphinxWarningLogRecord  # force subclassing to handle location
+            # force subclassing to handle location
+            record.__class__ = self.LogRecordClass  # type: ignore
 
         location = getattr(record, 'location', None)
         if isinstance(location, tuple):
@@ -426,8 +463,17 @@ class WarningLogRecordTranslator(logging.Filter):
         return True
 
 
-def get_node_location(node):
-    # type: (nodes.Node) -> str
+class InfoLogRecordTranslator(SphinxLogRecordTranslator):
+    """LogRecordTranslator for INFO level log records."""
+    LogRecordClass = SphinxInfoLogRecord
+
+
+class WarningLogRecordTranslator(SphinxLogRecordTranslator):
+    """LogRecordTranslator for WARNING level log records."""
+    LogRecordClass = SphinxWarningLogRecord
+
+
+def get_node_location(node: Node) -> str:
     (source, line) = get_source_line(node)
     if source and line:
         return "%s:%s" % (source, line)
@@ -440,28 +486,25 @@ def get_node_location(node):
 
 
 class ColorizeFormatter(logging.Formatter):
-    def format(self, record):
-        # type: (logging.LogRecord) -> str
-        message = super(ColorizeFormatter, self).format(record)
+    def format(self, record: logging.LogRecord) -> str:
+        message = super().format(record)
         color = getattr(record, 'color', None)
         if color is None:
             color = COLOR_MAP.get(record.levelno)
 
         if color:
-            return colorize(color, message)  # type: ignore
+            return colorize(color, message)
         else:
             return message
 
 
-class SafeEncodingWriter(object):
+class SafeEncodingWriter:
     """Stream writer which ignores UnicodeEncodeError silently"""
-    def __init__(self, stream):
-        # type: (IO) -> None
+    def __init__(self, stream: IO) -> None:
         self.stream = stream
         self.encoding = getattr(stream, 'encoding', 'ascii') or 'ascii'
 
-    def write(self, data):
-        # type: (unicode) -> None
+    def write(self, data: str) -> None:
         try:
             self.stream.write(data)
         except UnicodeEncodeError:
@@ -469,25 +512,21 @@ class SafeEncodingWriter(object):
             # non-encodable characters, then decode them.
             self.stream.write(data.encode(self.encoding, 'replace').decode(self.encoding))
 
-    def flush(self):
-        # type: () -> None
+    def flush(self) -> None:
         if hasattr(self.stream, 'flush'):
             self.stream.flush()
 
 
-class LastMessagesWriter(object):
+class LastMessagesWriter:
     """Stream writer which memories last 10 messages to save trackback"""
-    def __init__(self, app, stream):
-        # type: (Sphinx, IO) -> None
+    def __init__(self, app: "Sphinx", stream: IO) -> None:
         self.app = app
 
-    def write(self, data):
-        # type: (unicode) -> None
+    def write(self, data: str) -> None:
         self.app.messagelog.append(data)
 
 
-def setup(app, status, warning):
-    # type: (Sphinx, IO, IO) -> None
+def setup(app: "Sphinx", status: IO, warning: IO) -> None:
     """Setup root logger for Sphinx"""
     logger = logging.getLogger(NAMESPACE)
     logger.setLevel(logging.DEBUG)
@@ -499,6 +538,7 @@ def setup(app, status, warning):
 
     info_handler = NewLineStreamHandler(SafeEncodingWriter(status))  # type: ignore
     info_handler.addFilter(InfoFilter())
+    info_handler.addFilter(InfoLogRecordTranslator(app))
     info_handler.setLevel(VERBOSITY_MAP[app.verbosity])
     info_handler.setFormatter(ColorizeFormatter())
 
