@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.versioning
     ~~~~~~~~~~~~~~~~~
@@ -6,23 +5,26 @@
     Implements the low-level algorithms Sphinx uses for the versioning of
     doctrees.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from itertools import product
+import pickle
+import warnings
+from itertools import product, zip_longest
 from operator import itemgetter
+from os import path
+from typing import Any, Dict, Iterator
 from uuid import uuid4
 
-from six import iteritems
-from six.moves import cPickle as pickle
-from six.moves import range, zip_longest
+from docutils import nodes
+from docutils.nodes import Node
 
+from sphinx.deprecation import RemovedInSphinx30Warning
 from sphinx.transforms import SphinxTransform
 
 if False:
     # For type annotation
-    from typing import Any, Iterator  # NOQA
-    from docutils import nodes  # NOQA
+    from sphinx.application import Sphinx
 
 try:
     import Levenshtein
@@ -34,8 +36,7 @@ except ImportError:
 VERSIONING_RATIO = 65
 
 
-def add_uids(doctree, condition):
-    # type: (nodes.Node, Any) -> Iterator[nodes.Node]
+def add_uids(doctree: Node, condition: Any) -> Iterator[Node]:
     """Add a unique id to every node in the `doctree` which matches the
     condition and yield the nodes.
 
@@ -50,8 +51,7 @@ def add_uids(doctree, condition):
         yield node
 
 
-def merge_doctrees(old, new, condition):
-    # type: (nodes.Node, nodes.Node, Any) -> Iterator[nodes.Node]
+def merge_doctrees(old: Node, new: Node, condition: Any) -> Iterator[Node]:
     """Merge the `old` doctree with the `new` one while looking at nodes
     matching the `condition`.
 
@@ -100,7 +100,7 @@ def merge_doctrees(old, new, condition):
     # choose the old node with the best ratio for each new node and set the uid
     # as long as the ratio is under a certain value, in which case we consider
     # them not changed but different
-    ratios = sorted(iteritems(ratios), key=itemgetter(1))  # type: ignore
+    ratios = sorted(ratios.items(), key=itemgetter(1))  # type: ignore
     for (old_node, new_node), ratio in ratios:
         if new_node in seen:
             continue
@@ -118,8 +118,7 @@ def merge_doctrees(old, new, condition):
         yield new_node
 
 
-def get_ratio(old, new):
-    # type: (unicode, unicode) -> float
+def get_ratio(old: str, new: str) -> float:
     """Return a "similiarity ratio" (in percent) representing the similarity
     between the two strings where 0 is equal and anything above less than equal.
     """
@@ -132,8 +131,7 @@ def get_ratio(old, new):
         return levenshtein_distance(old, new) / (len(old) / 100.0)
 
 
-def levenshtein_distance(a, b):
-    # type: (unicode, unicode) -> int
+def levenshtein_distance(a: str, b: str) -> int:
     """Return the Levenshtein edit distance between two strings *a* and *b*."""
     if a == b:
         return 0
@@ -141,7 +139,7 @@ def levenshtein_distance(a, b):
         a, b = b, a
     if not a:
         return len(b)
-    previous_row = range(len(b) + 1)
+    previous_row = list(range(len(b) + 1))
     for i, column1 in enumerate(a):
         current_row = [i + 1]
         for j, column2 in enumerate(b):
@@ -149,24 +147,27 @@ def levenshtein_distance(a, b):
             deletions = current_row[j] + 1
             substitutions = previous_row[j] + (column1 != column2)
             current_row.append(min(insertions, deletions, substitutions))
-        previous_row = current_row  # type: ignore
+        previous_row = current_row
     return previous_row[-1]
 
 
 class UIDTransform(SphinxTransform):
     """Add UIDs to doctree for versioning."""
-    default_priority = 100
+    default_priority = 880
 
-    def apply(self):
+    def apply(self, **kwargs: Any) -> None:
         env = self.env
         old_doctree = None
+        if not env.versioning_condition:
+            return
+
         if env.versioning_compare:
             # get old doctree
             try:
-                filename = env.doc2path(env.docname, env.doctreedir, '.doctree')
+                filename = path.join(env.doctreedir, env.docname + '.doctree')
                 with open(filename, 'rb') as f:
                     old_doctree = pickle.load(f)
-            except EnvironmentError:
+            except OSError:
                 pass
 
         # add uids for versioning
@@ -176,7 +177,19 @@ class UIDTransform(SphinxTransform):
             list(merge_doctrees(old_doctree, self.document, env.versioning_condition))
 
 
-def prepare(document):
+def prepare(document: nodes.document) -> None:
     """Simple wrapper for UIDTransform."""
+    warnings.warn('versioning.prepare() is deprecated. Use UIDTransform instead.',
+                  RemovedInSphinx30Warning, stacklevel=2)
     transform = UIDTransform(document)
     transform.apply()
+
+
+def setup(app: "Sphinx") -> Dict[str, Any]:
+    app.add_transform(UIDTransform)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
