@@ -1,19 +1,17 @@
-# -*- coding: utf-8 -*-
 """
     test_build
     ~~~~~~~~~~
 
     Test all builders.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import pickle
 import sys
 from textwrap import dedent
+from unittest import mock
 
-import mock
 import pytest
 from docutils import nodes
 
@@ -31,13 +29,8 @@ def request_session_head(url, **kwargs):
 @pytest.fixture
 def nonascii_srcdir(request, rootdir, sphinx_test_tempdir):
     # If supported, build in a non-ASCII source dir
-    test_name = u'\u65e5\u672c\u8a9e'
+    test_name = '\u65e5\u672c\u8a9e'
     basedir = sphinx_test_tempdir / request.node.originalname
-    # Windows with versions prior to 3.2 (I think) doesn't support unicode on system path
-    # so we force a non-unicode path in that case
-    if (sys.platform == "win32" and
-            not (sys.version_info.major >= 3 and sys.version_info.minor >= 2)):
-        return basedir / 'all'
     try:
         srcdir = basedir / test_name
         if not srcdir.exists():
@@ -51,8 +44,8 @@ def nonascii_srcdir(request, rootdir, sphinx_test_tempdir):
             =======================
             """))
 
-        master_doc = srcdir / 'contents.txt'
-        master_doc.write_text(master_doc.text() + dedent(u"""
+        master_doc = srcdir / 'index.txt'
+        master_doc.write_text(master_doc.text() + dedent("""
                               .. toctree::
 
                                  %(test_name)s/%(test_name)s
@@ -64,11 +57,7 @@ def nonascii_srcdir(request, rootdir, sphinx_test_tempdir):
 #       (html, changes, epub, latex, texinfo and manpage)
 @pytest.mark.parametrize(
     "buildername",
-    [
-        # note: no 'html' - if it's ok with dirhtml it's ok with html
-        'dirhtml', 'singlehtml', 'pickle', 'json', 'text', 'htmlhelp', 'qthelp',
-        'applehelp', 'xml', 'pseudoxml', 'linkcheck',
-    ],
+    ['dirhtml', 'singlehtml', 'text', 'xml', 'pseudoxml', 'linkcheck'],
 )
 @mock.patch('sphinx.builders.linkcheck.requests.head',
             side_effect=request_session_head)
@@ -79,12 +68,12 @@ def test_build_all(requests_head, make_app, nonascii_srcdir, buildername):
 
 
 def test_master_doc_not_found(tempdir, make_app):
-    (tempdir / 'conf.py').write_text('master_doc = "index"')
+    (tempdir / 'conf.py').write_text('')
     assert tempdir.listdir() == ['conf.py']
 
     app = make_app('dummy', srcdir=tempdir)
     with pytest.raises(SphinxError):
-        app.builder.build_all()
+        app.builder.build_all()  # no index.rst
 
 
 @pytest.mark.sphinx(buildername='text', testroot='circular')
@@ -93,10 +82,10 @@ def test_circular_toctree(app, status, warning):
     warnings = warning.getvalue()
     assert (
         'circular toctree references detected, ignoring: '
-        'sub <- contents <- sub') in warnings
+        'sub <- index <- sub') in warnings
     assert (
         'circular toctree references detected, ignoring: '
-        'contents <- sub <- contents') in warnings
+        'index <- sub <- index') in warnings
 
 
 @pytest.mark.sphinx(buildername='text', testroot='numbered-circular')
@@ -105,10 +94,10 @@ def test_numbered_circular_toctree(app, status, warning):
     warnings = warning.getvalue()
     assert (
         'circular toctree references detected, ignoring: '
-        'sub <- contents <- sub') in warnings
+        'sub <- index <- sub') in warnings
     assert (
         'circular toctree references detected, ignoring: '
-        'contents <- sub <- contents') in warnings
+        'index <- sub <- index') in warnings
 
 
 @pytest.mark.sphinx(buildername='dummy', testroot='images')
@@ -116,7 +105,7 @@ def test_image_glob(app, status, warning):
     app.builder.build_all()
 
     # index.rst
-    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
+    doctree = app.env.get_doctree('index')
 
     assert isinstance(doctree[0][1], nodes.image)
     assert doctree[0][1]['candidates'] == {'*': 'rimg.png'}
@@ -141,7 +130,7 @@ def test_image_glob(app, status, warning):
     assert doctree[0][4][0]['uri'] == 'img.*'
 
     # subdir/index.rst
-    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
+    doctree = app.env.get_doctree('subdir/index')
 
     assert isinstance(doctree[0][1], nodes.image)
     sub = path('subdir')
