@@ -1,19 +1,18 @@
-# -*- coding: utf-8 -*-
 """
     test_build_gettext
     ~~~~~~~~~~~~~~~~~~
 
     Test the build process with gettext builder with the test root.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
 import gettext
 import os
 import re
-from subprocess import Popen, PIPE
+import subprocess
+from subprocess import CalledProcessError, PIPE
 
 import pytest
 
@@ -32,7 +31,7 @@ def test_build_gettext(app):
     assert (app.outdir / 'subdir.pot').isfile()
 
     # regression test for issue #960
-    catalog = (app.outdir / 'markup.pot').text(encoding='utf-8')
+    catalog = (app.outdir / 'markup.pot').text()
     assert 'msgid "something, something else, something more"' in catalog
 
 
@@ -42,37 +41,32 @@ def test_msgfmt(app):
     (app.outdir / 'en' / 'LC_MESSAGES').makedirs()
     with cd(app.outdir):
         try:
-            p = Popen(['msginit', '--no-translator', '-i', 'markup.pot',
-                       '--locale', 'en_US'],
-                      stdout=PIPE, stderr=PIPE)
+            args = ['msginit', '--no-translator', '-i', 'markup.pot', '--locale', 'en_US']
+            subprocess.run(args, stdout=PIPE, stderr=PIPE, check=True)
         except OSError:
             pytest.skip()  # most likely msginit was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                assert False, 'msginit exited with return code %s' % \
-                    p.returncode
+        except CalledProcessError as exc:
+            print(exc.stdout)
+            print(exc.stderr)
+            assert False, 'msginit exited with return code %s' % exc.returncode
+
         assert (app.outdir / 'en_US.po').isfile(), 'msginit failed'
         try:
-            p = Popen(['msgfmt', 'en_US.po', '-o',
-                       os.path.join('en', 'LC_MESSAGES', 'test_root.mo')],
-                      stdout=PIPE, stderr=PIPE)
+            args = ['msgfmt', 'en_US.po',
+                    '-o', os.path.join('en', 'LC_MESSAGES', 'test_root.mo')]
+            subprocess.run(args, stdout=PIPE, stderr=PIPE, check=True)
         except OSError:
             pytest.skip()  # most likely msgfmt was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                assert False, 'msgfmt exited with return code %s' % \
-                    p.returncode
+        except CalledProcessError as exc:
+            print(exc.stdout)
+            print(exc.stderr)
+            assert False, 'msgfmt exited with return code %s' % exc.returncode
+
         mo = app.outdir / 'en' / 'LC_MESSAGES' / 'test_root.mo'
         assert mo.isfile(), 'msgfmt failed'
 
     _ = gettext.translation('test_root', app.outdir, languages=['en']).gettext
-    assert _("Testing various markup") == u"Testing various markup"
+    assert _("Testing various markup") == "Testing various markup"
 
 
 @pytest.mark.sphinx(
@@ -90,7 +84,7 @@ def test_gettext_index_entries(app):
             return m.groups()[0]
         return None
 
-    pot = (app.outdir / 'index_entries.pot').text(encoding='utf-8')
+    pot = (app.outdir / 'index_entries.pot').text()
     msgids = [_f for _f in map(msgid_getter, pot.splitlines()) if _f]
 
     expected_msgids = [
@@ -139,7 +133,7 @@ def test_gettext_disable_index_entries(app):
             return m.groups()[0]
         return None
 
-    pot = (app.outdir / 'index_entries.pot').text(encoding='utf-8')
+    pot = (app.outdir / 'index_entries.pot').text()
     msgids = [_f for _f in map(msgid_getter, pot.splitlines()) if _f]
 
     expected_msgids = [
@@ -162,7 +156,7 @@ def test_gettext_template(app):
     app.builder.build_all()
     assert (app.outdir / 'sphinx.pot').isfile()
 
-    result = (app.outdir / 'sphinx.pot').text(encoding='utf-8')
+    result = (app.outdir / 'sphinx.pot').text()
     assert "Welcome" in result
     assert "Sphinx %(version)s" in result
 
@@ -172,7 +166,7 @@ def test_gettext_template_msgid_order_in_sphinxpot(app):
     app.builder.build_all()
     assert (app.outdir / 'sphinx.pot').isfile()
 
-    result = (app.outdir / 'sphinx.pot').text(encoding='utf-8')
+    result = (app.outdir / 'sphinx.pot').text()
     assert re.search(
         ('msgid "Template 1".*'
          'msgid "This is Template 1\\.".*'
