@@ -1,17 +1,17 @@
-# -*- coding: utf-8 -*-
 """
     test_domain_js
     ~~~~~~~~~~~~~~
 
     Tests the JavaScript Domain
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from unittest.mock import Mock
+
 import pytest
 from docutils import nodes
-from mock import Mock
 
 from sphinx import addnodes
 from sphinx.domains.javascript import JavaScriptDomain
@@ -39,25 +39,20 @@ def test_domain_js_xrefs(app, status, warning):
 
     doctree = app.env.get_doctree('roles')
     refnodes = list(doctree.traverse(addnodes.pending_xref))
-    assert_refnode(refnodes[0], None, None, u'TopLevel', u'class')
-    assert_refnode(refnodes[1], None, None, u'top_level', u'func')
-    assert_refnode(refnodes[2], None, u'NestedParentA', u'child_1', u'func')
-    assert_refnode(refnodes[3], None, u'NestedParentA',
-                   u'NestedChildA.subchild_2', u'func')
-    assert_refnode(refnodes[4], None, u'NestedParentA', u'child_2', u'func')
-    assert_refnode(refnodes[5], False, u'NestedParentA', u'any_child', domain='')
-    assert_refnode(refnodes[6], None, u'NestedParentA', u'NestedChildA', u'class')
-    assert_refnode(refnodes[7], None, u'NestedParentA.NestedChildA',
-                   u'subchild_2', u'func')
-    assert_refnode(refnodes[8], None, u'NestedParentA.NestedChildA',
-                   u'NestedParentA.child_1', u'func')
-    assert_refnode(refnodes[9], None, u'NestedParentA',
-                   u'NestedChildA.subchild_1', u'func')
-    assert_refnode(refnodes[10], None, u'NestedParentB', u'child_1', u'func')
-    assert_refnode(refnodes[11], None, u'NestedParentB', u'NestedParentB',
-                   u'class')
-    assert_refnode(refnodes[12], None, None, u'NestedParentA.NestedChildA',
-                   u'class')
+    assert_refnode(refnodes[0], None, None, 'TopLevel', 'class')
+    assert_refnode(refnodes[1], None, None, 'top_level', 'func')
+    assert_refnode(refnodes[2], None, 'NestedParentA', 'child_1', 'func')
+    assert_refnode(refnodes[3], None, 'NestedParentA', 'NestedChildA.subchild_2', 'func')
+    assert_refnode(refnodes[4], None, 'NestedParentA', 'child_2', 'func')
+    assert_refnode(refnodes[5], False, 'NestedParentA', 'any_child', domain='')
+    assert_refnode(refnodes[6], None, 'NestedParentA', 'NestedChildA', 'class')
+    assert_refnode(refnodes[7], None, 'NestedParentA.NestedChildA', 'subchild_2', 'func')
+    assert_refnode(refnodes[8], None, 'NestedParentA.NestedChildA',
+                   'NestedParentA.child_1', 'func')
+    assert_refnode(refnodes[9], None, 'NestedParentA', 'NestedChildA.subchild_1', 'func')
+    assert_refnode(refnodes[10], None, 'NestedParentB', 'child_1', 'func')
+    assert_refnode(refnodes[11], None, 'NestedParentB', 'NestedParentB', 'class')
+    assert_refnode(refnodes[12], None, None, 'NestedParentA.NestedChildA', 'class')
     assert len(refnodes) == 13
 
     doctree = app.env.get_doctree('module')
@@ -118,24 +113,23 @@ def test_domain_js_find_obj(app, status, warning):
 
     app.builder.build_all()
 
-    assert (find_obj(None, None, u'NONEXISTANT', u'class') ==
-            (None, None))
-    assert (find_obj(None, None, u'NestedParentA', u'class') ==
-            (u'NestedParentA', (u'roles', u'class')))
-    assert (find_obj(None, None, u'NestedParentA.NestedChildA', u'class') ==
-            (u'NestedParentA.NestedChildA', (u'roles', u'class')))
-    assert (find_obj(None, 'NestedParentA', u'NestedChildA', u'class') ==
-            (u'NestedParentA.NestedChildA', (u'roles', u'class')))
-    assert (find_obj(None, None, u'NestedParentA.NestedChildA.subchild_1', u'func') ==
-            (u'NestedParentA.NestedChildA.subchild_1', (u'roles', u'function')))
-    assert (find_obj(None, u'NestedParentA', u'NestedChildA.subchild_1', u'func') ==
-            (u'NestedParentA.NestedChildA.subchild_1', (u'roles', u'function')))
-    assert (find_obj(None, u'NestedParentA.NestedChildA', u'subchild_1', u'func') ==
-            (u'NestedParentA.NestedChildA.subchild_1', (u'roles', u'function')))
-    assert (find_obj(u'module_a.submodule', u'ModTopLevel', u'mod_child_2', u'meth') ==
-            (u'module_a.submodule.ModTopLevel.mod_child_2', (u'module', u'method')))
-    assert (find_obj(u'module_b.submodule', u'ModTopLevel', u'module_a.submodule', u'mod') ==
-            (u'module_a.submodule', (u'module', u'module')))
+    assert (find_obj(None, None, 'NONEXISTANT', 'class') == (None, None))
+    assert (find_obj(None, None, 'NestedParentA', 'class') ==
+            ('NestedParentA', ('roles', 'class')))
+    assert (find_obj(None, None, 'NestedParentA.NestedChildA', 'class') ==
+            ('NestedParentA.NestedChildA', ('roles', 'class')))
+    assert (find_obj(None, 'NestedParentA', 'NestedChildA', 'class') ==
+            ('NestedParentA.NestedChildA', ('roles', 'class')))
+    assert (find_obj(None, None, 'NestedParentA.NestedChildA.subchild_1', 'func') ==
+            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+    assert (find_obj(None, 'NestedParentA', 'NestedChildA.subchild_1', 'func') ==
+            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+    assert (find_obj(None, 'NestedParentA.NestedChildA', 'subchild_1', 'func') ==
+            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+    assert (find_obj('module_a.submodule', 'ModTopLevel', 'mod_child_2', 'meth') ==
+            ('module_a.submodule.ModTopLevel.mod_child_2', ('module', 'method')))
+    assert (find_obj('module_b.submodule', 'ModTopLevel', 'module_a.submodule', 'mod') ==
+            ('module_a.submodule', ('module', 'module')))
 
 
 def test_get_full_qualified_name():
