@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     test_coverage
     ~~~~~~~~~~~~~
 
     Test the coverage builder.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -38,7 +37,7 @@ def test_build(app, status, warning):
     undoc_py, undoc_c = pickle.loads((app.outdir / 'undoc.pickle').bytes())
     assert len(undoc_c) == 1
     # the key is the full path to the header file, which isn't testable
-    assert list(undoc_c.values())[0] == set([('function', 'Py_SphinxTest')])
+    assert list(undoc_c.values())[0] == {('function', 'Py_SphinxTest')}
 
     assert 'autodoc_target' in undoc_py
     assert 'funcs' in undoc_py['autodoc_target']
@@ -46,3 +45,22 @@ def test_build(app, status, warning):
     assert 'classes' in undoc_py['autodoc_target']
     assert 'Class' in undoc_py['autodoc_target']['classes']
     assert 'undocmeth' in undoc_py['autodoc_target']['classes']['Class']
+
+
+@pytest.mark.sphinx('coverage', testroot='ext-coverage')
+def test_coverage_ignore_pyobjects(app, status, warning):
+    app.builder.build_all()
+    actual = (app.outdir / 'python.txt').text()
+    expected = '''Undocumented Python objects
+===========================
+coverage_not_ignored
+--------------------
+Classes:
+ * Documented -- missing methods:
+
+   - not_ignored1
+   - not_ignored2
+ * NotIgnored
+
+'''
+    assert actual == expected
