@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     test_ext_imgconverter
     ~~~~~~~~~~~~~~~~~~~~~
 
     Test sphinx.ext.imgconverter extension.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -19,7 +18,7 @@ import pytest
 def test_ext_imgconverter(app, status, warning):
     app.builder.build_all()
 
-    content = (app.outdir / 'Python.tex').text()
+    content = (app.outdir / 'python.tex').text()
     assert '\\sphinxincludegraphics{{svgimg}.png}' in content
     assert not (app.outdir / 'svgimg.svg').exists()
     assert (app.outdir / 'svgimg.png').exists()
