@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     test_napoleon_iterators
     ~~~~~~~~~~~~~~~~~~~~~~~
@@ -6,7 +5,7 @@
     Tests for :mod:`sphinx.ext.napoleon.iterators` module.
 
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -341,7 +340,7 @@ class ModifyIterTest(BaseIteratorsTest):
         self.assertEqual(expected, [i for i in it])
 
     def test_modifier_rstrip_unicode(self):
-        a = [u'', u'  ', u'  a  ', u'b  ', u'  c', u'  ', u'']
+        a = ['', '  ', '  a  ', 'b  ', '  c', '  ', '']
         it = modify_iter(a, modifier=lambda s: s.rstrip())
-        expected = [u'', u'', u'  a', u'b', u'  c', u'', u'']
+        expected = ['', '', '  a', 'b', '  c', '', '']
         self.assertEqual(expected, [i for i in it])
