@@ -1,15 +1,15 @@
-# -*- coding: utf-8 -*-
 """
     test_highlighting
     ~~~~~~~~~~~~~~~~~
 
     Test the Pygments highlighting bridge.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import mock
+from unittest import mock
+
 from pygments.formatters.html import HtmlFormatter
 from pygments.lexer import RegexLexer
 from pygments.token import Text, Name
