@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     test_toctree
     ~~~~~~~~~~~~
 
     Test the HTML builder and check output against XPath.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import re
@@ -44,5 +43,5 @@ def test_numbered_toctree(app, status, warning):
     # give argument to :numbered: option
     index = (app.srcdir / 'index.rst').text()
     index = re.sub(':numbered:.*', ':numbered: 1', index)
-    (app.srcdir / 'index.rst').write_text(index, encoding='utf-8')
+    (app.srcdir / 'index.rst').write_text(index)
     app.builder.build_all()
